﻿using System;
using System.Collections;
using System.Text;
using OhymosNet;

namespace OhymosNet
{
    /// <summary>
    /// DataConnectionTypeに関するオプションのクラスです。
    ///
    /// </summary>
    
    [Serializable]
    public sealed class DataConnectionTypeOption
    {
        /// <summary> <japanese>
        /// 型に安全な定数の個数である 4 を取得します。
        /// 
        /// 
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> an <var>int</var> value
        /// </returns>
        /// 

        public static int NumOfDCT
        {
            get
            {
                Array dct = Enum.GetValues(typeof(DataConnectionType));
                return dct.Length;
            }
        }

        /// <summary> <japanese>
        /// 引数の値にしたがって以下のように DataConnectionType 型を返します。
        /// 
        /// <ul>
        /// <li> 0 : BROKENLINE</li>
        /// <li> 1 : STEP</li>
        /// <li> 2 : PULSE</li>
        /// <li> 3 : NOINTP</li>
        /// </ul>
        /// 
        /// これ以外の int 値の場合は、
        /// 例外 ArrayIndexOutOfBoundsException が投げられます。
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="k">an <var>int</var> value
        /// </param>
        /// <returns> a <var>DataConnectionType</var> value
        /// </returns>
        public static DataConnectionType getDctType(int k)
        {
            return (DataConnectionType)Enum.ToObject(typeof(DataConnectionType), k);
        }


        /// <summary> <japanese>
        /// 引数の値にしたがって以下のようにデータ補間型を表す文字列を返します。
        /// 
        /// <ul>
        /// <li> 0 : BROKENLINE</li>
        /// <li> 1 : STEP</li>
        /// <li> 2 : PULSE</li>
        /// <li> 3 : NOINTP</li>
        /// </ul>
        /// 
        /// これ以外の int 値の場合は、
        /// 例外 ArgumentNullException が投げられます。
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="k">an <var>int</var> value
        /// </param>
        /// <returns> a <var>String</var> value
        /// </returns>
        public static string getDctTypeStr(int k)
        {
            return Enum.GetName(typeof(DataConnectionType), k);
        }

        /// <summary> 
        ///  文字列 aStr からデータ補間型配列を読み取り、戻り値として返
        /// します。
        /// <p/>
        /// Converts the String argument <b>aStr</b> into the
        /// (Ref "DataConnectionType") vector and returns it.
        /// </summary>
        /// <remarks>
        /// For example, String " 2 1 0" is converted into <p/>
        /// ArrayList[2] = { DataConnectionType.STEP, DataConnectionType.BROKENLINE }
        /// </remarks>
        /// 
        /// <param name="aStr">a <var>String</var> value
        /// </param>
        /// <returns> a <var>ArrayList</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public static ArrayList parseDctVec(String aStr)
        {
            int n = 0; // The size of the vector.

            ArrayList dctVec = ArrayList.Synchronized(new ArrayList());

            Tokenizer st = new Tokenizer(aStr);

            try
            {
                n = Int32.Parse(st.NextToken());
            }
            catch (FormatException e)
            {
                throw e;
            }

            if (n < 0)
                throw new FormatException("negative vector size");

            for (int i = 0; i < n; i++)
            {
                int k = Int32.Parse(st.NextToken());
                if (0 <= k && k < DataConnectionTypeOption.NumOfDCT)
                    dctVec.Add(DataConnectionTypeOption.getDctType(k));
                else
                {
                    throw new FormatException("invalid DataConnectionType number: " + k);
                }
            }

            return dctVec;
        }
    
    } // DataConnectionTypeOption
}
