/// <(#)>  OhDataPack; class to express data pack. </(#)>
/// <summary>                  Make "OhDataPack.java" from "ohdatapack.h" by Satoh.
/// </summary> Modified and Tested by Naoto Ohshima. Date:2007/10/05 20:13:55
///            Last update : 2007/11/03 16:37:49
using System;
using System.IO;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
	/// {^f[^f\NXłB<p/>
    /// abstract class for treating a pair of data time and data value which we call datapack. 
    /// </summary>
    /// <remarks>
	/// f[^ƃf[^̃yA琬f[^pbN^쐬邽߂̒ۃf[^^fłB<p/>
	/// <p>
	/// ̃NX OhDataPack ł́A
	/// 
	/// <dl>
	/// <li> f[^L^ : _dataTime</li>
	/// </dl>
	/// LA
	/// 
	/// <dl>
	/// <li> f[^L^̐ݒ@\</li>
	/// <li> f[^L^̎擾@\</li>
	/// </dl>
	/// `܂B</p>
	/// 
	/// ef󂷂f[^̃f́ÃNX OhDataPack pč쐬ĉB
	/// 
    /// <pre><code>
    /// using OhymosNet;
    /// using System;
    /// using System.IO;
    /// 
    /// [Serializable]
    /// public class OhDataPackFoo :OhDataPack, ICloneable
    /// {
    ///     private final Foo _dataValue;
    /// 
    ///     // constructors
    ///     public OhDataPackFoo() { ... }
    ///     public OhDataPackFoo(OhTimePointImmutable aDataTime, Foo aDataValue) { ... }
    ///     public OhDataPackFoo(String aDataStr) { ... }
    /// 
    ///     // abstract methods in OhDataPack
    ///     public String dataType() { ... }
    ///     public void dpack2Ofile(SrintWriter sw) { ... }
    ///     public void dpack2File(SrintWriter sw) { ... }
    /// 
    ///     // nonabstract methods
    ///     public Object clone() { ... }
    ///     public Foo getDataValue() { ... }
    /// }
    /// </code></pre>
	/// 
	/// <p/>
	/// 
    /// OhDataPack;<p/>
	/// 
	/// 
	/// In hydrologic simulations in which the states are updated dynamically,
	/// the data which are sent and received by elements, subsytems, and total sytem of the hydrologic system
	/// are pairs of data value(s) and a "data time".
	/// Here, "data time" means the time to which the data are related. <p/>
	/// 
	/// For example, suppose we have the value of discharge Q at time t at some stream point.
	/// Then, we call the value of discharge Q "data value" and time t "data time". <p/>
	/// 
	/// We call a pair of data time and data value "datapack," and we prepare class OhDataPack,
	/// which is an abstract class used as a base class from which various datapack classes are derived.
	/// 
    /// 
	/// <pre>
	/// .....
	/// "datapack" = "data time" + "data value"
	/// ....
	/// </pre>
	/// 
	/// Because there are various data types for data value, there could be various datapack classes. <p/>
	/// 
    /// Class OhDataPack is a base class from which various datapack
    /// classes are derived.  Note that the classed derived from the
    /// OhDataPack class should be immutable.
    /// 
    /// </remarks>
	/// 
	/// <author>  <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
	/// </author>
	/// <version>  $Revision: 1.20 $ $Date: 2004/03/08 17:03:02 $
	/// </version>
    [Serializable]
	public abstract class OhDataPack:ICloneable
	{

        // @begin{fields} -------------------------------------------------------

        /// <summary> <japanese>
        /// f[^L^\ OhTimePoint IuWFNgłB
        /// </japanese>
        /// <p/><br/>
        /// The time when the data was recorded.
        /// It is of type <a href="OhTimePoint.html">OhTimePoint</a>.
        /// 
        /// </summary>
        [NonSerialized]
        protected internal OhTimePoint _dataTime;

        /// <summary> <japanese>
        /// OS ɈˑȂsR[h\łB
        /// </japanese>
        /// <p/><br/>
        /// New Line.
        /// 
        /// </summary>
        protected internal static String NEWLINE = Environment.NewLine;

        // @end{fields} ---------------------------------------------------------

        // @begin{abstract method} ---------------------------------------------

        /// <summary> <japanese>
        /// f[^̌^擾܂B
        /// ۃvpeBłB<p/>
        /// 
        /// Ⴆ΁AMf[^ int ^Ȃ "int" Ԃ悤ɒ`܂B
        /// </japanese>
        /// <p/><br/>
        /// This property gets the data type.
        /// <br/>
        ///  
        /// 
        /// </summary>
        /// <returns> a <var>String</var> value
        /// </returns>
        public abstract String DataType { get;}


        // data transformation.

        /// <summary> <japanese>
        /// f[^pbNptH[}bgŃt@Cɏo܂B<p/>
        /// 
        /// ۃ\bhłB
        /// </japanese>
        /// <p/><br/>
        /// A data pack is written out to a file by the format for internal processing.
        /// 
        /// </summary>
        /// <param name="sw">a <var>PrintWriter</var> value
        /// </param>
        public abstract void dpack2File(StreamWriter sw);

        /// <summary> <japanese>
        /// f[^pbNȈՃtH[}bgŃt@Cɏo܂B<p/>
        /// 
        /// ۃ\bhłB
        /// </japanese>
        /// <p/><br/>
        /// A data pack is written out to a file by simple format.
        /// 
        /// </summary>
        /// <param name="sw">a <var>PrintWriter</var> value
        /// </param>
        public abstract void dpack2Ofile(StreamWriter sw);

        /// <summary>
        /// N[𐶐܂B
        /// </summary>
        /// <returns></returns>
        public abstract Object Clone();

        // @end{abstract method} -----------------------------------------------
		
        // @begin{constructor} -------------------------------------------------

        /// <summary> 
        /// <p>
        /// ftHgRXgN^ OhDataPack() ́A
        /// oϐ _dataTime  <a href="OhTimePoint.html">OhTimePoint</a> ɏ܂B
        /// 
        /// </p><br/>
        /// OhDataPack() initializes _dataTime to OhTimePoint().
        /// 
        /// </summary>
        public OhDataPack()
        {
            _dataTime = new OhTimePoint();
        }

        // @end{constructor} ---------------------------------------------------
		

		
        // begin{property}-------------------------------------------------------
        /// <summary>
		/// _dataTime̎擾Eݒs܂B
        /// <ul>
        /// <li>get : a <var>OhTimePoint</var> value </li>
        /// <li>set : a <var>OhTimePoint</var> value </li>
        /// </ul>
		/// </summary>
		public IOhTimePointImmutable DataTime
		{
			get
			{
				return _dataTime;
			}
			
			set
			{
				_dataTime.copy(value);
			}
		}

        // end{property}---------------------------------------------------------
		
		
		
		
		
		// @begin{methods} -----------------------------------------------------
		
		/// <summary> <japanese>
		/// Rs[\bhłB
		/// 
		/// aDp Ɠ _dataTime  OhDataPack 쐬܂B
		/// </japanese>
        /// <p/>   <br/>
		/// Copy method.
		/// 
		/// </summary>
		/// <param name="aDp">an <var>OhDataPack</var> value
		/// </param>
        //public virtual void  copy(OhDataPack aDp)
        //{
        //    _dataTime.copy(aDp._dataTime);
        //}
		
		
		// setting and getting data recording times
	} // OhDataPack
}