/// <(#)>  OhDataPackFloat; class to express datapacks of type float. </(#)>
/// <summary>                     Made from "OhDataPackFloat.java" by Ohshima.
/// </summary> Modified and Tested by Naoto Ohshima. Date:2007/10/09 13:44:36
///            Last update : 2007/11/03 18:05:10
using System;
using System.IO;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> <japanese>
    /// f[^l float ^̃f[^pbN\NXłB
    /// 
    /// ̃NX <a href="OhDataPack.html">OhDataPack</a> pĂ܂B
    /// </japanese>
    /// <p/>
    /// OhDataPackFloat;<br/>
    /// 
    /// OhDataPackFloat is a class of datapacks of type float. <p/>
    /// 
    /// It extends <a href="OhymosNet.OhDataPack.html">OhDataPack</a> class. <p/>
    /// </summary>
    /// <remarks>
    /// float^̃f[^pbNƂ́Af[^L^float^̃f[^lƂ̑ĝƂ܂B<p/>
    /// OhTimePoint^̃f[^L^float^̃f[^lA܂̓f[^pbN񂩂
    /// OhDataPack^̃IuWFNg𐶐Af[^pbNƂ܂B<p/>
    /// 
    /// A datapack of type float is a pair of data of data time and
    /// data of type float.<p/>
    /// </remarks>
    public class OhDataPackFloat : OhDataPack, ICloneable
    {

        // @begin{field} -----------------------------------------------------

        /// <summary> <japanese>
        /// float ^̃f[^i[郁oϐłB
        /// </japanese>
        /// </summary>
        private float _dataValue;

        // @end{field} -------------------------------------------------------


        // @begin{constructors} ----------------------------------------------

        /// <summary> <japanese>
        /// ftHgRXgN^łB
        /// </japanese>
        /// 
        /// </summary>
        public OhDataPackFloat()
        {
            _dataValue = 0;
        }

        /// <summary> <japanese>
        /// f[^l aDataValueAf[^L^ aDataTime  OhDataPackFloat 쐬܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aDataTime">an <var>OhTimePoint</var> value
        /// </param>
        /// <param name="aDataValue">a <var>float</var> value
        /// </param>
        public OhDataPackFloat(IOhTimePointImmutable aDataTime, float aDataValue)
        {
            _dataTime.copy(aDataTime);
            _dataValue = aDataValue;
        }

        /// <summary> <japanese>
        /// _dataStr Ɋi[Ăf[^pbN񂩂f[^pbN̒lǂݎA
        /// _dataTime, _dataValue Ɋi[ꂽ OhDataPackFloat 𐶐܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aDataStr">a <var>String</var> value<br/>
        /// f[^pbN̕łB
        /// </param>
        public OhDataPackFloat(String aDataStr)
        {
            OhStringReader sr;
            float dataValue = 0;
            try
            {
                sr = new OhStringReader(aDataStr);

                _dataTime.Year = Int32.Parse(sr.readWord());
                _dataTime.Tsec = Int64.Parse(sr.readWord());
                _dataTime.Frac = Double.Parse(sr.readWord());
                dataValue = Single.Parse(sr.readWord());
            }
            catch (IOException e)
            {
                Console.WriteLine(e.StackTrace);
            }
            catch (FormatException e)
            {
                Console.WriteLine(e.StackTrace);
            }
            _dataValue = dataValue;
        }

        // @end{constructors} ------------------------------------------------  

        // @begin{properties} --------------------------------------------------  
        /// <summary> <japanese>
        /// _dataValue 擾܂B
        /// </japanese>
        /// <br/>
        /// Gets the data value.
        /// <ul>
        /// <li>get : a <var>float</var> value </li>
        /// </ul>
        /// </summary>
        ///
        public float DataValue
        {
            get
            {
                return _dataValue;
            }
        }

        /// <summary> <japanese>
        /// f[^ľ^ float 𕶎 "float" ƂĎ擾܂B
        /// </japanese>
        /// <p/>
        /// Returns the data type "float".
        /// <ul>
        /// <li>get : a <var>string</var> "float" </li>
        /// </ul>
        /// </summary>
        /// 
        public override String DataType
        {
            get { return "float"; }
        }

        // @end{property} ----------------------------------------------------  

        // @begin{implemention of abstract methods} ---------------------------------------------------  

        // 0. copy method.

        /// <summary> <japanese>
        /// Rs[\bhłB
        /// 
        /// aDpi Ɠ _dataTime, _dataValue  OhDataPackFloat 쐬܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aDpi">an <var>OhDataPackFloat</var> value
        /// </param>
        //public void copy(OhDataPackFloat aDpi)
        //{
        //    _dataTime.copy(aDpi._dataTime);
        //    _dataValue = aDpi._dataValue;
        //}

        /// <summary> <japanese>
        /// f[^lƃf[^L^擾܂B
        /// f[^l float ^z aDataValue ̑vfɊi[܂B
        /// </japanese>
        /// <p/>
        /// Returns the data time and data value.
        /// 
        /// </summary>
        /// <param name="aDataTime">an <var>OhTimePoint</var> value
        /// </param>
        /// <param name="aDataValue">a <var>float[]</var> value
        /// </param>
        //public void getData(OhTimePoint aDataTime, float[] aDataValue)
        //{
        //    aDataTime.copy(_dataTime);
        //    aDataValue[0] = _dataValue;
        //}

        /// <summary> <japanese>
        /// _dataValue, _dataTime ݒ肵܂B
        /// </japanese>
        /// <p/>
        /// Sets the data time and data value.
        /// 
        /// </summary>
        /// <param name="aDataTime">an <var>OhTimePoint</var> value
        /// </param>
        /// <param name="aDataValue">a <var>float</var> value
        /// </param>
        //public void setData(OhTimePoint aDataTime, float aDataValue)
        //{
        //    _dataTime = aDataTime; //; "setDataTime(aDataTime);"
        //    _dataValue = aDataValue;
        //}

        /// <summary> <japanese>
        /// f[^pbNȈՃtH[}bgŃt@Cɏo܂B
        /// 
        /// ȈՃtH[}bgƂ́A_year, _tsec, _dataValue 󔒂ŋ؂Ĉsɏo܂B
        /// 
        /// <pre>
        /// [EXAMPLE]  2003 38600 3
        /// </pre>
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="sw">a <var>StreamWriter</var> value
        /// </param>
        public override void dpack2Ofile(StreamWriter sw)
        {
            sw.Write(_dataTime.Year + " " + _dataTime.Tsec + " " + _dataTime.Frac + " " + DataValue);
        }

        /// <summary> <japanese>
        /// f[^pbNptH[}bgŃt@Cɏo܂B
        /// 
        /// ptH[}bgƂ́A_year, _tsec, _dataValue ̂ꂼijʂł̂
        /// ЂƂsďo܂B
        /// 
        /// <pre>
        /// [EXAMPLE]  (2003)
        /// (38600)
        /// (3)
        /// </pre>
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="sw">a <var>StreamWriter</var> value
        /// </param>
        public override void dpack2File(StreamWriter sw)
        {
            sw.Write("(" + _dataTime.Year + ")" + NEWLINE);
            sw.Write("(" + _dataTime.Tsec + ")" + NEWLINE);
            sw.Write("(" + _dataTime.Frac + ")" + NEWLINE);
            sw.Write("(" + _dataValue + ")" + NEWLINE);
        }

        // @end{implemention of abstract methods} -----------------------------------------------------

        // @begin{methods} -------------------------------------------

        /// <summary> Returns a deep copy of the current OhDataPackFloat object<p/>
        /// 
        /// <japanese>
        /// ݂ OhDataPackFloat IuWFNg̃N[ ([Rs[) 
        /// Ԃ܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> an <var>Object</var> value
        /// </returns>
        public override Object Clone()
        {
            OhDataPackFloat dp = new OhDataPackFloat(_dataTime, _dataValue);
            return dp;
        }


        /// <summary> Multiplies the parameter aFactor to the instance variable
        /// _dataValue.<p/>
        /// 
        /// <japanese>
        ///  aFactor ̒lCX^Xϐ _dataValue Ɋ|܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aFactor">a <var>double</var> value
        /// </param>
        public virtual void multiply(double aFactor)
        {
            _dataValue *= (float)aFactor;
        }

        /// <summary> Returns the string expression of this object
        /// 
        /// </summary>
        /// <returns> a <var>String</var> value
        /// </returns>
        public override String ToString()
        {
            return _dataTime.ToString() + " " + _dataValue;
        }

    } // OhDataPackFloat
}