/// <(#)>  OhElement; base class of element models. </(#)>
/// <summary> 
/// </summary>
using System;
using System.IO;
using System.Text;
using OhymosNet;

namespace OhymosNet
{

    /// <summary>
    /// {^vff\NXłB<p/>
    /// ۃNXłB 
    /// ̃NX<a href ="OhymosNet.OhObject.html">OhObject</a>
    /// pĂ܂B<p/>
    /// 
    /// OhElement;<p/>
    /// Abstract class for element models.
    /// This class It extends <a href ="OhymosNet.OhObject.html">OhObject</a> class.
    /// </summary>
    /// <remarks>
    /// vff́Ap^, ԗ, M[q, M[qf[^oɋLA
    /// p^EԗʁEM[q̓̓f[^pĐvZsȂAԗʂXVA
    /// M[qo̓f[^𑗐M܂B
    /// vZƂ̂AevffɋʂȍƂƋʂłȂƂƂ𕪗A
    /// ʕ̂ݒoEKî{^vffłB
    /// ̃NX́A{^vff\NX`Ă܂B<p/>
    /// 
    /// vffLSM[q, SM[q́A
    /// _receivePortArray, _sendPortArray ɓo^܂B
    /// {^vffł́A̓o^葱ɊւKisȂA
    /// vff̍쐬Ƃ̊ȑfi߂ƂƂɁA
    /// evffɓo^葱Ɋւ铯̐M^Ă܂B
    /// ܂Aevf̑M[q̐ڑ͕nfsȂ܂A
    /// ̍ƂeՂɂ邽߁AvffL鑗M[qɊւ錟@\`܂B<p/>
    /// 
    /// ʏ́Avff̏ԂXV邽߂ɕKvȃf[^͎M[qoĎ擾悤ɐ݌v܂A
    /// ނ𓾂Ȃꍇ́AuڒʐMvɂĕKvȃf[^擾dg݂[U`ł悤ɁA
    /// ڒʐMvf̔zL邽߂̃oϐ _objectArray pӂĂ܂B
    /// 
    /// 
    /// <p/>
    /// 
    /// 
    /// 
    /// An element model
    /// 
    /// <ul>
    /// <li> has data members for storing parameters, state variables, receive ports, send port,</li>
    /// <li> calculates the time change of the state variables using input data obtained through the receive ports, and</li>
    /// <li> sends output data through the send ports.</li>
    /// </ul>
    /// 
    /// In hydrolgic simulations, various element models are needed because various processes are involved. <p/>
    /// 
    /// Class OhElement expresses the common part of those hydrologic element models.
    /// </remarks>
    /// <author>  <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
    /// </author>
    /// <version>  $Revision: 1.145 $ $Date: 2004/03/08 17:03:02 $
    /// </version>
    public abstract class OhElement : OhObject
    {
        // @begin {fields} -----------------------------------------------------

        /// <summary> 
        /// <a href="OhElement.html#initialWork()">OhElement.initialWork()</a> 
        /// I(true)ۂ(false)\tOłB
        /// 
        /// <p/>
        /// Whether the initial work has been completed or not.
        /// 
        /// </summary>
        protected internal bool _initialWorkDoneP;

        /// <summary> 
        /// <a href="OhElement.html#doShareInfo()">OhElement.doShareInfo()</a>Share_info 
        /// I(true)ۂ(false)\tOłB
        /// 
        /// <p/>
        /// Whether the "shareInfo" has been completed or not.
        /// 
        /// </summary>
        protected internal bool _shareInfoDoneP;

        /// <summary> 
        /// Ɨp̐`XgłB
        /// _receivePortArray, _sendPortArray ̍쐬Ɏgp܂B
        /// 
        /// <p/>
        /// A list for storing Object.
        /// It is used as an object for work at the time of creation of "_receivePortArray" and "_sendPortArray".
        /// 
        /// </summary>
        protected internal OhLinkedList _objectList;

        /// <summary> 
        /// vfLM[q̔złB
        /// 
        /// <p/>
        /// Array of "OhReceivePort".
        /// 
        /// </summary>
        protected internal Object[] _receivePortArray;

        /// <summary> 
        /// _receivePortArray ̗vfłB
        /// 
        /// <p/>
        /// Number of receive ports.
        /// 
        /// </summary>
        protected internal int _numberOfReceivePorts;

        /// <summary> 
        /// vfL鑗M[q̔złB
        /// 
        /// <p/>
        /// Array of "OhSendPort".
        /// 
        /// </summary>
        protected internal Object[] _sendPortArray;

        /// <summary> 
        /// _sendPortArray ̗vfłB
        /// 
        /// <p/>
        /// Number of send ports.
        /// 
        /// </summary>
        protected internal int _numberOfSendPorts;

        /// <summary> 
        /// ڒʐMvf̔złB
        /// 
        /// <p/>
        /// Array of "OhObject".
        /// 
        /// </summary>
        protected internal OhObject[] _objectArray;

        /// <summary> 
        /// _objectArray ̗vfłB
        /// 
        /// <p/>
        /// Number of objects.
        /// 
        /// </summary>
        protected internal int _numberOfObjects;

        /// <summary> 
        /// f[^łB
        /// 
        /// <p/>
        /// Data string.
        /// 
        /// </summary>
        protected internal String _dataString;

        /// <summary> 
        /// ݎ(vZςݎ)łB
        /// 
        /// <p/>
        /// Current(Calculated time) time.
        /// "OhTimePoint" is defined in "OhTimePoint.java".
        /// 
        /// </summary>
        protected internal OhTimePoint _currentTime;

        /// <summary> 
        /// vZPXebv̎(^CXebv)[sec]łB
        /// 
        /// <p/>
        /// Time step.[sec]
        /// 
        /// </summary>
        protected internal OhTime _timeStep;


        /// <summary> 
        /// vZɂďԂm肵Ă鎞łB
        /// 
        /// <p/>
        /// Time which the state has decided in repeated calculation.
        /// 
        /// </summary>
        protected internal OhTimePoint _fixedTime;

        /// <summary> 
        /// vZڕW[sec]łB<p/>
        /// Repeated calculation target time. [sec] <p/>
        /// </summary>
        /// <remarks>
        /// iӁ)
        /// vZvfł́AvZJn _fixedTime, _targetTime  _currentTime ƈvKv܂B
        /// <a href="OhElement.html#setInitialState(OhStreamReader)">OhElement.setInitialState(OhBufferedReader)</a>
        ///  _currentTime ݒ肳ꂽA
        /// <a href="OhElement.html#setCur2Fix2Tar()">OhElement.setCur2Fix2Tar()</a> ĂԂA
        ///  _targetTime = _fixedTime = _currentTime; ƂĂB
        /// 
        /// <p/>
        /// 
        /// It is necessary to make "_fixedTime" and "_targetTime" in agreement with "_currentTime"
        /// with the element which carries out repeated calculation at the time of a calculation start. <p/>
        /// 
        /// After initial setting of the "_currentTime" is carried out by "setInitialState()",
        /// call "void cur2fix2tar()" which the bottom defines or give as direct
        /// "_targetTime = _fixedTime = _currentTime;".
        /// </remarks>
        protected internal OhTimePoint _targetTime;

        /// <summary> 
        /// vZ悤Ƃ̂ɌvZłȂ̗vfɂƂɁA
        /// _calReqP = true ɐݒ肳悤ɂ܂B
        /// āAOhElement  _calReqP = true ̂Ƃ͌vZ悤ɓw͂܂B
        /// _calReqP ͂̃NXŎn߂ĒǉꂽoϐłB
        /// ̃NX̃IuWFNgƂɂ _calReqP  false ɏ܂B
        /// 
        /// <p/>
        /// Flag of request of calculation.
        /// Initial value is NO.
        /// When a cause incalculable although the lower-stream-of-a-river side tended to calculate is in this element,
        /// it is made to be set as "_calReqP = true".
        /// And, "OhElement" tries to calculate hard at the time of "_calReqP = true".
        /// 
        /// </summary>
        protected internal bool _calReqP;

        // @end {instance variable or static variable} --------------------------------


        // @begin {initialization} ---------------------------------------------

        /// <summary> 
        /// \vf̃NXt@Cǂ݂ƂĂ肠Šnf
        /// ܂͕nfŗp
        /// RXgN^̕⏕\bhłB<p/>
        /// 
        /// Initialization with the information which is given through a file.
        /// </summary>
        /// <remarks>
        /// ̃RXgN^ō쐬ƂɌĂ΂邱Ƃz肵Ă܂B
        /// Ⴆ΁AclassName, objectName, objectNumber t@CǂݎĂƂA
        /// ΏۂƂt@C̃Xg[ br ƂƂA
        /// 
        /// <pre><var>
        /// (OhElement)elm = (OhElement)Activator.CreateInstance("OhymosNet." + className);
        /// elm.init(objectName, objectNum, br);
        /// </var></pre>
        /// 
        /// ̂悤ɂč\vf蓖Ă܂B<p/>
        /// 
        /// \vf̃NXt@Cǂ݂ƂĂ肠镔nf
        /// p悤ɂ邽߂ɂ́A
        /// ̃\bh`ĂKv܂B<p/>
        /// 
        /// ܂OHyMoS.NETŗpӂĂŠnfOhScfTotalSystempꍇ
        /// ̃\bh`ĂKv܂B
        /// 
        /// <example>
        /// </example>
        /// 
        /// init(aObjName, aObjNum, aSr) `̍\vfĂ΂ꂽƂɂ́A
        /// O
        /// <a href="OhError.html">OhError</a>
        /// ܂B
        /// </remarks>
        /// <p/>
        /// 
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void init(String aObjName, int aObjNum, OhStreamReader aSr)
        {
            if (true)
                throw new OhError("This init(aObjName, aObjNum, aSr) method is not defined.");
        }


        /// <summary> 
        /// IuWFNgAIuWFNgԍAڒʐMvf̔zAf[^܂B
        /// ڒʐMvf̔zɊւẮA
        /// aNumOfObjs ̗vfzɊmۂ܂B
        /// 
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value
        /// </param>
        /// <param name="aDataStr">a <var>String</var> value
        /// </param>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void init(String aObjName, int aObjNum, int aNumOfObjs, String aDataStr)
        {
            base.init(aObjName, aObjNum);
            ObjectArray = aNumOfObjs;
            DataString = aDataStr;
        }

        /// <summary> 
        /// IuWFNgAIuWFNgԍAڒʐMvf̔z܂B
        /// </summary>
        /// <remarks>
        /// ڒʐMvf̔zɊւẮA
        /// aNumOfObjs ̗vfzɊmۂ܂B
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value
        /// </param>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void init(String aObjName, int aObjNum, int aNumOfObjs)
        {
            base.init(aObjName, aObjNum);
            ObjectArray = aNumOfObjs;
            DataString = null;
        }

        /// <summary> 
        /// IuWFNgAIuWFNgԍAf[^܂B
        /// 
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aDataStr">a <var>String</var> value
        /// </param>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void init(String aObjName, int aObjNum, String aDataStr)
        {

            base.init(aObjName, aObjNum);
            ObjectArray = 0;
            DataString = aDataStr;
        }

        /// <summary> 
        /// IuWFNgAIuWFNgԍ܂B
        /// 
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void init(String aObjName, int aObjNum)
        {
            base.init(aObjName, aObjNum);
            ObjectArray = 0;
            DataString = null;
        }

        // @end {initialization} -----------------------------------------------


        // @begin {constructors} -----------------------------------------------

        /// <summary> 
        /// ftHgRXgN^łB<p/>
        /// 
        /// ftHgRXgN^
        /// 
        /// <p/>
        /// <dl>
        /// <li> _className         : "OhElement"</li>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// <p/>
        /// 
        /// ɐݒ肵܂B
        /// 
        /// </summary>
        public OhElement()
            : base()
        {
            _initialWorkDoneP = false;
            _shareInfoDoneP = false;
            _objectList = null;
            _receivePortArray = null;
            _numberOfReceivePorts = -1;
            _sendPortArray = null;
            _numberOfSendPorts = -1;
            _objectArray = null;
            _numberOfObjects = -1;
            _dataString = null;
            _currentTime = new OhTimePoint();
            _timeStep = null;
            _fixedTime = new OhTimePoint();
            _targetTime = new OhTimePoint();
            _calReqP = false;

            _className = "OhElement";
        }


        /// <summary> 
        /// ̃RXgN^̓oϐȉ̂悤ɐݒ肵܂B<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhElement"</li>
        /// <li> _objectName        : aObjName</li>
        /// <li> _objectNumber      : aObjNum</li>
        /// <li> _objectArray       : Ɋm(̔z̗vf aNumOfObjs )</li>
        /// <li> _dataString        : aStr</li>
        /// </dl>
        /// 
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value
        /// </param>
        /// <param name="aStr">a <var>String</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhElement(String aObjName, int aObjNum, int aNumOfObjs, String aStr)
            : base()
        {
            _initialWorkDoneP = false;
            _shareInfoDoneP = false;
            _objectList = null;
            _receivePortArray = null;
            _numberOfReceivePorts = -1;
            _sendPortArray = null;
            _numberOfSendPorts = -1;
            _objectArray = null;
            _numberOfObjects = -1;
            _dataString = null;
            _currentTime = new OhTimePoint();
            _timeStep = null;
            _fixedTime = new OhTimePoint();
            _targetTime = new OhTimePoint();
            _calReqP = false;

            _className = "OhElement";

            init(aObjName, aObjNum, aNumOfObjs, aStr);
        }


        /// <summary> 
        /// ̃RXgN^̓oϐȉ̂悤ɐݒ肵܂B<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhElement"</li>
        /// <li> _objectName        : aObjName</li>
        /// <li> _objectNumber      : aObjNum</li>
        /// <li> _dataString        : aStr</li>
        /// </dl>
        /// 
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aStr">a <var>String</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhElement(String aObjName, int aObjNum, String aStr)
            : base()
        {
            _initialWorkDoneP = false;
            _shareInfoDoneP = false;
            _objectList = null;
            _receivePortArray = null;
            _numberOfReceivePorts = -1;
            _sendPortArray = null;
            _numberOfSendPorts = -1;
            _objectArray = null;
            _numberOfObjects = -1;
            _dataString = null;
            _currentTime = new OhTimePoint();
            _timeStep = null;
            _fixedTime = new OhTimePoint();
            _targetTime = new OhTimePoint();
            _calReqP = false;

            _className = "OhElement";

            init(aObjName, aObjNum, aStr);
        }


        // @end {constructors} -------------------------------------------------

        // @begin {properties}------------------------------------------------------------
        /// <summary> 
        /// ݎ _currentTime Ԃ܂B
        /// 
        /// <p/>
        /// Returns '_currentTime'.
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        virtual public OhTimePoint CurrentTime
        {
            get
            {
                return _currentTime;
            }

        }
        /// <summary> 
        /// ^CXebv _timeStep Ԃ܂B
        /// 
        /// <p/>
        /// Returns '_timeStep'.
        /// 
        /// </summary>
        /// <returns> a <var>long</var> value
        /// </returns>
        virtual public IOhTimeImmutable TimeStep
        {
            get
            {
                return _timeStep;
            }

        }
        /// <summary> 
        /// Xebv _currentTime Ԃ܂B
        /// 
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePointImmutable</var> value
        /// </returns>
        virtual public IOhTimePointImmutable NextTime
        {
            get
            {
                return OhTimePoint.add(_currentTime, _timeStep);
            }

        }
        /// <summary> 
        /// vZIԂ܂B
        /// 
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        virtual public OhTimePoint TerminalTime
        {
            get
            {
                return ((OhElement)_owner).TerminalTime;
            }

        }
        /// <summary> 
        /// ԖڕW_owner 擾܂B
        /// 
        /// <p/>
        /// Returns the medium term target time from '_owner'.
        /// 
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        virtual public OhTimePoint MediumTermTargetTime
        {
            get
            {
                return ((OhElement)_owner).MediumTermTargetTime;
            }

        }
        /// <summary> 
        /// vZɂďԂm肵Ă鎞 _fixedTime Ԃ܂B
        /// 
        /// <p/>
        /// Returns '_fixedTime'.
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        virtual public OhTimePoint FixedTime
        {
            get
            {
                return _fixedTime;
            }

        }
        /// <summary> 
        /// vZڕW _targetTime Ԃ܂B
        /// 
        /// <p/>
        /// Returns '_targetTime'.
        /// 
        /// </summary>
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        virtual public OhTimePoint TargetTime
        {
            get
            {
                return _targetTime;
            }

        }

        /// <summary> 
        /// _objectArray  _numberOfObjects ݒ肵܂B<p/>
        /// Sets <b>_objectArray</b> and <b>_numberOfObjects</b>.
        /// </summary>
        /// 
        /// <remarks>
        /// aNumOfObjs (arg) ̃IuWFNgL邽߂̗̈q[
        /// vɊmۂA_objectArray ̐擪AhXw悤ɃZb
        /// g܂B<p/>
        /// 
        /// _objectArray ͒ڒʐM鑊L邽߂Ɏg܂B
        /// <p/>
        /// 
        /// ܂A
        /// 
        /// <pre>
        /// EaNumOfObjs (arg) ̏ꍇB
        /// E_numberOfObjects  0 ȏ̏ꍇBi쐬ς݁j
        /// </pre>
        /// 
        /// ̏ꍇ OhError IuWFNg𑗏o܂B
        /// 
        /// <pre>
        /// [ARGUMENT]
        /// EaNumOfObjs -> _objectArray ɋLIuWFNgB
        /// [RETURN]
        /// Etrue --> _objectArray mۂłꍇB
        /// Efalse -> aNumOfObjs ̏ꍇA܂ _numberOfObjects  0 ȏ̏ꍇBi쐬ς݁j
        /// </pre>
        /// 
        /// Acquire the memory to store <b>aNumOfObjs</b> objects and
        /// makes <b>_objectArray</b> point to the memory.
        /// </remarks>
        ///
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual int ObjectArray
        {
            set
            {
                if (value < 0)
                {
                    throw new ArgumentException(oPrintString("    "));
                }

                if ((_numberOfObjects >= 0) || (_objectArray != null))
                {
                    throw new OhError(oPrintString("    "));
                }

                try
                {
                    _objectArray = new OhObject[value];
                }
                catch (IndexOutOfRangeException e)
                {
                    throw new IndexOutOfRangeException(oPrintString("\t"));
                }

                for (int i = 0; i < value; i++)
                    _objectArray[i] = null;

                _numberOfObjects = value;
            }

        }

        /// <summary> 
        /// vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎
        /// ׂf[^̃f[^L^Ԃ܂B<p/>
        /// 
        /// NecessaryTimeFrom()  NecessaryTimeTo() ܂ł̃f[
        /// ^pbNM[q aRp MĂȂ΁Avf͌vZ
        /// Jn邱Ƃł܂B
        /// 
        /// ł͎M[qɈ˂炸A_currentTimeԂ悤ɂĂ܂B
        /// KvΔhNXievffjōĒ`ĂB
        /// <p/>
        /// 
        /// Returns the data record time of the data which
        /// <b>OhElement</b> should receive from <b>aRp</b> out of data
        /// required for next calculation. <p/>
        /// 
        /// <b>OhElement</b> cannot start calculation if the data
        /// pack from
        /// <b>getNecessaryTimeFrom</b> to <b>getNecessaryTimeTo</b>
        /// is not
        /// received from the receiving terminal <b>aRp</b>,
        /// <b>OhElement</b>
        /// cannot start calculation.
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// 
        /// <returns> an <var>OhTimePoint</var> value
        /// </returns>
        public virtual IOhTimePointImmutable NecessaryTimeFrom(OhReceivePort aRp)
        {
            return _currentTime;
        }


        /// <summary> 
        /// vf̌vZɕKvȃf[^̂AaRp (arg) 󂯎
        /// ׂf[^̃f[^L^Ԃ܂B<p/>
        /// 
        /// NecessaryTimeFrom()  NecessaryTimeTo() ܂ł̃f[
        /// ^pbNM[q aRp MĂȂ΁Avf͌vZ
        /// Jn邱Ƃł܂B
        /// 
        /// ł͎M[qɈ˂炸A_currentTimeԂ悤ɂĂ܂B
        /// KvΔhNXievffjōĒ`ĂB
        /// <p/>
        /// 
        /// Returns the data record time of the data which
        /// <b>OhElement</b> should receive from <b>aRp</b> out of data
        /// required for next calculation. <p/>
        /// 
        /// <b>OhElement</b> cannot start calculation if the
        /// data pack from
        /// <b>NecessaryTimeFrom</b> to <b>NecessaryTimeTo</b>
        /// is not
        /// received from the receiving terminal <b>aRp</b>,
        /// <b>OhElement</b>
        /// cannot start calculation.
        /// 
        /// 
        /// 
        /// 
        /// </summary>
        /// 
        /// <returns> an <var>IOhTimePointImmutable</var> value
        /// </returns>
        public virtual IOhTimePointImmutable NecessaryTimeTo(OhReceivePort aRp)
        {
            //; (*32)
            return NextTime;
        }


        /// <summary> 
        /// f[^ _dataString ݒ肵܂B<p/>
        /// 
        /// Sets the data string.
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual String DataString
        {
            set
            {
                if (_dataString != null)
                {
                    throw new OhError(oPrintString("\t"));
                }
                _dataString = value;
            }

        }
        // @begin {properties}------------------------------------------------------------

        // @begin {abstract methods} ------------------------------------------------------

        // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_objectArray"

        /// <summary> 
        /// hNX̑SM[qo^܂B<p/>
        /// 
        /// ̃\bh́AhNX̑SM[q
        /// <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>
        /// ɂo^܂B<p/>
        /// 
        /// o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B<p/>
        /// 
        /// Ⴆ΁AhNXŎM[q abc `ꍇA
        /// 
        /// <pre>
        /// public void registerReceivePort() {
        /// register((Object) abc);
        /// }
        /// </pre>
        /// 
        /// Ƃ΁Aabc M[qƂēo^ł܂B
        /// 
        /// <p/>
        /// Declares method registerReceivePorts() as abstract method.
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// "registerReceivePorts()" registers all the receiving ports of an inherited class,
        /// and all sending ports by
        /// <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>. <p/>
        /// 
        /// The port which is not registered is not applicable to sending and receiving,
        /// since it is not connectable. <br/>
        /// For example, when an inherited class defines the receiving port "abc",
        /// if it is performed as follows, "abc" can be registered as a receiving terminal.
        /// 
        /// <pre>
        /// void registerReceivePort() {
        /// register((Object)abc);
        /// }
        /// </pre>
        /// 
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract void registerReceivePorts();


        /// <summary> 
        /// hNX̑SM[qo^܂B<p/>
        /// 
        /// ̃\bh́AhNX̑SM[q
        /// <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>
        /// ɂo^܂B<p/>
        /// 
        /// o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B<p/>
        /// 
        /// Ⴆ΁AhNXőM[q abc `ꍇA
        /// 
        /// <pre>
        /// public void registerSendPort() {
        /// register((Object) abc);
        /// }
        /// </pre>
        /// 
        /// Ƃ΁Aabc 𑗐M[qƂēo^ł܂B
        /// 
        /// <p/>
        /// 
        /// Declares method "registerSendPorts() as abstract method.
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// "registerSendPorts()" registers all the receiving ports of an inherited class,
        /// and all sending ports by
        /// <a href="OhElement.html#register(java.lang.Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(java.lang.Object, int)">OhElement.register(Object, int)</a>. <p/>
        /// 
        /// The port which is not registered is not applicable to sending and receiving,
        /// since it is not connectable.
        /// For example, when an inherited class defines the receiving port "abc",
        /// if it is performed as follows, "abc" can be registered as a receiving terminal.
        /// 
        /// <pre>
        /// void registerSendPort() {
        /// register((Object)abc);
        /// }
        /// </pre>
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract void registerSendPorts();

        // 2. connection function

        // 3. numerical initialization

        /// <summary> 
        /// p^ݒ肷郁\bhAehNXɉĒ`܂B
        /// ̓Xg[ aSr Ŏw肳t@Cf[^ǂ݂Ƃăp^ݒ肵܂B<p/>
        /// 
        /// <p/>
        /// 
        /// Declares method "setParameter()" as abstract method.
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// Parameters are read in the input stream "aSr".
        /// It is necessary to create an element model according to each inherited class.
        /// Data is read in the file specified by the input stream "aSr",
        /// and the amount of initial states is set up. <p/>
        /// 
        /// Please have the program and consistency which save the last state at a file.
        /// 
        /// </summary>
        /// <param name="aSr">a <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract void setParameter(OhStreamReader aSr);

        /// <summary> 
        /// ԗʂݒ肷郁\bhAehNXɉĒ`܂B
        /// ̓Xg[ aSr Ŏw肳t@Cf[^ǂ݂Ƃďԗʂݒ肵܂B<p/>
        /// 
        /// ŏIԂt@CɕۑvOƂ܂悤ɂĂB
        /// 
        /// <p/>
        /// Declares method "setInitialState()" as abstract method.
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// The method which sets up the amount of initial states is defined according to each inherited class.
        /// 
        /// </summary>
        /// <param name="aSr">a <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract void setInitialState(OhStreamReader aSr);

        // 4. time about a simulation

        // 5. execution of calculation

        /// <summary> 
        /// ̏o͂sȂ܂B<p/>
        /// 
        /// ԗʂ߂ɁA_currentTime ł̏o͂`܂B
        /// M[qɎƏo͒l݂܂B
        /// _currnetTime ł̏o͂vZłꍇ true Ԃ܂B
        /// Kvȓ̓f[^ĂȂȂǂāA
        /// _currentTime ̏o͂vZłȂꍇ false Ԃ܂B<p/>
        /// 
        /// vfɂẮAsetInitialState() ̒
        /// ̏o͂悤ɂs悢m܂B
        /// Ƃ΁Alݒ̂߂ɗ^镶ɂāA
        /// ̏o͂̂ς肷ꍇȂǂłB
        /// ꍇ́AinitialOutput() ͉Ȃ悤ɒ`āA
        /// setInitialState() ŏ̏o͂悤ɒ`Ă\܂B
        /// 
        /// <p/>
        /// 
        /// An initial output.
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// The output in "_currentTime" is defined immediately after deciding the amount of initial states.
        /// Time and an output value are written in a sending port.
        /// "true" is returned when the output in "_currentTime" is able to be calculated.
        /// "false" is returned when the output of "_currentTime" is not able to be calculated
        /// by required input data not being obtained.
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract bool initialOutput();

        /// <summary> 
        /// ^CXebv̌vZsȂ܂B<p/>
        /// Calculation of a time step.
        /// </summary>
        /// <remarks>
        /// ehNXŃfɉ _timeStep [sec] vZA
        /// ߂lƂĕԂ\bh`ĉB<p/>
        /// 
        /// ܂A_timeStep ǂ̂悤ȏŁAǂ̂悤Ȓlɂ邩`܂B<p/>
        /// 
        /// Ⴆ΁A_timeStep l 3600 [sec] ɐݒ肷ꍇA
        /// 
        /// <pre><code>
        /// public long calculateTimeStep() 
        /// {
        ///     return 3600;
        /// }
        /// </code></pre>
        /// 
        /// ƒ`ĉB<p/>
        /// 
        /// ܂AL̂悤Ƀoϐ _k ̒lɂ _timeStep ύX邱Ƃł܂B<p/>
        /// 
        /// <pre><code>
        /// public long calculateTimeStep() 
        /// {
        ///     return (_k > 0) ? 360 : 60;
        /// }
        /// 
        /// public long calculateTimeStep() 
        /// {
        ///     return 360 * pow(10, _k);
        /// }
        /// </code></pre>
        /// 
        /// _timeStep ́Awork() ̒Őݒ肳̂ŁA
        /// 
        /// <pre><code>
        /// public long calculateTimeStep() 
        /// {
        ///     _timeStep = 1;
        ///     return 1;
        /// }
        /// </code></pre>
        /// 
        /// ̂悤ɁÃ\bh̒ŕύXȂłB<p/>
        /// 
        /// ۃ\bhȂ̂ŁAhNXŕK`ĉB<p/>
        /// 
        /// ߂l: ̌vZ̃^CXebvB[sec]
        /// 
        /// <p/>
        /// 
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// Please calculate "_timeStep" according to a model in each inherited class,
        /// and define the method which returns it as a return value.
        /// That is, it defines under what conditions "_timeStep" is made into what value.
        /// </remarks>
        /// <returns> a <var>long</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract OhTime calculateTimeStep();

        /// <summary> 
        /// vZJn̉Es̔f܂B<p/>
        /// Judgment whether calculation can be started or not.
        /// </summary>
        /// <remarks>
        /// ܂A_currentTime  _terminalTime ȏ̏ꍇAvZς݂Ɣf false Ԃ܂B<p/>
        /// 
        /// ɁASĂ̏LM[q getNecessaryTimeFrom(), getNecessaryTimeTo() Ԃ
        /// f[^pbN擾\ł邱ƂmF܂B
        /// 擾\ȏꍇAvZs\Ɣf true Ԃ܂B
        /// 擾s\ȏꍇAvZssƔf false Ԃ܂B<p/>
        /// 
        /// f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
        /// ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
        /// <pre>
        /// [߂l]
        /// Etrue --> vZs\̏ꍇB
        /// Efalse -> vZss\̏ꍇA܂͌vZς݂̏ꍇB
        /// </pre>
        /// 
        /// <p/>
        /// 
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// First, when "_currentTime" is more than "_terminalTime",
        /// it is judged that calculation has finished, and "false" is returned. <p/>
        /// 
        /// Next, it checks that all possession receiving ports can acquire the data pack
        /// between "getNecessaryTimeFrom" and "getNecessaryTimeTo".
        /// It is judged that calculation execution is possible the case of being acquirable,
        /// "true" is returned.
        /// It is judged that calculation execution is impossible the case of being unacquirable,
        /// "false" is returned. <p/>
        /// 
        /// A calculation request is advanced to the receiving port
        /// which will have replied that data is unacquirable.
        /// In particular, a calculation request message is sent to a receiving port.
        /// </remarks>
        /// 
        /// 
        /// <returns> a <var>bool</var> value<p/>
        /// <pre>
        /// Etrue --> When calculation execution is possible.
        /// Efalse -> When calculation execution is impossible.
        ///            When calculation has finished.
        /// </pre>
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract bool canYouCalculate();

        /// <summary> 
        /// PXebv̌vZ葱`܂B<p/>
        /// 
        /// e[UƎ̗vff쐬ہAPXebv̌vZ葱`ĉB
        /// 
        /// ʓIȗvfł́A
        /// 
        /// <ul>
        /// <li> ŏAeԗʂɂ _currentTime ɂli[ĂB</li>
        /// <li> ł _currentTime + _timeStep ɂԗ, o͒lvZB</li>
        /// <li> ̃\bhIAԗʂ _currentTime + _timeStep ɂlɍXVĂB</li>
        /// </ul>
        /// 
        /// ̂R_𖞑悤ɁAL (1)  (4) ɋLqĉB
        /// 
        /// <pre>
        /// (1) M[q̓́B
        /// (2) vZB
        /// (3) M[qւ̏óB
        /// (4) ԗʂ̍XVB
        /// </pre>
        /// 
        /// (2) ł́Ap[^A_currentTime ɂԗʁAM[q̓͂pA
        /// _currentTime + _timeStep ɂԗʁAMׂo͒lvZ鐔f̂PXebv
        /// ̓Iɒ`ĉB<p/>
        /// 
        /// ̃\bh̒ _currentTime, _timeStep ύXȂŉB
        /// KvȂ _timeStep Xɍׂԋ敪ɕČvZł܂A
        /// ͍̎ƗpϐpA_currentTime, _timeStep ύXȂ悤ɒӂĉB<p/>
        /// 
        /// vZɕKvȑSĂ̐ĺAf[^oyяLM[q̓͂ƂĎ擾ł܂B
        /// ]Ĉ͕svłB<p/>
        /// 
        /// \ʂvZłꍇ́Atrue Ԃ܂B
        /// calculate() ̓ŌvZԃXebvvZA
        /// ̌vZԃXebṽf[^擾łȂȂǂāA
        /// vZi߂邱ƂłȂꍇ́Afalse Ԃ܂B<p/>
        /// 
        /// ۃ\bhȂ̂ŁAhNXŕK`ĉB
        /// 
        /// <pre>
        /// [ACCIDENT]
        /// Ẽ\bh̒ _currentTime, _timeStep ύXꂽꍇB
        /// </pre>
        /// 
        /// <p/>
        /// 
        /// Calculation procedure for 1 step.
        /// In classes derived from class Element, its definition should be given.
        /// 
        /// <ul>
        /// <li>At first, in each amount of states, the value at "_currentTime" is stored.</li>
        /// <li>Here, the amount of states and output value in "_currentTime + _timeStep" are calculated.</li>
        /// <li>Next, the amount of states is updated by the value in a certain time.</li>
        /// </ul>
        /// 
        /// In a general element,
        /// Please describe following (1) to (4) in order to fulfill these 3 points.
        /// 
        /// <pre>
        /// (1) the input from a receiving port.
        /// (2) numerical computation.
        /// (3) the output to a sending port.
        /// (4) updating of the amount of states.
        /// </pre>
        /// 
        /// At (2), by using parameters, the amount of states in "_currentTime" , and the input from a receiving ports,
        /// please define concretely one step of a numerical model
        /// which calculate the amount of states and the output value
        /// which should be transmitted in "_currentTime + _timeStep". <p/>
        /// 
        /// In this method, please do not change "_currentTime" and "_timeStep". <p/>
        /// 
        /// All numerical values required for calculation are acquirable
        /// as instance variable and input from a possession receiving port.
        /// Therefore, arguments are unnecessary. <p/>
        /// 
        /// The case where it is calculable as planned,
        /// "true" is returned.
        /// When a calculation time step is calculated inside "calculate()",
        /// the data for the calculation time step cannot be acquired and calculation is not able to be advanced,
        /// "false" is returned.
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract bool calculate();

        /// <summary> 
        /// _currentime 
        /// <a href="OhTotalhtml#_terminalTime">OhTotal_terminalTime</a>
        /// Ԃ̌vZASY`܂B<p/>
        /// 
        /// calculate(), calculateTimeStep() ȂǂpāA_terminalTime ܂ł̌vZs܂B<p/>
        /// 
        /// ܂A
        /// <a href="OhElement.html#canYouCalculate()">OhElement.canYouCalculate()</a>
        /// ̖߂l false Ȃ΁AvZJnsƔfAfalse Ԃ܂B<p/>
        /// 
        /// ߂l true ̏ꍇ́A
        /// true łA
        /// <a href="OhElement.html#calculate()">OhElement.calculate()</a>,
        /// _currentTime.add(_timeStep) JԂs܂B<p/>
        /// 
        /// ŌɁA
        /// <a href="OhElement.html#setDataPackArray()">OhElement.setDataPackArray()</a>,
        /// sAtrue Ԃ܂B
        /// 
        /// <pre> 
        /// [߂l]
        /// Etrue --> vZsꍇB
        /// Efalse -> vZs\̏ꍇA܂͌vZς݂̏ꍇB
        /// </pre>
        /// 
        /// <p/>
        /// 
        /// Calculate between "_currentTime" and "_terminalTime".
        /// In classes derived from class Element, its definition should be given. <p/>
        /// 
        /// Calculation by "_terminalTime" is performed using "calculate()", "calculateTimeStep()", etc. <p/>
        /// 
        /// First, if return value of "canYouCalculate()" becomes "false",
        /// it will be judged that a calculation start is impossible,
        /// "false" is returned. <p/>
        /// 
        /// When the return value is "true",
        /// "calculate()" and "_currentTime += _timeStep" are repeated and performed,
        /// as long as the return value of "canYouCalculate" is "true".  <p/>
        /// 
        /// Finally, "setDataPackArray()" and "reportToSender()" are performed and "true" is returned.
        /// 
        /// <pre>
        /// [RETURN]
        /// Etrue --> When calculation was performed.
        /// Efalse -> When calculation is impossible.
        /// When calculation has finished.
        /// </pre>
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public abstract bool work();

        // 6. information output function

        // 7. the function to save the last state at a file

        /// <summary> 
        /// vf̍ŏIԂo̓Xg[ sw Ŏw肳t@CɏލƂ郁\bhłB
        /// [U`vffł́Ã\bhK`Ȃ΂Ȃ܂B<p/>
        /// 
        /// ŏIԂގdl͎̂悤ɂĂB
        /// <pre>
        /// (data_string)
        /// </pre>
        /// 
        /// <p/>
        /// This is the method which does the work which writes the last state of an element
        /// in the file specified by the output stream "aSr". <p/>
        /// 
        /// In classes derived from class Element, its definition should be given.
        /// 
        /// </summary>
        /// <param name="aPr">a <var>StreamWriter</var> value
        /// </param>
        public abstract void saveTerminalState(StreamWriter aPr);

        // @end {abstract methods} ---------------------------------------------



        // @begin {methods} ----------------------------------------------------

        // 1. creation and reference of "_receivePortArray", "_sendPortArray" and "_objectArray"

        /// <summary> 
        /// _objectList  OhObject ^IuWFNg擾܂B<p/>
        /// Finds an object typed Object from "_objectList".
        /// </summary>
        /// <remarks>
        /// ̃\bh́ANX, IuWFNg, IuWFNgԍw肵āA
        /// _objectList  OhObject ^IuWFNg擾܂B<p/>
        /// 
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B<p/>
        /// 
        /// _objectList  null ̏ꍇ́AG[bZ[W\Anull Ԃ܂B
        /// 
        /// <pre><dl>
        /// 
        /// <dt>[ACCIDENT]</dt>
        /// <dd>EOhObject ^ȊO̕ϐ object_list Ɏ^Ăꍇ</dd>
        /// </dl></pre>
        /// 
        /// <p/>
        /// Finds an object with class name 'aClassName', object name 'aObjName', and
        /// object number 'aObjNum' from "_objectList"
        /// and returns the object.
        /// When such an object could not be found from "_objectList",
        /// "null" will be returned.
        /// </remarks>
        /// <param name="aClassName">a <var>String</var> value<p/>
        /// 擾IuWFNg̃NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// 擾IuWFNg̃IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// 擾IuWFNg̃IuWFNgԍ
        /// </param>
        /// <returns> an <var>OhObject</var> value<p/>
        /// Object ^ɃLXgꂽwIuWFNg<p/>
        /// wIuWFNg𔭌łȂA܂_objectList ݂Ȃꍇ͖߂lnullɂȂ܂B
        ///       
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OhObject getFromList(String aClassName, String aObjName, int aObjNum)
        {
            //; (*1)
            if (_objectList == null)
            {
                //; The list was already arranged.
                string str = "\tAsked > [" + aClassName + "] : '" + aObjName + "' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("_objectList is null.", str);
            }

            for (int i = 0; i < _objectList.Count; i++)
            {
                try
                {
                    if (((OhObject)_objectList[i]).objCompare(aClassName, aObjName, aObjNum) == true)
                        return (OhObject)_objectList[i]; // The appointed object was discovered.
                }
                catch (NullReferenceException e)
                {
                    e = new NullReferenceException("At least, an element of _objectList is null.");
                    Console.WriteLine(e.GetType().ToString());
                    Console.WriteLine(e.StackTrace);
                    Console.WriteLine(e.Message);
                    throw e;
                }
            }
            return null; // The appointed object was not registered.
        }

        /// <summary> 
        /// _objectList  OhObject ^IuWFNg擾܂B
        /// </summary>
        /// <remarks>
        /// 
        /// Finds an object with object name 'aObjName', and object number 'aObjNum' from "_objectList"
        /// and returns the object.
        /// When such an object could not be found from "_objectList",
        /// "null" will be returned.
        /// </remarks>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <returns> an <var>OhObject</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OhObject getFromList(String aObjName, int aObjNum)
        {
            //; (*2)
            if (_objectList == null)
            {
                // The list was already arranged.
                string str = "\tAsked > " + aObjName + " : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("_objectList is null.", str);
            }


            for (int i = 0; i < _objectList.Count; i++)
            {
                try
                {
                    if (((OhObject)_objectList[i]).objCompare(aObjName, aObjNum) == true)
                        return (OhObject)_objectList[i]; // The appointed object was discovered.
                }
                catch (NullReferenceException e)
                {
                    e = new NullReferenceException("At least, an element of _objectList is null.");
                    Console.WriteLine(e.GetType().ToString());
                    Console.WriteLine(e.StackTrace);
                    Console.WriteLine(e.Message);
                    throw e;
                }
            }
            return null; // The appointed object was not registered.
        }


        /// <summary> 
        /// OhObject ^IuWFNg̓o^ _owner ̐ݒs܂B<p/>
        /// 
        /// Registers an object 'aObj' to the data member '_objectList'.
        /// </summary>
        /// <remarks>
        ///  OhObject ^IuWFNg "aObj"  _objectList ɓo^܂B
        /// ɁÃ\bhĂяoIuWFNgithisj "aObj"  _owner ƂȂ悤ɐݒ肳܂B<p/>
        /// 
        /// o^ɂȀꍇ̓G[bZ[Wo͂܂B@
        /// <ul>
        /// <li> aObj  null ̏ꍇB</li>
        /// <li> object_list  NULL ̏ꍇB</li>
        /// <li> NXAIuWFNgAIuWFNgԍS aObj ƈvIuWFNg
        /// _objectList ɓo^ĂꍇB</li>
        /// <li>aObj  _owner ݒς݂̏ꍇB</li>
        /// </ul>
        /// 
        /// ܂Ȁꍇُ͈𔭐܂B
        /// 
        /// <ul>
        /// <li> aObj  OhObject ^ȊO̕ϐi[ĂꍇB</li>
        /// <li> _objectList  OhObject ^ȊO̕ϐ^ĂꍇB</li>
        /// </ul>
        /// </remarks>
        /// 
        /// 
        /// <param name="aObj">an <var>OhObject</var> value<p/>
        /// OhObject ^ɃLXgꂽo^IuWFNgB
        /// </param>
        /// <exception cref="OhError">
        /// <ul>
        /// <li>_objectListnull̏ꍇ</li>
        /// <li>aObj̃IuWFNg܂̓IuWFNgԍݒ肳ĂȂꍇ</li>
        /// <li>aObjłɓo^Ăꍇ</li>
        /// </ul>
        /// 
        /// </exception>
        /// <exception cref="ArgumentException">aObjnull̏ꍇ
        /// </exception>
        public virtual void register(OhObject aObj)
        {
            //; (*3)
            if (aObj == null)
            {
                // The argument is inaccurate.
                throw new ArgumentException("\"aObj\"(arg) is null." + NEWLINE + oPrintString("\t"));
            }

            if (_objectList == null)
            {
                // The list was already arranged.
                throw new OhError("\"_objectList\" is null", oPrintString("\t"));

            }

            // Comparison of class name, object name, and object number.
            // When variables other than "Object" type are stored in "aObj", abnormalities occur.
            //
            String className = aObj.ClassName;
            String objName = aObj.ObjectName;
            int objNum = aObj.ObjectNumber;

            if (objName == null || objNum == -1)
            {
                string str = aObj.oPrintString("    registered > ") + oPrintString("\t");
                throw new OhError("At least, \"ObjectName\" or \"ObjectNumber\" is not set.", str);
            }

            if (getFromList(className, objName, objNum) != null)
            {
                string str = oPrintString("\t") + aObj.oPrintString("    registered > ");
                throw new OhError("\"aObj\"(arg) has been already set in \"_objectList\".", str);
            }


            aObj.Owner = this; //; aObj  _owner  ̃\bhĂяo this ɐݒ肷B

            _objectList.Add(aObj);

        }
        /// <summary>
        /// C++/CLIŋLqꍇregisterłBe
        /// <a href = "OhymosNet.OhElement.register_overload_1.html">register(OhObject)</a>
        /// lłB
        /// </summary>
        /// <param name="aObj"></param>
        public virtual void registerForCLI(OhObject aObj)
        {
            //; (*3)
            if (aObj == null)
            {
                // The argument is inaccurate.
                throw new ArgumentException("\"aObj\"(arg) is null." + NEWLINE + oPrintString("\t"));
            }

            if (_objectList == null)
            {
                // The list was already arranged.
                throw new OhError("\"_objectList\" is null", oPrintString("\t"));

            }

            // Comparison of class name, object name, and object number.
            // When variables other than "Object" type are stored in "aObj", abnormalities occur.
            //
            String className = aObj.ClassName;
            String objName = aObj.ObjectName;
            int objNum = aObj.ObjectNumber;

            if (objName == null || objNum == -1)
            {
                string str = aObj.oPrintString("    registered > ") + oPrintString("\t");
                throw new OhError("At least, \"ObjectName\" or \"ObjectNumber\" is not set.", str);
            }

            if (getFromList(className, objName, objNum) != null)
            {
                string str = oPrintString("\t") + aObj.oPrintString("    registered > ");
                throw new OhError("\"aObj\"(arg) has been already set in \"_objectList\".", str);
            }


            aObj.Owner = this; //; aObj  _owner  ̃\bhĂяo this ɐݒ肷B

            _objectList.Add(aObj);

        }

        /// <summary> 
        /// _objectArray̗vfCfbNXw肵āAOhObject ^IuWFNg̓o^ _owner ̐ݒsȂ܂B
        /// <p/>
        /// Registers the object "aObj" with which the current object communicates directly
        /// to object_array[num].
        /// </summary>
        /// <remarks>_objectArray aNum Ԗڂ̗vf aObj o^܂Bo^trueԂ܂B
        /// </remarks>
        /// <param name="aObj">an <var>OhObject</var> value
        /// </param>
        /// <param name="aNum">an <var>int</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void register(OhObject aObj, int aNum)
        {
            //; (*6)
            if ((aNum < 0) || (aNum >= _numberOfObjects))
            {
                string str = "\tnum (arg) = " + aNum + ", _numberOfObjects = " + _numberOfObjects + NEWLINE + oPrintString("\t");
                throw new ArgumentException("\"aNum\" is out of range of \"_numberOfObjects\"." + NEWLINE + str);
            }

            if (aObj == null)
            {
                throw new ArgumentException("\"aObj\"(arg) is null" + NEWLINE + oPrintString("\t"));
            }

            if (_objectArray[aNum] != null)
            {
                string str = "\tDoubly set _objectArray[" + aNum + "]." + NEWLINE
                           + _objectArray[aNum].oPrintString("    first > ")
                           + aObj.oPrintString("    next > ");
                throw new OhError("\"aObj\"(first arg) have been already set in \"_objectArray\"[\"aNum\"(second arg)].", str);
            }

            aObj.Owner = this; // javałł͂Ȃ

            _objectArray[aNum] = aObj;
        }


        /// <summary>
        /// C++/CLIŋLqꍇregisterłBe
        /// <a href = "OhymosNet.OhElement.register_overload_2.html">register(OhObject, int)</a>
        /// lłB
        /// </summary>
        /// <param name="aObj">an <var>OhObject</var> value
        /// </param>
        /// <param name="aNum">an <var>int</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void registerForCLI(OhObject aObj, int aNum)
        {
            //; (*6)
            if ((aNum < 0) || (aNum >= _numberOfObjects))
            {
                string str = "\tnum (arg) = " + aNum + ", _numberOfObjects = " + _numberOfObjects + NEWLINE + oPrintString("\t");
                throw new ArgumentException("\"aNum\" is out of range of \"_numberOfObjects\"." + NEWLINE + str);
            }

            if (aObj == null)
            {
                throw new ArgumentException("\"aObj\"(arg) is null" + NEWLINE + oPrintString("\t"));
            }

            if (_objectArray[aNum] != null)
            {
                string str = "\tDoubly set _objectArray[" + aNum + "]." + NEWLINE
                           + _objectArray[aNum].oPrintString("    first > ")
                           + aObj.oPrintString("    next > ");
                throw new OhError("\"aObj\"(first arg) have been already set in \"_objectArray\"[\"aNum\"(second arg)].", str);
            }

            aObj.Owner = this; // javałł͂Ȃ

            _objectArray[aNum] = aObj;
        }


        /// <summary> 
        ///_objectList z񉻂ĕԂA
        /// ɁA aNumOfObjs izĵPԖڂɗvf _numberOfObjects i[܂B<p/>
        /// 
        ///  Creates an array of objects which are included in '_objectList'
        /// and stores it in '_aObjArray' and the number of elements in 'aNumOfObjs'.
        /// </summary>
        /// 
        /// <remarks>
        /// _objectList  null wĂꍇAG[bZ[W\܂B<p/>
        /// 
        /// ܂A aNumOfObjs[0]  0 ȏ̏ꍇ́Aɔz쐬ς݂ƔfA
        /// G[bZ[W\܂B
        /// z񉻌A_numOfObjs[0]  0 ȏɐݒ肳܂B
        /// ]āAzݒOɂ _numOfObjs[0]  0 ɏĂĉB<p/>
        /// 
        /// _numberOfReceivePort, _numberOfSendPort ́ARXgN^ -1 ɏ܂B<p/>
        /// 
        /// </remarks>
        /// <p/>
        ///
        /// 
        /// <param name="aNumOfObjs">an <var>int[]</var> value<p/>
        /// Pvf_objectList ̗vfi_numberOfObjectsji[int^z
        /// </param>
        /// <returns> an <var>Object[]</var> value<p/>
        /// _objectList z񉻂 Object ^z
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object[] makeObjectArray(int[] aNumOfObjs)
        {
            //; (*4)
            if (_objectList == null)
            {
                throw new OhError("\"_objectList\" is null", oPrintString("\t"));
            }

            if (aNumOfObjs[0] >= 0)
            {
                throw new OhError("\"aNumOfObjs[0]\" have been already set.", oPrintString("\t"));
            }

            Object[] aObjArray = (Object[])_objectList.ToArray(typeof(Object));

            aNumOfObjs[0] = _objectList.Count;

            return aObjArray;
        }



        /// <summary> 
        /// Object ^IuWFNgLpz쐬܂B
        /// ܂A_receivePortArray, _sendPortArray, _objectArray 쐬܂B<p/>
        /// Makes "_receivePortArray" and "_sendPortArray".
        /// </summary>
        /// 
        /// <remarks>
        /// ܂A_objectList  null ł邱ƂmF܂B
        /// null łȂꍇA_objectList gpƔfăG[bZ[W\Areturn ܂B<p/>
        /// 
        /// ɁAȉ̍Ƃs܂B
        /// 
        /// <ol>
        /// <li> OhLinkedList IuWFNgq[vɍ쐬 _objectList ɐݒ肷B</li>
        /// <li> OhElement.registerReceivePort(), OhElenment.makeObjectArray() p
        /// _receivePortArray 쐬B</li>
        /// <li> OhLinkedList.clear() ɂ _objectList B</li>
        /// <li> OhElement.registerSendPort(), OhElement.makeObjectArray() p
        /// _sendPortArray 쐬B</li>
        /// </ol>
        /// 
        /// Ō _objectList  null ɐݒ肵܂B<p/>
        /// </remarks>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void construct()
        {
            //; (*7)
            if (_objectList != null)
            {
                throw new OhError("\"_objectList\" has been used.", oPrintString("    "));
            }

            _objectList = new OhLinkedList();

            // --- Makes _receivePortArray. ---

            registerReceivePorts(); // Abstract method call. Throws "OhError".

            int[] numberOfReceivePortsArray = new int[] { _numberOfReceivePorts };

            _receivePortArray = makeObjectArray(numberOfReceivePortsArray); //; (*4)
            _numberOfReceivePorts = numberOfReceivePortsArray[0];

            _objectList.Clear();

            // --- Makes _sendPortArray. ---

            registerSendPorts(); // Abstract method call.

            int[] numberOfSendPortsArray = new int[] { _numberOfSendPorts };

            _sendPortArray = makeObjectArray(numberOfSendPortsArray);
            _numberOfSendPorts = numberOfSendPortsArray[0];

            _objectList = null;
        }



        /// <summary> 
        /// OhObject ^IuWFNg擾܂B<p/>
        /// gets object typed Object.
        /// </summary>
        /// <remarks>
        /// getReceivePort(), getSendPort(), getObject() ŗp⏕\bh̖S܂B
        /// 
        /// <p/>
        /// Finds an object with class name 'aClassName', object name 'aObjName', and object number 'aObjNum'
        /// from 'aObjArray'
        /// and returns the object.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value<p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// <param name="aObjArray">an <var>Object[]</var> value<p/>
        /// IuWFNgi[ĂIuWFNg^z
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value<p/>
        /// IuWFNgz̗vf
        /// </param>
        /// <returns> an <var>OhObject</var> value<p/>
        /// w肳ꂽIuWFNg
        /// </returns>
        public virtual OhObject getFromArray(String aClassName, String aObjName, int aObjNum, Object[] aObjArray, int aNumOfObjs)
        {
            //; (*8)
            if (aNumOfObjs < 0)
            {
                // The list is not arranged.
                throw new ArgumentException("The list is not arranged." + NEWLINE + oPrintString("\t"));
            }

            for (int k = 0; k < aNumOfObjs; k++)
                if (((OhObject)aObjArray[k]).objCompare(aClassName, aObjName, aObjNum) == true)
                    return (OhObject)aObjArray[k]; // The appointed object was discovered.

            return null;
        }

        /// <summary> 
        /// Object ^IuWFNg擾܂B<p/>
        /// gets object typed Object.
        /// </summary>
        /// <remarks>
        /// getReceivePort(), getSendPort(), getObject() ŗp⏕\bh̖S܂B
        /// 
        /// <p/>
        /// Finds an object with class name 'aClassName', object name 'aObjName', and object number 'aObjNum'
        /// from 'aObjArray'
        /// and returns the object.
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// <param name="aObjArray">an <var>Object[]</var> value<p/>
        /// IuWFNgi[ĂIuWFNg^z
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value<p/>
        /// IuWFNgz̗vf
        /// </param>
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽIuWFNg
        /// </returns>
        public virtual Object getFromArray(String aObjName, int aObjNum, Object[] aObjArray, int aNumOfObjs)
        {
            //; (*9)
            if (aNumOfObjs < 0)
            {
                // The list is not arranged.
                throw new ArgumentException("The list is not arranged." + NEWLINE + oPrintString("\t"));
            }

            for (int k = 0; k < aNumOfObjs; k++)
                if (((OhObject)aObjArray[k]).objCompare(aObjName, aObjNum) == true)
                    return aObjArray[k]; // The appointed object was discovered.

            return null;
        }

        /// <summary> 
        /// _receivePortArray M[q擾܂B<p/>
        /// gets a receiving port from "_receivePortArray".
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgAIuWFNgԍw肷邱ƂɂA
        /// _receivePortArray M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ̓G[bZ[Wo͂܂B
        /// 
        /// <p/>
        /// By appointing class name, object name, and object number,
        /// acquires a receiving port from "_receivePortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value<p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// 
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getReceivePort(String aClassName, String aObjName, int aObjNum)
        {
            //; (*10-1)
            Object p = getFromArray(aClassName, aObjName, aObjNum, _receivePortArray, _numberOfReceivePorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked >  "+ aClassName + " : \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(receiving port) was not registered.", str);
            }
            return p;
        }

        /// <summary> 
        /// _receivePortArray M[q擾܂B<p/>
        /// gets a receiving port from "_receivePortArray".
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgw肷邱ƂɂA
        /// _receivePortArray M[q擾܂B
        /// IuWFNgԍ0̂̂ƂȂ܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing class name and object name,
        /// acquires a receiving port from "_receivePortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value<p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getReceivePort(String aClassName, String aObjName)
        {
            //; (*10-2)
            Object p = getFromArray(aClassName, aObjName, 0, _receivePortArray, _numberOfReceivePorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(receiving port) was not registered.", str);
            }
            return p;
        }

        /// <summary> 
        /// _sendPortArray 瑗M[q擾܂B<p/>
        /// gets a sending port from "_sendPortArray".
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgAIuWFNgԍw肷邱ƂɂA
        /// _sendPortArray 瑗M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing class name, object name, and object number,
        /// acquires a sending port from "_sendPortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value <p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value <p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value <p/>
        /// IuWFNgԍ
        /// </param>
        /// <returns> an <var>Object</var> value <p/>
        /// w肵M[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getSendPort(String aClassName, String aObjName, int aObjNum)
        {
            //; (*11-1)
            Object p = getFromArray(aClassName, aObjName, aObjNum, _sendPortArray, _numberOfSendPorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(sending port) was not registered.", str);
            }
            return p;
        }


        /// <summary> 
        /// _sendPortArray 瑗M[q擾܂B<p/>
        /// gets a sending port from "_sendPortArray".
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgw肷邱ƂɂA
        /// _sendPortArray 瑗M[q擾܂B
        /// IuWFNgԍ0̂̂ƂȂ܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing class name and object name,
        /// acquires a sending port from "_sendPortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value <p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value <p/>
        /// IuWFNg
        /// </param>
        /// <returns> an <var>Object</var> value <p/>
        /// w肵M[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getSendPort(String aClassName, String aObjName)
        {
            //; (*11-2)
            Object p = getFromArray(aClassName, aObjName, 0, _sendPortArray, _numberOfSendPorts);

            if (p == null)
            {
                //; The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(sending port) was not registered.", str);
            }
            return p;
        }



        /// <summary> 
        /// _receivePortArray M[q擾܂B<p/>
        /// gets a receiving port from "_receivePortArray".
        /// </summary>
        /// <remarks>
        /// IuWFNgAIuWFNgԍw肷邱ƂɂA
        /// _receivePortArray M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing object name and object number,
        /// acquires a receiving port from "_receivePortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// 
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getReceivePort(String aObjName, int aObjNum)
        {
            //; (*12-1)
            Object p = getFromArray(aObjName, aObjNum, _receivePortArray, _numberOfReceivePorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(receiving port) was not registered.", str);
            }
            return p;
        }


        /// <summary> 
        /// _receivePortArray M[q擾܂B<p/>
        /// gets a receiving port from "_receivePortArray".
        /// </summary>
        /// <remarks>
        /// IuWFNgw肷邱ƂɂA
        /// _receivePortArray M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing object name,
        /// acquires a receiving port from "_receivePortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// 
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getReceivePort(String aObjName)
        {
            //; (*12-2)
            Object p = getFromArray(aObjName, 0, _receivePortArray, _numberOfReceivePorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(receiving port) was not registered.", str);
            }
            return p;
        }


        /// <summary> 
        /// _sendPortArray 瑗M[q擾܂B<p/>
        /// gets a sending port from "_sendPortArray".
        /// </summary>
        /// <remarks>
        /// IuWFNgAIuWFNgԍw肷邱ƂɂA
        /// _receivePortArray 瑗M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing object name and object number,
        /// acquires a sending port from "_sendPortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// 
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getSendPort(String aObjName, int aObjNum)
        {
            //; (*13-1)
            Object p = getFromArray(aObjName, aObjNum, _sendPortArray, _numberOfSendPorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(sending port) was not registered.", str);
            }
            return p;
        }


        /// <summary> 
        /// _sendPortArray 瑗M[q擾܂B<p/>
        /// gets a sending port from "_sendPortArray".
        /// </summary>
        /// <remarks>
        /// IuWFNgw肷邱ƂɂA
        /// _sendPortArray 瑗M[q擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// 
        /// <p/>
        /// By appointing object name,
        /// acquires a sending port from "_sendPortArray". <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽM[q
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getSendPort(String aObjName)
        {
            //; (*13-2)
            Object p = getFromArray(aObjName, 0, _sendPortArray, _numberOfSendPorts);

            if (p == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object(sending port) was not registered.", str);
            }
            return p;
        }



        /// <summary> 
        /// _objectArrayIuWFNg擾܂B<p/>
        /// gets a object from _objectArray.
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgAIuWFNgԍw肷邱Ƃɂ
        /// _objectArrayIuWFNg擾܂B
        /// w肳ꂽIuWFNgo^ĂȂꍇ null Ԃ܂B
        /// <p/>
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value<p/>
        /// NX
        /// </param>
        /// <param name="aObjName">a <var>String</var> value<p/>
        /// IuWFNg
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value<p/>
        /// IuWFNgԍ
        /// </param>
        /// <returns> an <var>Object</var> value<p/>
        /// w肳ꂽIuWFNg
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getObject(String aClassName, String aObjName, int aObjNum)
        {
            //; (*14-1)
            Object obj = getFromArray(aClassName, aObjName, aObjNum, _objectArray, _numberOfObjects);

            if (obj == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : " + aObjNum + " :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object was not registered.", str);
            }
            return obj;
        }


        /// <summary> 
        /// _objectArrayIuWFNg擾܂B<p/>
        /// gets a object from _objectArray.
        /// </summary>
        /// <remarks>
        /// w肳ꂽIuWFNgo^ĂȂꍇ G[bZ[WԂ܂B
        /// <p/>
        /// At this time, the object number is set to 0. <p/>
        /// 
        /// When the appointed object is not registered,
        /// "null" is returned.
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value
        /// </param>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <returns> an <var>Object</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getObject(String aClassName, String aObjName)
        {
            //; (*14-2)
            Object obj = getFromArray(aClassName, aObjName, 0, _objectArray, _numberOfObjects);

            if (obj == null)
            {
                // The appointed object was not registered.
                string str = "\tAsked > \'" + aObjName + "\' : 0 :" + NEWLINE + oPrintString("\t");
                throw new OhError("The appointed object was not registered.", str);
            }
            return obj;
        }



        /// <summary> 
        /// \IuWFNg(\vf)擾܂B<p/>
        /// Acquires a composition object.
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgAIuWFNgԍw肵āA_objectArray \vf擾܂B
        /// OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
        /// ł͕KG[sȂ悤ɂĂ܂B
        /// hNXł
        /// <a href="OhSubSystem.html">OhSubSystem</a>,
        /// <a href="OhTotalSystem.html">OhTotalSystem</a>
        /// ŃI[o[Ch܂B
        ///  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
        /// ۃ\bhɂĂ܂B
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value
        /// </param>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <returns> an <var>OhObject</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OhObject getComponent(String aClassName, String aObjName, int aObjNum)
        {
            if (true)
            {
                string str = "\taClassName (arg) = " + aClassName + NEWLINE
                            + "aObjName (arg) = " + aObjName + NEWLINE
                            + "aObjNum (arg) = " + aObjNum + NEWLINE
                            + oPrintString("\t");

                throw new OhError("\"OhElemet\" cannot have any component.",str);
            }
        }


        /// <summary> 
        /// \IuWFNg(\vf)擾܂B<p/>
        ///  Acquires a composition object.
        /// At this time, the object number is set to 0.
        /// </summary>
        /// <remarks>
        /// NXAIuWFNgw肵āA_objectArray \vf擾܂B
        /// IuWFNgԍ 0 ɂȂ܂B
        /// OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
        /// ł͕KG[sȂ悤ɂĂ܂B
        /// hNXł
        /// <a href="OhSubSystem.html">OhSubSystem</a>,
        /// <a href="OhTotalSystem.html">OhTotalSystem</a>
        /// ŃI[o[Ch܂B
        ///  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
        /// ۃ\bhɂĂ܂B
        /// </remarks>
        /// 
        /// <param name="aClassName">a <var>String</var> value
        /// </param>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <returns> an <var>OhObject</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OhObject getComponent(String aClassName, String aObjName)
        {
            if (true)
            {
                string str = "\taClassName (arg) = " + aClassName + NEWLINE
                           + "aObjName (arg) = " + aObjName + NEWLINE
                           + "aObjNum (arg) = 0" + NEWLINE
                           + oPrintString("\t");

                throw new OhError("\"OhElemet\" cannot have any component.", str);
            }
        }


        /// <summary> 
        /// \IuWFNg(\vf)擾܂B<p/>
        /// Acquires a composition object.
        /// </summary>
        /// <remarks>
        /// IuWFNgAIuWFNgԍw肵āA_objectArray \vf擾܂B
        /// OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
        /// ł͕KG[sȂ悤ɂĂ܂B
        /// hNXł
        /// <a href="OhSubSystem.html">OhSubSystem</a>,
        /// <a href="OhTotalSystem.html">OhTotalSystem</a>
        /// ŃI[o[Ch܂B
        ///  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
        /// ۃ\bhɂĂ܂B
        /// </remarks>
        /// 
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <returns> an <var>Object</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getComponent(String aObjName, int aObjNum)
        {
            if (true)
            {
                StringWriter sw = new StringWriter();
                sw.Write("aObjName (arg) = " + aObjName + NEWLINE + "aObjNum (arg) = " + aObjNum + NEWLINE);
                oPrint(sw, "\t");
                String str = sw.ToString();
                sw.Close();
                throw new OhError("\"OhElemet\" cannot have any component.", str);
            }
        }


        /// <summary> 
        /// \IuWFNg(\vf)擾܂B<p/>
        /// Acquires a composition object.
        /// At this time, the object number is set to 0.
        /// </summary>
        /// <remarks>
        /// IuWFNgw肵āA_objectArray \vf擾܂B
        /// OhElement NX͍ŏPʂłvfȂ̂ŁA\vf͎Ă܂B
        /// ł͕KG[sȂ悤ɂĂ܂B
        /// hNXł
        /// <a href="OhSubSystem.html">OhSubSystem</a>,
        /// <a href="OhTotalSystem.html">OhTotalSystem</a>
        /// ŃI[o[Ch܂B
        ///  OhElement NXpĐVɃNX郆[UȂĂ悤ɁA
        /// ۃ\bhɂĂ܂B
        /// </remarks>
        /// <p/>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <returns> an <var>Object</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual Object getComponent(String aObjName)
        {
            if (true)
            {
                string str = "aObjName (arg) = " + aObjName + NEWLINE
                           + "aObjNum (arg) = 0" + NEWLINE
                           + oPrintString("\t");
                throw new OhError("\"OhElemet\" cannot have any component.", str);
            }
        }


        // 2. connection function

        /// <summary> 
        /// SĂ̏LM[q
        /// <a href="OhSendPort.html#doSwitch()">OhSendPort.doSwitch()</a>
        /// s܂B<p/>
        /// Orders the send ports of this element to execute doSwitch().
        /// </summary>
        /// <remarks>
        /// ̓Iɂ́Aȉs܂B
        /// 
        /// <ol>
        /// <li> MM[qz _dataPackArray 쐬B</li>
        /// <li> MM[q̐ _numberOfReceivers ɐݒ肷B</li>
        /// <li> _dataPackArray ̃mۂB</li>
        /// <li> _numberOfDataPacks ̒l -1  0 ɕύXB</li>
        /// </ol>
        /// 
        /// ̃\bh̎s͏LM[qڑANł܂B<p/>
        /// 
        /// <a href="OhSendPort.html#doSwitch()">OhSendPort.doSwitch()</a>
        /// is performed with all possession sending ports.
        /// </remarks>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void doSwitch()
        {
            //; (*15)
            int k = 0;

            // An unexchangeable thing is in receiving ports or relay ports.
            //
            for (k = 0; k < _numberOfReceivePorts; k++)
                ((OhPort)_receivePortArray[k]).doSwitch();

            // An unexchangeable thing is in sending ports or relay ports.
            //
            for (k = 0; k < _numberOfSendPorts; k++)
                ((OhPort)_sendPortArray[k]).doSwitch();

        }



        /// <summary> 
        /// vZƑO̊mFsȂ܂B<p/>
        /// Execute checks before calculation work. 
        /// </summary>
        /// <remarks>
        /// M[qAM[qf[^pbN𑗎M\ȏԂł邱ƂmF܂B<p/>
        /// 
        /// SM[q OhReceivePort.areYouReady ̖߂lAyёSM[q OhSendPort.areYouReady 
        /// ߂l true ł邱ƂmF܂B
        /// ߂l false ̒[q݂ꍇ false ԂA݂Ȃꍇ true Ԃ܂B<p/>
        /// 
        /// ̃\bh̖߂l true ̏ꍇAOhElement ͌vZsł܂B<p/>
        /// 
        /// false ̏ꍇ́Af[^pbNMłȂM[qA
        /// ܂̓f[^pbN𑗐MłȂM[q݂A
        ///  OhElement, M Element ŌvZsł܂B<p/>
        /// 
        /// ̏ꍇAڑƂɌ肪邩AOhSendPort  doSwitch() sĂȂ\܂B
        /// 
        /// <pre>
        /// [RETURN]
        /// Etrue --> S[q areYouReady ̖߂l true ̏ꍇB
        /// Efalse -> ߂l false ̒[q݂ꍇB
        /// </pre>
        /// <p/>
        /// 
        /// Checks to see sending ports and receiving ports are in the state
        /// which can send and receive the data packs.
        /// </remarks>
        /// 
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool areYouReady()
        {
            //; (*16)
            bool flag = true; // For judgment.
            int k = 0;

            // --- Check "_receivePort". ---

            bool bufFlag = true; //; For judgment.
            for (k = 0; k < _numberOfReceivePorts; k++)
                if (((OhPort)_receivePortArray[k]).areYouReady() != true)
                {
                    StreamWriter temp_writer;
                    temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
                    temp_writer.AutoFlush = true;
                    ((OhPort)_receivePortArray[k]).oPrint(temp_writer, "\t");
                    bufFlag = false;
                }

            if (bufFlag != true)
            {
                throw new OhError("Receive ports or relay ports are not connected.", oPrintString("\t"));
            }

            // --- Check "_sendPortArray". ---

            bufFlag = true; //; For judgment.

            for (k = 0; k < _numberOfSendPorts; k++)
                if (((OhPort)_sendPortArray[k]).areYouReady() != true)
                {
                    StreamWriter temp_writer2;
                    temp_writer2 = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
                    temp_writer2.AutoFlush = true;
                    ((OhPort)_sendPortArray[k]).oPrint(temp_writer2, "\t");
                    bufFlag = false;
                }

            if (bufFlag != true)
            {
                throw new OhError("Receive ports or relay ports are not connected.", oPrintString("\t"));
            }

            // --- Check "_objectArray". ---

            bufFlag = true; // For judgment.

            for (k = 0; k < _numberOfObjects; k++)
                if (_objectArray[k] == null)
                {
                    string str = "\t_objectArray[" + k + "] has not set." + NEWLINE + oPrintString("\t");
                    throw new OhError(str);
                }

            if (bufFlag != true)
            {
                throw new OhError(oPrintString("\t"));
            }

            return flag;
        }


        // 3. numerical initialization

        /// <summary> 
        /// f[^ _dataString  null ɐݒ肵܂B
        /// 
        /// <p/>
        /// Clear the data string.
        /// 
        /// </summary>
        public virtual void clearDataString()
        {
            //; (*18)
            _dataString = null;
        }

        /// <summary> 
        /// ̃Xg[ aSr sǂݍ݁A̍s𕶎ƂĕԂ܂B<p/>
        /// 
        /// Read a line from the input stream <b>aSr</b> and store the
        /// line into <b>aStr</b>. 
        /// </summary>
        /// <remarks>
        /// # Ŏn܂sA܂͋󔒍s̓RgsƌȂAXLbv܂B
        /// ܂A󔒂͍폜܂B<p/>
        /// 
        /// The lines which start with "#" or
        /// contain only white characters are regarded as comment lines
        /// and skipped.
        /// </remarks>
        /// <param name="aSr">a <var>OhStreamReader</var> value
        /// </param>
        /// <param name="aStr">a <var>String</var> value
        /// </param>
        /// <param name="aStockP">a <var>bool[]</var> value
        /// </param>
        /// <param name="aIntFlag">a <var>int[]</var> value
        /// </param>
        /// <returns> a <var>String</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual String getLine(OhStreamReader aSr, String aStr, bool[] aStockP, int[] aIntFlag)
        {
            try
            {
                String str = null;

                if (aStockP[0] == false)
                {
                    while (true)
                    {
                        if ((aStr = aSr.ReadLine()) == null)
                            return null;

                        if (aStr.Length == 0 || aStr.Substring(0, 1).Equals("#"))
                            continue;

                        str = aStr.Trim();

                        break;
                    }

                    aIntFlag[0] = str[0];
                }
                else
                {
                    aStockP[0] = false;
                    str = aStr.Trim();

                    aIntFlag[0] = str[0];
                }
            }
            catch (Exception e)
            {
                throw new OhError("Error in getLine()." + NEWLINE + oPrintString("\t"));
            }

            return aStr;
        }

        /// <summary> 
        /// p^ǂݍރ\bhłB
        /// `ς݂ŁA[U͐VKɒ`Kv͂܂B
        /// 
        /// <p/>
        /// Gets the parameter.
        /// 
        /// 
        /// </summary>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void getParameter(OhStreamReader aSr)
        {
            //; (*22)

            try
            {
                if (aSr.skipWhites() == -1)
                {
                    OhError e = new OhError("Parameter file error.");
                    throw e;
                }

                if ((char)aSr.Read() != '{')
                {
                    string str = "Now reading parameters for " + NEWLINE + oPrintString(" > ");
                    throw new OhError("{ should come first", str);
                }

                aSr.ReadLine();

                setParameter(aSr); // Abstract method call.


                if (aSr.skipWhites() == -1)
                {
                    OhError e = new OhError("Parameter file error.");
                    throw e;
                }

                if ((char)aSr.Read() != '}')
                {
                    string str = "Now reading parameters for " + NEWLINE + oPrintString(" > ");
                    throw new OhError("Should end with }", str);
                }

                aSr.ReadLine();
            }
            catch (IOException e)
            {
                throw new OhError("Error in parameter line.", oPrintString("\t"));
            }
        }



        /// <summary> 
        /// ԗʂǂݍރ\bhłB
        /// 
        /// <p/>
        /// Gets the initial state.
        /// 
        /// 
        /// </summary>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void getInitialState(OhStreamReader aSr)
        {
            //; (*23)
            try
            {
                if (aSr.skipWhites() == -1)
                {
                    OhError e = new OhError("Initial state file error.");
                    throw e;
                }

                if ((char)aSr.Read() != '{')
                {
                    string str = "Now reading parameters for " + NEWLINE + oPrintString(" > ");
                    throw new OhError("{ should come first", str);
                }

                aSr.ReadLine();

                setInitialState(aSr); // Abstract method call.

                if (aSr.skipWhites() == -1)
                {
                    OhError e = new OhError("Initial state file error.");
                    throw e;
                }

                if ((char)aSr.Read() != '}')
                {
                    string str = "Now reading parameters for " + NEWLINE + oPrintString(" > ");
                    throw new OhError("Should end with {", str);
                }

                aSr.ReadLine();
            }
            catch (IOException e)
            {
                throw new OhError("Error in initial state line.", oPrintString("\t"));
            }
        }


        /// <summary> 
        /// ڒʐM𗘗păp^Ȃǂ̏L邽߂̃\bhłB<p/>
        /// This is a method for sharing information, such as a parameter,
        /// using direct communication.
        /// </summary>
        /// <remarks>
        /// ̃\bh́AڒʐM𗘗păp^Ȃǂ̏L邽߂̃\bhłB
        /// lXȗp@l܂A
        /// Ⴆ΂IuWFNg̃p^𑼂̃IuWFNgłpꍇȂǂɗpł܂B
        /// ꂼꂪp^ setParameter() Őݒ肷̂ł͂ȂA
        /// ݒ肵p^lARs[Ă炤A
        /// 邢͂̃p^Ă炤AȂǂƂgl܂B
        /// LƂsȂꍇ true ԂĉB
        /// ƂsȂȂꍇ false ԂĉB<p/>
        /// ftHgł͒P true Ԃ悤ɒ`Ă܂B
        /// </remarks>
        /// <p/>
        /// 
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        public virtual bool shareInfo()
        {
            return true;
        }


        /// <summary> 
        /// ̃\bh doShareInfo() Ăяo\bhłB
        /// ߂l
        /// <a href="OperationFlag.html">OperationFlag</a>
        /// ^łB<p/>
        /// This is the method which calls 'doShareInfo()'.
        /// </summary>
        /// 
        /// <remarks>
        /// doShareInfo() ́A<p/>
        /// 
        /// doShareInfo() ܂ȂĂȂꍇA
        /// 
        /// <ol>
        /// <li> doShareInfo() ĂсA߂l true Ȃ _shareInfoDoneP  true ɐݒ肵A
        /// <a href="OperationFlag.html#E_DONE">OperationFlag.E_DONE</a> Ԃ܂B</li>
        /// <li> doShareInfo() ̖߂l false Ȃ 񋤗LƂłȂƂāA
        /// <a href="OperationFlag.html#E_UNDONE">OperationFlag.E_UNDONE</a> Ԃ܂B</li>
        /// </ol>
        /// 
        /// doShareInfo() łɂłꍇ
        /// <a href="OperationFlag.html#E_ALREADY">OperationFlag.E_ALREADY</a> Ԃ܂B
        /// </remarks>
        /// 
        /// <returns> an <var>OperationFlag</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OperationFlag doShareInfo()
        {
            //; (*24)
            if (_shareInfoDoneP == true)
                return OperationFlag.E_ALREADY;
            if (shareInfo() == true)
                return OperationFlag.E_UNDONE;

            _shareInfoDoneP = true;
            return OperationFlag.E_DONE;
        }


        /// <summary> 
        /// vfɐڑĂ邷ׂĂ̑M[q̃f[^Ɏ^Ăf[^pbNA
        /// Xg[ aSwijŎw肳Ăt@Cɏo܂B<p/>
        /// The data pack recorded on the data of all the sending ports connected to the element
        /// is written out to the file specified by stream 'aSw'.
        /// </summary>
        /// <remarks>
        /// ^̎d́AM[q̓̃\bh
        /// <a href="OhSendPort.html#saveSendPortStock()">OhSendPort.saveSendPortStock()</a>
        /// gčsȂ܂B<p/>
        /// 
        /// <a href="OhSubhtml">OhSubSystem</a>
        /// ł͍Ē`܂B
        /// 
        /// <p/>
        ///
        /// Work of inclusion is performed using method 'saveSendPortStock' of the same name of a sending port. <p/>
        /// 
        /// This method is re-defined by 'OhSubSystem'.
        /// </remarks>
        /// 
        /// <param name="aSw">a <var>StreamWriter</var> value<p/>
        /// oXg[
        /// </param>
        public virtual void saveSendPortStock(StreamWriter aSw)
        {
            //; (*25)
            if (_numberOfSendPorts <= 0)
                return;

            aSw.Write(_className + " " + _objectName + " " + _objectNumber + " (" + NEWLINE);

            for (int k = 0; k < _numberOfSendPorts; k++)
            {
                aSw.Write("(" + k + ") # sending ports number." + NEWLINE);
                ((OhSendPort)_sendPortArray[k]).saveSendPortStock(aSw);
            }

            aSw.Write(")" + NEWLINE);
        }


        /// <summary> 
        /// f[^ _dataString ǂ݂ƂAvfɐڑĂ鑗M[qɃf[^^܂B
        ///   
        /// <p/>
        /// The data character sequence '_dataString' is read
        /// and data is recorded on the sending port connected to the element.
        /// 
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void restoreSendPortStock()
        {
            //; (*26)
            int k;
            int m;
            String dStr = _dataString;

            try
            {
                OhStringReader br = new OhStringReader(dStr);

                for (int i = 0; i < _numberOfSendPorts; i++)
                {

                    k = Int32.Parse(br.readWord());

                    if (k != i)
                    {
                        throw new OhError(oPrintString("\t"));
                    }

                    m = Int32.Parse(br.readWord());

                    for (int j = 0; j < m; j++)
                    {
                        String dataPackStr = br.readWord() + " " + br.readWord() + " " + br.readWord();
                        ((OhSendPort)_sendPortArray[i]).sendByString(dataPackStr);
                    }
                }
            }
            catch (IOException e)
            {
                Console.WriteLine(e.StackTrace);
            }
            catch (FormatException e)
            {
                Console.WriteLine(e.StackTrace);
            }
        }


        // 4. time about a simulation

        /// <summary> 
        /// _targetTime, _fixedTime  _currentTime Ɉv郁\bhłB
        /// </summary>
        /// <remarks>
        /// vfԔvZsȂvffł́A
        /// setInitialState()  _currentTime ݒ肳ꂽ
        /// ̃\bhĂԕKv܂B
        /// </remarks>
        public virtual void setCur2Fix2Tar()
        {
            OhTimePoint crnTime = new OhTimePoint();
            crnTime.copy(_currentTime);
            _targetTime = crnTime;

            OhTimePoint crnTp = new OhTimePoint();
            crnTp.copy(_currentTime);
            _fixedTime = crnTp;
        }


        /// <summary> 
        /// _currentTime XV܂B
        /// 
        /// 
        /// </summary>
       public virtual void renewCurrentTime()
        {
            _currentTime.add(_timeStep);
        }


        /// <summary> 
        /// _currentTime  _fixedTime ɖ߂܂B
        /// 
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void rewindCurrentTime()
        {
            //; (*27)
            if (_currentTime.CompareTo(_fixedTime) <= 0)
            {
                throw new OhError("'_currentTime' not exceeds '_fixedTime.'", oPrintString(" > "));
            }

            _currentTime.copy(_fixedTime);
        }


        /// <summary> 
        /// _fixedTime  _currentTime ɐi߂܂B
        /// 
        /// <p/>
        /// Updates '_fixedTime'.
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void renewFixedTime()
        {
            //; (*28)
            if (_currentTime.CompareTo(_fixedTime) <= 0)
            {
                throw new OhError("'_fixedTime' exceeds '_currentTime.'", oPrintString(" > "));
            }

            _fixedTime.copy(_currentTime);
        }


        /// <summary> 
        /// ̃\bh OhElement ł͎g܂B<p/>
        /// This method is not used by OhElement.
        /// </summary>
        /// <remarks>
        /// ̃\bh OhElement ł͎g킸A
        /// <a href="OhSubSystem.html">OhSubSystem</a> ōĒ`Ďg܂B
        /// Ȃ̃NXŒ`ƂƁAOhElement ł\vfɑ΂ĂAOA
        /// ̃\bhĂԕKv邩łB
        /// ڍׂ́A
        /// <a href="OhSubSystem.html#initializeTargetTime()">OhSubSystem.initializeTargetTime()</a>
        /// QƂĉB
        /// 
        /// <p/>
        /// This method is not used by OhElement.
        /// It uses re-defining by
        /// <a href="OhSubSystem.html">OhSubSystem</a>.
        /// The reason for having defined this method is that
        /// it is necessary to call this method on principle also to the OhElement type composition element.
        /// </remarks>
        public virtual void initializeTargetTime()
        {
        }


        /// <summary> 
        /// vZڕW _targetTime XV܂B
        /// ɔvZԊԊuƂ܂B
        /// <p/>
        /// 
        /// Updates '_targetTime'.
        /// 
        /// 
        /// </summary>
        /// <param name="aT">a <var>long</var> value. Repeated calculation time interval.
        /// </param>
        public virtual void renewTargetTime(IOhTimeImmutable aT)
        {
            //; (*29)
            OhTime time = new OhTime(aT.Sec, aT.Frac);
            OhTimePoint tarTime = OhTimePoint.add(_targetTime, aT);
            if (tarTime.CompareTo(TerminalTime) > 0)
            {
                if ((OhOption.Default.TimeControlOption == TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME)
                    && (_targetTime.CompareTo(TerminalTime) < 0))
                {
                    time = OhTimePoint.sub(TerminalTime, _targetTime);
                }
            }
            _targetTime.add(time);
        }


        /// <summary> 
        /// vZڕWvZ܂B
        /// 
        /// <p/>
        /// Calculates repeated calculation target time.
        /// 
        /// 
        /// </summary>
        /// <returns> a <var>long</var> value
        /// </returns>
        public virtual OhTime calculateIterationStep()
        {
            //; (*30)
            throw new OhError("'_fixedTime' exceeds '_currentTime.'", oPrintString("\t"));
        }

        /// <summary> 
        /// ݎƌvZԊԊu̒lɏ܂B<p/>
        /// 
        /// <b>_currentTime</b> and <b>_timeStep</b> of
        /// <b>OhElement</b> are
        /// initialized to <b>aTp</b>.
        /// </summary>
        /// <remarks>
        /// _currentTime, _timeStep  aTp , 0 ɏ܂B<p/>
        /// 
        /// aTp  0 ȂA_currentTime ύX܂B<p/>
        /// 
        /// (Ref "OhSubSystem") g _currentTime 
        /// ƂɁÅe\vf _curentTime ݒ肵܂B̂ƂA
        /// \bh (Ref "OhSubSystem#initialTime()") gp܂B
        /// <p/>
        /// 
        /// TuVXe OhSubSystem 悤Ƃ鎞 tps 
        /// ƂƁA\vf _currentTime ́Aŏ tps ɐݒ肳
        /// ܂B̌A\vfgɑ΂āA _currentTime 
        /// 邽߂ɁAԃt@C^ꂽ 
        /// tpe łƂƁA OhElement::initialTime(tpe) 
        /// ΂܂B<p/>
        /// 
        /// ۂɂ́Atps  tpe ͓ł邱ƂƎv܂B
        /// ŁAOhElement.initialTime(OhTime) ̈ 
        /// OhTimePoint.ZERO łƂɂ́A_currentTime ̕ύXȗ
        /// 悤ɂĂ܂B̂߁Aԃt@C̒ŁA
        /// tpe w肷Ƃł́AOhTimePoint.ZERO Ӗf[^
        /// ƂłAt@C쐬ƂȕւɂȂ܂B
        /// </remarks>
        /// <param name="aTp">an <var>OhTimePoint</var> value
        /// </param>
        public virtual void initialTime(OhTimePoint aTp)
        {
            if (aTp.CompareTo(OhTimePoint.ZERO) > 0)
            {
                //
                // Note that _currentTime has been set to new
                // OhTimePoint() in the constructors of the OhElement
                // class. So _currentTime is not null.
                //
                _currentTime.copy(aTp);
            }
            _timeStep = new OhTime(); // same as OhTime(0, 0.0);
        }

        // ȉAo[W

        /// <summary> 
        /// _currentTime, _timeStep  aTp (arg), 0 ɏ true Ԃ܂B
        /// <p/>
        /// '_currentTime' and '_timeStep'of <var>OhElement</var> are initialized to 'aTp' and 0,
        /// and returns 'true'.
        /// </summary>
        /// <remarks>
        /// O _timeStep  0 ȏ̏ꍇA
        /// 
        /// <ul>
        /// <li> _currentTime, _timeStep  aTp, 0 ɐݒς݂̏ꍇA
        /// xbZ[Wo͌Atrue Ԃ܂B</li>
        /// <li> ȊȌꍇAG[bZ[Wo͌Afalse Ԃ܂B</li>
        /// </ul>
        /// </remarks>
        /// 
        /// <param name="aTp">an <var>OhTimePoint</var> value
        /// </param>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        //public virtual bool isInitialTime(OhTimePoint aTp)
        //{
        //    //; (*33)
        //    if (_timeStep >= 0)
        //    {
        //        if ((_currentTime.CompareTo(aTp)) == 0 && (_timeStep == 0))
        //        {
        //            StringWriter sw = new StringWriter();
        //            oPrint(sw, "\t");
        //            String str = sw.ToString();
        //            sw.Close();
        //            throw new OhError("'_fixedTime' exceeds '_currentTime.'", str);
        //        }

        //        StringWriter sw2 = new StringWriter();
        //        sw2.Write("\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE);
        //        sw2.Write("\taTp (arg) = " + aTp + NEWLINE);
        //        oPrint(sw2, "\t");
        //        String str2 = sw2.ToString();
        //        sw2.Close();
        //        throw new OhError("'_fixedTime' exceeds '_currentTime.'", str2);
        //    }

        //    _currentTime = aTp;
        //    _timeStep = 0L;

        //    return true;
        //}


        // 5. execution of calculation

        /// <summary> 
        /// ƂsȂ܂B<p/>
        /// 
        /// ɂāASf[^擾܂B
        /// 
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool initialWork()
        {
            //; (*34)
            if (initialOutput() == false)
                //; Throws "OhError".
                return false;

            setDataPackArray();

            return true;
        }


        /// <summary> 
        ///  initialWork() Iǂ𔻒f܂B<p/>
        /// </summary>
        /// <remarks>
        /// yƂ܂ȂĂȂꍇz
        /// 
        /// <ol>
        /// <li> łׂĂ̎M[q̃f[^擾ł΁A
        /// Ƃ\łƔfāAinitialWork() ĂсA
        /// _initialWorkdoneP  true ɐݒ肵
        /// <a href="OperationFlag.html#E_DONE">OperationFlag.E_DONE</a>
        /// Ԃ܂B</li>
        /// <li> ŏȂƂЂƂ̎M[q̃f[^擾łȂƂ́A
        /// Ƃs\łƔf
        /// <a href="OperationFlag.html#E_UNDONE">OperationFlag.E_UNDONE</a>
        /// Ԃ܂B</li>
        /// </ol>
        /// 
        /// yƂłɂłꍇz
        /// <ol>
        /// <li><a href="OperationFlag.html#E_ALREADY">OperationFlag.E_ALREADY</a>
        /// Ԃ܂B</li>
        /// </ol></remarks>
        /// 
        /// <returns> an <var>OperationFlag</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual OperationFlag doInitialWork()
        {
            //; (*35)
            if (_initialWorkDoneP == true)
                return OperationFlag.E_ALREADY;
            if (initialWork() != true)
                //; Throws "OhError".
                return OperationFlag.E_UNDONE;

            _initialWorkDoneP = true;
            return OperationFlag.E_DONE;
        }

       
        /// <summary> 
        /// canYouCalculate() ̌łB<p/>
        /// </summary>
        /// <remarks>
        /// getNecessaryTimeFrom(), getNecessaryTimeTo() Œ`鎞Ԃ̊Ԃ
        /// f[^擾łΌvZ\ƔfB
        /// IԂ̂RłB
        /// getNecessaryTimeFrom(), getNecessaryTimeTo() \bh́A
        /// [URɍĒ`ł܂AftHgł
        /// getNecessaryTimeFrom()  _currentTime ԂA
        /// getNecessaryTimeTo  _currentTime + _timeStep ̒lԂ܂B<p/>
        /// 
        /// f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
        /// ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
        /// 
        /// Ƃ΁A
        /// 
        /// <code>
        /// public bool canYouCalculate()
        /// {
        ///     return canYouCalculate0();
        /// }
        /// </code>
        /// 
        /// 
        /// ƒ`΂悢łB
        /// </remarks>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool canYouCalculate0()
        {
            //; (*36)
            bool reply = true;
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                // ------------ for loop start.
                IOhTimePointImmutable tFrom;
                IOhTimePointImmutable tTo;

                Object rp = _receivePortArray[k];

                tFrom = NecessaryTimeFrom((OhReceivePort)rp); //;  _currentTime ԂB
                tTo = NecessaryTimeTo((OhReceivePort)rp);

                if (((OhPort)rp).canYouGetData(tFrom, tTo) != true)
                {
                    ((OhPort)rp).calReq();
                    reply = false;
                }
            } // -------------------------------------------------------- for loop end.

            return reply;
        }


        /// <summary> 
        /// canYouCalculate() ̌łB<p/>
        /// </summary>
        /// 
        /// getNecessaryTimeFrom() Œ`f[^擾łΌvZ\Ɣf܂B
        /// ȉꍇ getNecessaryTimeTo() \bh`Ӗ͂܂Bj<p/>
        /// 
        /// f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
        /// ̓Iɂ́AM[q calReq() bZ[W𑗂܂B<p/>
        /// 
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool canYouCalculate1()
        {
            //; (*37)
            bool reply = true;
            int k = 0;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                // ------------ for loop start.
                IOhTimePointImmutable tFrom = new OhTimePoint();

                Object rp = _receivePortArray[k];

                tFrom = NecessaryTimeFrom((OhReceivePort)rp);


                if (((OhPort)rp).canYouGetData(tFrom, tFrom) != true)
                {
                    ((OhPort)rp).calReq();
                    reply = false;
                }
            } // -------------------------------------------------------- for loop end.

            return reply;
        }



        /// <summary> 
        /// Mf[^pbNzXV܂B<p/>
        /// Sends data packs. <p/>
        /// </summary>
        /// <remarks>
        /// SĂ̏LM[q
        /// <a href="OhSendPort.html#setDataPackArray()">OhSendPort.setDataPackArray()</a>
        /// pA
        /// Mf[^pbNzXV܂B<p/>
        /// 
        /// ɂAÂf[^pbNz͏AVf[^pbNz񂪍쐬܂B
        /// 
        /// <p/>
        /// 
        /// 'OhSendPort setDataPackArray' is used for all possession sending ports,
        /// and sending data pack arrangement is updated.
        /// Thereby, old data pack arrangement is eliminated and new data pack arrangement is created. <p/>
        /// 
        /// But, nothing is carried out in fact.
        /// </remarks>
        public virtual void setDataPackArray()
        {
            //; (*38)
            for (int k = 0; k < _numberOfSendPorts; k++)
            {
                ((OhPort)_sendPortArray[k]).setDataPackArray();
            }
        }



        /// <summary> 
        /// svf[^pbN܂B<p/>
        /// Deletion of an unnecessary datapacks.
        /// </summary>
        /// <remarks>
        /// SĂ̏LM[q
        /// <a href="OhReceivePort.html#reduceDataPackStock()">OhReceivePort.reduceDataPackStock()</a>
        /// pA
        /// MM[qɌvZI񍐂sȂ܂B<p/>
        /// 
        /// ɂAMM[qŕsvf[^pbNAf[^pbNz񂪍XV܂B
        /// </remarks>
        /// 
        public virtual void reduceDataPackStock()
        {
            //; (*39)
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
                ((OhPort)_receivePortArray[k]).reduceDataPackStock();

            for (k = 0; k < _numberOfSendPorts; k++)
                ((OhPort)_sendPortArray[k]).reduceDataPackStock();
        }



        /// <summary> Returns true.
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        public virtual bool printStatus()
        {
            return true;
        }


        /// <summary> Records status.
        /// 
        /// </summary>
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool recordStatus()
        {
            //; (*41)
            if (_dataString == null)
            {
                throw new OhError(oPrintString("\t"));
            }

            printStatus();

            return true;
        }


        /// <summary> 
        /// work() ̗łB<p/>
        /// Example for "work()".
        /// </summary>
        /// <remarks>
        /// _timeStep 𓾂ĂAcalculate() s܂B
        /// 
        /// <p/>
        /// 'Work' which performs 'calculate()' after obtaining '_timeStep'.
        /// </remarks>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool work0()
        {
            //; (*42)
            //; vZvȂA_currenetTime ԖڕWzĂΌvZȂB
            //;
            if (_calReqP == false && _currentTime.CompareTo(MediumTermTargetTime) >= 0)
                return false;

            bool calc = false;

            while (true)
            {

                // Setup of a time step.
                //
                _timeStep = calculateTimeStep();
                if (_timeStep.toDouble() <= 0.0)
                {
                    // Abstract method call. Throw "OhError".
                    if (_timeStep.toDouble() == 0.0)
                        return false;

                    string str = "\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE + oPrintString("\t");
                    throw new ArgumentException(str);
                }

                OhTimePoint crnTime = OhTimePoint.add(_currentTime, _timeStep);
                if (crnTime.CompareTo(TerminalTime) > 0)
                {
                    if (OhOption.Default.TimeControlOption == TimeControl.DO_NOT_GO_BEYOND_TERMINAL_TIME)
                    {
                        _timeStep = OhTimePoint.sub(TerminalTime, _currentTime);
                    }
                }

                // Check before calculation.
                //
                if (canYouCalculate() != true)
                {
                    //; Throw "OhCannotGetDataError".
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                // Calculation of 1 step.
                //
                if (calculate() == false)
                {
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                calc = true;
                _calReqP = false;

                // Update _currentTime.
                //
                _currentTime.add(_timeStep);
                if (_currentTime.CompareTo(MediumTermTargetTime) >= 0)
                    break;
            }

            // Process after calculation.
            //
            setDataPackArray();
            reduceDataPackStock();

            return true;
        }


        /// <summary> 
        /// work() ̗łB<p/>
        /// Example for "work()".
        /// </summary>
        /// <remarks>
        /// _timeStep ́Acalculate() ̓ŌvZB
        /// 
        /// <p/>
        /// '_timeStep' is calculated inside 'calculate()'.
        /// </remarks>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool work1()
        {
            //; (*43)
            //;  vZvȂAcurrenet_time ԖڕWzĂΌvZȂB
            //;
            if (_calReqP == false && _currentTime.CompareTo(MediumTermTargetTime) >= 0)
                return false;

            bool calc = false;

            while (true)
            {
                bool flgDbg;

                // Check before calculation.
                //
                if ((flgDbg = canYouCalculate()) != true)
                {
                    //; Throw "OhCannotGetDataError".
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                // Calculation of 1 step.
                //
                if (calculate() != false)
                {
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                calc = true;
                _calReqP = false;

                // Update _currentTime.
                //
                if (_timeStep.toDouble() <= 0.0)
                    break;

                _currentTime.add(_timeStep);

                if (_currentTime.CompareTo(MediumTermTargetTime) >= 0)
                    break;
            }

            // Process after calculation.
            //
            setDataPackArray();
            reduceDataPackStock();

            return true;
        }


        /// <summary> 
        /// vZvfp work() ̗łB
        /// <p/>
        /// For the element which carries out repeated calculation.
        /// 
        /// </summary>
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual bool work2()
        {
            //; (*44)
            if (_fixedTime.CompareTo(TerminalTime) >= 0)
                // Finished calculation.
                return false;

            if (_currentTime.CompareTo(TerminalTime) >= 0)
                // Waiting other element.
                return false;

            bool calc = false;


            while (true)
            {

                // Setup of a time step.
                //
                _timeStep = calculateTimeStep();
                if (_timeStep.toDouble() <= 0.0)
                {
                    // Abstract methos call.
                    if (_timeStep.toDouble() == 0.0)
                        return false;

                    string str = "\t_currentTime = " + _currentTime + ", _timeStep = " + _timeStep + NEWLINE + oPrintString("\t");
                    throw new ArgumentException(str);
                }

                // Make not to exceed '_targetTime'.
                //
                OhTimePoint crnTime = OhTimePoint.add(_currentTime, _timeStep);
                if (crnTime.CompareTo(_targetTime) > 0)
                    _timeStep = OhTimePoint.sub(_targetTime, _currentTime);

                // Check before calculation.
                //
                if (canYouCalculate() != true)
                {
                    //; Throw "OhCannotGetDataError".
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                // Calculation of 1 step.
                //
                if (calculate() != false)
                {
                    if (calc == false)
                        return false;
                    break; // Finish without calculating.
                }

                calc = true;

                // Update _currentTime.
                //
                _currentTime.add(_timeStep);
                if (_currentTime.CompareTo(_targetTime) >= 0)
                    break;
            }

            // Process after calculation.
            //
            return true;
        }

        /// <summary>
        /// vZ _targetTime ɓBǂ𔻒f܂B
        /// <p/>
        /// Judges whether repeated calculation reached '_targetTime'.
        /// </summary>
        /// 
        /// <remarks>
        /// BĂ trueABĂȂ false Ԃ܂B
        /// </remarks>
        /// 
        /// 
        /// <returns> a <var>bool</var> value
        /// </returns>
        public virtual bool hasReachedTargetTime()
        {
            //; (*46)
            if (_currentTime.CompareTo(_targetTime) >= 0)
                return true;
            return false;
        }


        /// <summary> Convergence of the repeated calculation between elements is
        /// judged. Please be sure to re-define by the element which
        /// performs repeated calculation. In the function, please
        /// return 'true' if it converges. When not giving a
        /// re-definition, an error message is sent and it exits.
        /// 
        /// 
        /// <p/>
        /// vfԂ̔vZ̎𔻒肵܂BvZsȂvf
        /// KĒ`ĉB trueAĂȂ 
        /// false ԂĉBĒ`Ȃꍇ̓G[bZ[W
        /// o exit ܂B
        ///   
        /// 
        /// </summary>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        public virtual bool hasConverged()
        {
            throw new OhError(oPrintString("\t"));
        }
       
        /// <summary> 
        /// ڕW܂ł̃f[^̏o͂ƁAf[^pbN̏郁\bhłB
        /// 
        /// <p/>
        /// It is the method which carries out output of the data by repetitive target time,
        /// and processing of a data pack.
        /// 
        /// </summary>
        public virtual void flushData()
        {
            //; (*48)
            sendStockedData();

            setDataPackArray();

            reduceDataPackStock();
        }


        /// <summary> 
        /// ڕW܂łɂ܂f[^o͂郁\bhłB<p/>
        /// It is the method which outputs the data which accumulated by repetitive target time.
        /// </summary>
        /// 
        /// <remarks>
        /// o͒[q́AႦ
        /// <a href="OhSendDouble.html#sendData(OhTimePoint, double)">OhSendDouble.sendData(OhTimePoint, double)</a>
        /// Ȃǂ𗘗pĉB
        /// vZsȂvfŕKĒ`ĉB
        /// Ē`Ȃꍇ̓G[bZ[Wo exit ܂B
        /// 
        /// <p/>
        /// For example, please use 'OhSendDouble sendData(OhTimePoint aT, double aValue)'. <p/>
        /// 
        /// Please be sure to re-define by the element which performs repeated calculation.
        /// When not giving a re-definition, an error message is sent, and it exits.
        /// </remarks>
        public virtual void sendStockedData()
        {
            //; (*49)
            throw new OhError(oPrintString("\t"));
        }


        /// <summary> 
        /// ̗vfˑĂ鑼̗vf֌vZv`B܂B
        /// </summary>
        /// <remarks>
        /// ̗vfvZ邱ƂɂȂA
        /// KvƂȂf[^Mł邩`FbN郁\bh canYouCalculate() s邱ƂŁA
        /// ̗vfˑĂ鑼̗vf֌vZv`B܂B<p/>
        /// 
        /// \bh canYouCalculate() sŁA
        /// ̗vfւ̌vZvƂL^܂B
        /// ̗vf work() ł̗vl܂B<p/>
        /// 
        /// ̃\bh́A̗vf̑M[qĂ΂܂B
        /// nł́Ã\bhĂ΂邱Ƃ͂܂B
        /// p[q́AvZv𑗐M[qɓ]邾ŁA
        /// p[q̏L҂ł镔nɂ͌vZv𑗂ȂłB<p/>
        /// 
        /// ̒`sK؂łꍇ́ApNXōĒ`ĂB 
        /// </remarks>
        /// 
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void calReq()
        {
            OhTime timeStepBackup = _timeStep;
            _timeStep = calculateTimeStep(); // Abstract method call.
            //; throws "OhError".
            canYouCalculate(); //; Throw "OhCannotGetDataError" and "OhError".
            _timeStep = timeStepBackup;
            _calReqP = true;
        }


        // 6. information output function

        /// <summary> 
        /// LIuWFNg̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession objects.
        /// 
        /// </summary>
        /// <param name="aSw">a <var>StreamWriter</var> value
        /// </param>
        /// <param name="aHead">a <var>String</var> value
        /// </param>
        /// <param name="aTitle">a <var>String</var> value
        /// </param>
        /// <param name="aPortArray">an <var>Object[]</var> value
        /// </param>
        /// <param name="aNumOfObjs">an <var>int</var> value
        /// </param>
        public virtual void printObjects(StreamWriter aSw, String aHead, String aTitle, Object[] aPortArray, int aNumOfObjs)
        {
            if (aTitle != null)
            {
                aSw.Write(aHead + aTitle);
                oPrint(aSw, "of : ");
            }
            else
                oPrint(aSw, aHead);

            for (int k = 0; k < aNumOfObjs; k++)
            {
                aSw.Write(aHead);
                ((OhObject)aPortArray[k]).cPrint(aSw, "    ");
            }
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession receiving ports.
        /// 
        /// </summary>
        public virtual void printReceivePorts(StreamWriter aSw, String aHead)
        {
            printObjects(aSw, aHead, "receiving ports :", _receivePortArray, _numberOfReceivePorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession receiving ports.
        /// 
        /// </summary>
        public virtual void printReceivePorts(StreamWriter aSw)
        {
            printObjects(aSw, "    ", "receiving ports :", _receivePortArray, _numberOfReceivePorts);
        }


        /// <summary> 
        /// LM[q̏̏o
        /// 
        /// <p/>
        /// Information output on possession receiving ports.
        /// 
        /// </summary>
        public virtual void printReceivePorts(String aHead)
        {
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            StreamWriter pw = temp_writer;
            printObjects(pw, aHead, "receiving ports :", _receivePortArray, _numberOfReceivePorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession receiving ports.
        /// 
        /// </summary>
        public virtual void printReceivePorts()
        {
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            printObjects(temp_writer, "    ", "receiving ports :", _receivePortArray, _numberOfReceivePorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession sending ports.
        /// 
        /// </summary>
        public virtual void printSendPorts(StreamWriter aSw, String aHead)
        {
            printObjects(aSw, aHead, "sending ports :", _sendPortArray, _numberOfSendPorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession sending ports.
        /// 
        /// </summary>
        public virtual void printSendPorts(StreamWriter aSw)
        {
            printObjects(aSw, "    ", "sending ports :", _sendPortArray, _numberOfSendPorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession sending ports.
        /// 
        /// </summary>
        public virtual void printSendPorts(String aHead)
        {
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            StreamWriter pw = temp_writer;
            printObjects(pw, aHead, "sending ports :", _sendPortArray, _numberOfSendPorts);
        }


        /// <summary> 
        /// LM[q̏o͂܂B
        /// 
        /// <p/>
        /// Information output on possession sending ports.
        /// 
        /// </summary>
        public virtual void printSendPorts()
        {
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            StreamWriter pw = temp_writer;
            printObjects(pw, "    ", "sending ports :", _sendPortArray, _numberOfSendPorts);
        }


        /// <summary> 
        /// ڑo͂܂B
        /// 
        /// <p/>
        /// Information output on connection.
        /// 
        /// </summary>
        public virtual void printConnection(StreamWriter aSw, String aHead)
        {
            //; (*51-1)
            String str = aHead + "    ";
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_receivePortArray[k]).cPrint(aSw, aHead);
                ((OhSenderPort)_receivePortArray[k]).printConnection(aSw, str);
            }

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_sendPortArray[k]).cPrint(aSw, aHead);
                ((OhSenderPort)_sendPortArray[k]).printConnection(aSw, str);
            }
        }


        /// <summary> 
        /// ڑ̏o
        /// 
        /// <p/>
        /// Information output on connection.
        /// At this time, the header is "    ".
        /// 
        /// </summary>
        public virtual void printConnection(StreamWriter aSw)
        {
            //; (*51-2)
            String head = "    ";
            String str = "        ";
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_receivePortArray[k]).cPrint(aSw, head);
                ((OhSenderPort)_receivePortArray[k]).printConnection(aSw, str);
            }

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_sendPortArray[k]).cPrint(aSw, head);
                ((OhSenderPort)_sendPortArray[k]).printConnection(aSw, str);
            }
        }


        /// <summary> 
        /// ڑo͂܂B
        /// 
        /// <p/>
        /// Information output on connection.
        /// 
        /// </summary>
        public virtual void printConnection(String aHead)
        {
            //; (*51-3)
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            StreamWriter sw = temp_writer;
            String str = aHead + "    ";
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_receivePortArray[k]).cPrint(sw, aHead);
                ((OhSenderPort)_receivePortArray[k]).printConnection(sw, str);
            }

            for (k = 0; k < _numberOfSendPorts; k++)
            {
                ((OhSenderPort)_sendPortArray[k]).cPrint(sw, aHead);
                ((OhSenderPort)_sendPortArray[k]).printConnection(sw, str);
            }
        }


        /// <summary> 
        /// ڑo͂܂B
        /// 
        /// <p/>
        /// Information output on connection.
        /// 
        /// </summary>
        public virtual void printConnection()
        {
            //; (*51-4)
            StreamWriter temp_writer;
            temp_writer = new StreamWriter(Console.OpenStandardError(), Encoding.Default);
            temp_writer.AutoFlush = true;
            StreamWriter sw = temp_writer;
            String head = "    ";
            String str = "        ";
            int k;

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_receivePortArray[k]).cPrint(sw, head);
                ((OhSenderPort)_receivePortArray[k]).printConnection(sw, str);
            }

            for (k = 0; k < _numberOfReceivePorts; k++)
            {
                ((OhSenderPort)_sendPortArray[k]).cPrint(sw, head);
                ((OhSenderPort)_sendPortArray[k]).printConnection(sw, str);
            }
        }


        // 7. the function to save the last state at a file

        /// <summary> 
        /// ŏIԂۑ܂B
        /// 
        /// 
        /// </summary>
        /// <param name="aSw">a <var>StreamWriter</var> value
        /// </param>
        public virtual void terminalState(StreamWriter aSw)
        {
            //; (*52)
            aSw.Write(_className + " " + _objectName + " " + _objectNumber + NEWLINE);
            aSw.Write("{" + NEWLINE);

            saveTerminalState(aSw);

            aSw.Write(NEWLINE);
            aSw.Write("}" + NEWLINE);
        }


        // 8. addition

        /// <summary> 
        /// f̗vf쐬܂B<p/>
        /// Creates an Element which has the class name specified by
        /// the argument.
        /// </summary>
        /// <remarks>
        /// ɁAf̗vfƂȂ郂f\NX^^ƁA
        /// ̗vff̃IuWFNg𐶐܂BŎw肳ꂽNX
        /// pbP[W OhymosNet ɑ̂Ƃ܂B<p/>
        /// ̃\bh͍\`t@CpŠnf
        /// <a href ="OhymosNet.OhScfTotalSystem.html">OhScfTotalSystem</a>
        /// RXgN^Ŏgp܂B
        /// </remarks>
        /// <example>
        /// <code>
        /// // NX̊܂܂AZu
        /// OhAssembly a ;
        /// 
        /// // ^NX
        /// string className = "Storage";
        /// 
        /// // NX^擾
        /// Type t = a.GetType("OhymosNet." + className );
        /// 
        /// OhElement elm = OhElement.create(t);
        /// </code></example>
        /// 
        /// <param name="aType">a <var>Type</var> value<p/>
        /// f̗vf\^
        /// </param>
        /// <returns> an <var>OhElement</var> value<p/>
        /// f̗vfƂȂIuWFNg
        /// </returns>
        /// 
        public static OhElement create(Type aType)
        {
            OhElement e = (OhElement)Activator.CreateInstance(aType);
            return e;
        }

        

        // @end {methods} ------------------------------------------------------
    } // OhElement
}