/// <(#)>  OhError; OHyMoS.NET error exception class. </(#)>
/// <summary> Modified and tested by Naoto Ohshima. Date:2007/10/05 14:10:37
///                                          Last update:2008/02/26 14:50:05  
/// </summary>
using System;
using System.IO;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
	/// OHyMoS.NET ̗ONXłB<p/>
    /// OHyMoS.NET error exception class.
    /// OhError is a derived class of the Exception class. 
    /// </summary>
    /// 
    /// <remarks>
    /// O𓊂ɂ̓RXgN^OhErrorIuWFNg𐶐A
    /// throw܂BāA\bhA͌Ăяo\bh
    /// catch悤ɂ܂B<p/>
    /// ȉɗ܂B<p/>
    /// <p/>
    /// 
	/// The OhError class object constructor is used as
	/// 
	/// <code>
	/// .....
	/// try 
    /// {
	///     ...
	///     OhError e = new OhError(errorMessage1, errorMessage2);
	///     throw e;
	/// }
	/// catch (OhError e) 
    /// {
	///     System.Console.WriteLine(e.Message);
	/// }
	/// .....
	/// </code>
	/// 
	/// The example of the creation method of an error message is as follows.
	/// 
	/// <code>
	/// .....
	/// if(some error status was detected) 
    /// {
    ///     throw new OhError("Already exits \"_dataPackArray\".", oPrintString(" > "));
	/// }
	/// .....
	/// </code>
    /// </remarks>
	[Serializable]
	public class OhError:Exception
	{
        // @begin{field} -------------------------------------------------------
		
		/// <summary> <japanese>
		/// G[bZ[WłB
		/// </japanese>
		/// <p/>
		/// Error message.
		/// 
		/// </summary>
		private String _errorMessage;

        private String _errorMessage1;
        private String _errorMessage2;

		
		// @end{field} ---------------------------------------------------------
		
		
		// @begin{constructor} -------------------------------------------------
		
		/// <summary> <japanese>
		/// G[bZ[W aErrorMessage1 RXgN^łB
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aErrorMessage1">a <var>String</var> value
		/// </param>
		public OhError(String aErrorMessage1):base(aErrorMessage1)
		{
			_errorMessage1 = aErrorMessage1;
		}
		
		/// <summary> <japanese>
		/// 1sڂ aErrorMessage1A2sڂ aErrorMessage2 ̃G[bZ[WRXgN^łB
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aErrorMessage1">a <var>String</var> value
		/// </param>
		/// <param name="aErrorMessage2">a <var>String</var> value
		/// </param>
		public OhError(String aErrorMessage1, String aErrorMessage2)
            :base(aErrorMessage1 + Environment.NewLine + aErrorMessage2)
		{
            _errorMessage1 = aErrorMessage1;
            _errorMessage2 = aErrorMessage2;
		}
		
		// @end{constructor} ---------------------------------------------------
		
		
		// @begin{property} -----------------------------------------------------

        /// <summary> <japanese>
		/// G[bZ[W _errorMessage 擾܂B
		/// </japanese>
		/// <p/>
		/// <ul>
		/// get: a <var>String</var> value
        /// </ul>
        /// </summary>
        public string ErrorMessage
        {
            get { return base.Message; }
        }
        // @end{property} -------------------------------------------------------

        // @begin{method} -------------------------------------------------------
		/// <summary> <japanese>
		/// G[bZ[W _errorMessage ̊es̐擪ɈɂƂ aPrefix tԂ܂B
		/// </japanese>
		/// <p/>
		/// Returns the message which is made by adding 'aPrefix' before each line of the message
		/// that was given when the exception object was thrown.
		/// 
		/// </summary>
		/// <param name="aPrefix">a <var>String</var> value
		/// </param>
		/// <returns> a <var>String</var> value
		/// </returns>
        public String what(String aPrefix)
        {
            try
            {
                if (_errorMessage2 == null)
                {
                    _errorMessage = aPrefix + _errorMessage1;
                }
                else
                {
                    _errorMessage = aPrefix + _errorMessage1 + Environment.NewLine + aPrefix + _errorMessage2;
                }
            }

            catch (ArgumentException e)
            {
                Console.WriteLine(e.StackTrace);
            }
            
            return _errorMessage;
        }

        /// <summary>
        /// Writes the exception stack trace to the received stream
        /// </summary>
        /// <param name="throwable">Exception to obtain information from</param>
        /// <param name="stream">Output sream used to write to</param>
        public static void WriteStackTrace(Exception throwable, TextWriter stream)
        {
            stream.Write(throwable.StackTrace);
            stream.Flush();
        }
		
		// @end{methods} -------------------------------------------------------
	} // OhError
}