﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;

namespace OhymosNet
{
    /// <summary>
    /// PostgreSQLのデータベースからデータ値が double 型であるデータパックを読み取る入力端子を表すクラスです。
    /// このクラスは<a href ="OhymosNet.OhInputDBPgSql.html">OhInputDBPgSql</a>を
    /// 継承しています。
    /// <p/>
    /// The OhInputDBPgSqlDouble class is a class of input port which
    /// reads in datapacks of type double from an input table in PostgreSQL database and
    /// sends them. <p/>
    /// It extends <a href ="OhymosNet.OhInputDBPgSql.html">OhInputDBPgSql</a> class.<p/>
    /// </summary>
    /// <remarks>
    /// データベースからデータ記録時刻とデータ値を読み取り、データパックを作成します。
    /// 具体的には<a href ="OhymosNet.OhInputDBPgSqlDouble.html">makeDataPack(DataRow)</a>が
    /// テーブルの行データを与えられ、データパックへ変換します。
    /// </remarks>
    public class OhInputDBPgSqlDouble : OhInputDBPgSql
    {
        /// <summary>
        /// デフォルトコンストラクタです。
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// <a href="OhymosNet.OhInputDBPgSql.init_overload_1.html">OhInputDBPgSql.init(String, int, OhStreamReader)</a> 
        /// を実行しません。その結果、以下の設定となります。<p/>
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// デフォルトコンストラクタを使用する場合、
        /// <a href="OhymosNet.OhInputDBPgSql.init_overload_1.html">OhInputDBPgSql.init(String, int, OhStreamReader)</a> 
        /// などを用いてメンバ変数を初期化して下さい。
        /// </remarks>
        public OhInputDBPgSqlDouble()
            : base()
        {
            _className = "OhInputDBPgSqlDouble";
        }

        /// <summary>
        /// 行データからデータパックを作成します。
        /// </summary>
        /// <param name="aDr">行データ</param>
        /// <returns>a <var>OhDataPack</var> value<p/>
        /// 作成されたデータパック</returns>
        public override OhDataPack makeDataPack(DataRow aDr)
        {
            IOhTimePointImmutable dateTime = convertDateTime(aDr);
            double value = (double)aDr[_valueColumn];
            OhDataPackDouble dp = new OhDataPackDouble(dateTime, value);
            return (OhDataPack)dp;
        }

        /// <summary>
        /// データ型を取得します。ここでは"double"となります。
        /// </summary>
        public override string DataType
        {
            get { return "double"; }
        }

        }
}
