﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;

namespace OhymosNet
{
    /// <summary>
    /// SQLServer用入力端子モデルを表すクラスです。抽象クラスです。
    /// このクラスは<a href="OhymosNet.OhInputDB.html">OhInputDB</a>を継承しています。
    /// <p/>
    /// The OhInputDBSql class is an abstract class of port models of
    /// input DB for SQLServer. 
    /// It extends <a href="OhymosNet.OhInputDB.html">OhInputDB</a> class.<p/>
    /// </summary>
    /// <remarks>
    /// SQLServer用の入力端子モデルを表すクラスです。
    /// このクラスではデータベースファイルから読み取った情報と
    /// コマンドラインから入力されるユーザID、パスワードを基に、
    /// SQLServerデータベースに接続します。<p/>
    /// そして、データベースから取得したデータを保持する
    /// データセットオブジェクトを作成します。
    /// 
    /// </remarks>
    public abstract class OhInputDBSql : OhInputDB
    {
        /// <summary>
        /// デフォルトコンストラクタです。
        /// </summary>
        /// <remarks>
		/// デフォルトコンストラクタは
        /// <a href="OhymosNet.OhInputDBSql.init_overload_1.html">OhInputDBSql.init(String, int)</a> 
		/// を実行しません。その結果、以下の設定となります。<p/>
		/// <dl>
		/// <li> _objectName        : null</li>
		/// <li> _objectNumber      : -1</li>
		/// </dl>
		/// 
		/// <p/>
        /// デフォルトコンストラクタを使用する場合、
        /// <a href="OhymosNet.OhInputDBSql.init_overload_1.html">OhInputDBSql.init(String, int)</a> 
		/// を用いてメンバ変数を初期化して下さい。
        /// </remarks>
        public OhInputDBSql()
            : base()
        {
            _className = "OhInputDBSql";
        }

        /// <summary>
        /// テーブル名、オブジェクト番号を初期化します。
        /// </summary>
        /// <param name="aTableName">テーブル名</param>
        /// <param name="aTableNum">入力端子番号</param>
        public override void init(string aTableName, int aTableNum)
        {
            base.init(aTableName, aTableNum);
        }

        /// <summary>
        /// データベースに接続して、データセットを作成します。
        /// </summary>
        /// <remarks>
        /// まず、ユーザに認証方法を尋ねます。
        /// （基本的にWindows認証とした方がいいと思います）
        /// その後、dbファイルで指定された列
        /// （_timeColumns、_valueColumn）
        /// だけを選択し、
        /// その列で構成されるデータセットを作成します。
        /// </remarks>
        /// <returns>
        /// a <var>DataSet</var> value<p/>
        /// 作成したデータセット
        /// </returns>
        /// <exception cref="OhError">
        /// ユーザID、パスワード、またはdbファイルの内容が間違っている場合
        /// </exception>
        public override DataSet makeDataSet()
        {
            Console.WriteLine("サーバ\t\t{0}",_serverName);
            Console.WriteLine("データベース\t{0}",_databaseName);
            Console.WriteLine("テーブル\t{0}",_objectName);
            Console.WriteLine("に接続します");
            Console.WriteLine();
            
            
            // DataSetのインスタンスの作成
            DataSet ds = new DataSet();
            bool initFlag = false;
            while (true)
            {
                Console.Write("Windows認証を使用しますか？y or n" + NEWLINE + ">");

                string flag = Console.ReadLine();
                while (flag != "y" && flag != "n")
                {
                    flag = OhInterface.getReply("文字が無効です。正しく入力してください。y or n", "");
                }

                // データベース接続のための接続文字列
                SqlConnectionStringBuilder builder = new SqlConnectionStringBuilder();
                builder["Server"] = _serverName;
                builder["Database"] = _databaseName;
                builder["integrated Security"] = true; // trueならWindows認証を用いてDBに接続する

                if (flag == "n")
                {
                    builder["integrated Security"] = false;
                    Console.Write("データベースに接続するためのユーザーIDを入力してください" + NEWLINE + ">");
                    string userID = Console.ReadLine();
                    builder["User ID"] = userID;
                    Console.Write("パスワードを入力してください" + NEWLINE + ">");
                    string password = Console.ReadLine();
                    builder["Password"] = password;
                }

                string objectName;
                int objectNumber;
                OhStreamReader sr;
                if (initFlag)
                {
                    sr = new OhStreamReader(_dbFile);
                    objectName = _objectName;
                    objectNumber = _objectNumber;
                    _objectName = null;
                    _objectNumber = -1;
                    init(objectName, objectNumber, sr);
                }

               
                string connStr = builder.ConnectionString;

                // レコード取得のためのクエリ
                StringBuilder sb = new StringBuilder("select ");
                foreach (string str in _timeColumns)
                    sb.Append(str + ",");
                sb.Append(_valueColumn + " from " + _objectName);

                string sqlStr = sb.ToString();

                // SqlDataAdapterのインスタンスの作成
                SqlDataAdapter da = new SqlDataAdapter(sqlStr, connStr);
                try
                {
                    Console.WriteLine("Connecting the database...");

                    // 指定されたクエリを実行し、テーブルをDataSetオブジェクト内に格納
                    da.Fill(ds, _objectName);
                }

                catch (SqlException e)
                {
                    Console.WriteLine("ユーザーID、パスワード、またはdbファイルの内容が間違っています");
                    Console.WriteLine("ユーザーID、パスワード、またはdbファイルの内容を確認してください");
                    initFlag = true;
                    continue;
                }
                break;
            }
            return ds;
        }
    }
}
