﻿using System;
using System.Collections.Generic;
using System.Collections;
using System.Data;
using System.IO;
using System.Text;

namespace OhymosNet
{
    /// <summary>
    /// SQLServerのデータベースからデータ値が long 型であるデータパックを読み取る入力端子を表すクラスです。<p/>
    /// このクラスは<a href ="OhymosNet.OhInputDBSql.html">OhInputDBSql</a>を
    /// 継承しています。
    /// <p/>
    /// The OhInputDBSqlLong class is a class of input port which
    /// reads in datapacks of type long from an input table in PostgreSQL database and
    /// sends them. <p/>
    /// It extends <a href ="OhymosNet.OhInputDBSql.html">OhInputDBSql</a> class.<p/>
    /// </summary>
    /// <remarks>
    /// データベースからデータ記録時刻とデータ値を読み取り、データパックを作成します。
    /// 具体的には<a href ="OhymosNet.OhInputDBSqlLong.html">makeDataPack(DataRow)</a>が
    /// テーブルの行データを与えられ、データパックへ変換します。
    /// </remarks>
    public class OhInputDBSqlLong : OhInputDBSql
    {
        /// <summary>
        /// デフォルトコンストラクタです。
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// <a href="OhymosNet.OhInputDBSql.init_overload_1.html">OhInputDBSql.init(String, int, OhStreamReader)</a> 
        /// を実行しません。その結果、以下の設定となります。<p/>
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// デフォルトコンストラクタを使用する場合、
        /// <a href="OhymosNet.OhInputDBSql.init_overload_1.html">OhInputDBSql.init(String, int, OhStreamReader)</a> 
        /// などを用いてメンバ変数を初期化して下さい。
        /// </remarks>
        public OhInputDBSqlLong()
            : base()
        {
            _className = "OhInputDBSqlLong";
        }


        /// <summary>
        /// 行データからデータパックを作成します。
        /// </summary>
        /// <param name="aDr">行データ</param>
        /// <returns>a <var>OhDataPack</var> value<p/>
        /// 作成されたデータパック</returns>
        public override OhDataPack makeDataPack(DataRow aDr)
        {
            IOhTimePointImmutable dateTime = convertDateTime(aDr);
            long value = (long)aDr[_valueColumn];
            OhDataPackLong dp = new OhDataPackLong(dateTime, value);
            return (OhDataPack)dp;
        }

        /// <summary>
        /// データ型を取得します。ここでは"long"となります。
        /// </summary>
        public override string DataType
        {
            get { return "long"; }
        }

    }
}