/// <(#)>  OhInputFile; abstract class to express port models of input files. </(#)>
/// Modified and tested by Naoto Ohshima. Date:2007/12/02 15:17:09
/// Last update : 2008/02/26 19:42:38
/// 
using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
    /// ̓t@CpM[qf\NXłB
    /// ۃNXłB
    /// ̃NX<a href="OhymosNet.OhInputPort.html">OhInputPort</a>
    /// p܂B<p/>
    /// The OhInputFile class is an abstract class of port models of
    /// input files. It extends <a href="OhymosNet.OhInputPort.html">OhInputPort</a>.<p/>
    /// </summary>
    /// <remarks>
    /// ̓t@CpM[qf́AM[q݂̂vff
    /// łƉ߂邱Ƃł܂BȂ킿Ãf́At@
    /// Cf[^ǂݍ݁Af[^pbNɕϊđM[qɃf[
    /// ^pbNXgbN܂B<p/>
    /// 
    /// ۂɂ́A̓t@CpM[qf́Avff̂P
    /// ăfꂸɁAM[qf̔hfƂĎ
    /// Ă܂BȂ킿AM[qfɃt@Cǂݎ@\
    /// ǉāA̓t@CpM[qf쐬Ă܂B
    /// ̊֌Ŵ͎悤łB<p/>
    /// 
    /// NX OhInputFile ł́A<p/>
    /// 
    /// E̓t@C             : <a href="OhymosNet.OhObject._objectName.html">OhObject._objectName</a><br/>
    /// E̓t@C̍ŏIXV : _fileTimeAtInput <p/>
    /// 
    /// LA<p/>
    /// 
    /// E̓t@C̃f[^pbNM@\ <p/>
    /// 
    /// `܂B<p/>
    /// 
    /// p^t@C⏉lf[^t@CȂǂ́ANX
    /// <a href="OhymosNet.OhTotalSystem.html">OhTotalSystem</a> ̃IuWFNg
    /// <a href="OhymosNet.OhTotalSystem.readParameterFile.html">OhTotalSystem.readParameterFile()</a> 
    /// <a href="OhymosNet.OhTotalSystem.readInitialStateFile.html">OhTotalSystem.readInitialStateFile()</a> Ȃǂ
    /// pēǂݎƂ܂B
    /// āAp^⏉lȂǂ́A[qʂċ킯ł͂܂B
    /// OhInputFile ̓t@CƂ́A
    /// ԓIɕωf[^^Ăt@CłB<p/>
    /// 
    /// NX OhInputFile ̃\bh
    /// <a href="OhInputFile.inputData.html">OhInputFile.inputData()</a> ́A
    /// f[^Ԍ^zw肷si#&amp; dctVecŎn܂sj
    /// t@Cǂݎ@\Ă܂B
    /// 
    /// NX OhInputFile ̃\bh 
    /// <a href="OhInputFile.inputData.html">OhInputFile.inputData()</a>ǂݎf[^̗܂B
    /// <example>
    /// # rainfall.ia
    /// # ----------------------------------------------------------
    /// # Year(int) Sec(long) Frac(double) R(double, mm/hour)
    /// # DataConnectionType = 0:BROKENLINE 1:STEP 2:PULSE 3:NOINTP
    /// #&amp; dctvec 1 1
    /// #
    /// 
    /// 2000     0 0.0 0.0
    /// 2000  7200 0.0 5.0
    /// 2000 14400 0.0 0.0
    /// 2000 50000 0.0 0.0
    /// 
    /// <br/>
    /// s # Ŏn܂s̓RgsłB<p/>
    /// 
    /// As # ̒ &amp;s͈ӖĂāA
    /// ̉pPɂāÄӖقȂ܂B̗ŁA#&amp; dctvec 
    /// ƏĂs܂ÁAf[^Ԍ^zw
    /// ĂsłBf[^Ԍ^ɂẮANX
    /// <a href ="OhymosNet.DataConnectionType.html">DataConnectionType</a>QƂĂB
    /// ̗́Af[^1̃f[^ŁÃf[^Ԍ^́ASTEP Ȃ킿AKi
    /// łƎw肵Ă܂B<p/>
    /// 
    /// f[^̍ŏ3̃f[^́A<a href = "OhymosNet.OhTimePoint.html">OhTimePoint</a>^
    /// f[^Ńf[^\킵Ă܂B̌̃f[^́Af[^lŁA
    /// ̗ł́AOhTimePoint(2000, 0, 0.0) ̎̃f[^ĺA0.0 
    /// łB
    /// </example><p/>
    /// 
    /// OHyMoS.NET ł́A<a href ="OhymosNet.OhInputFile.html">OhInputFile</a>A<a href ="OhymosNet.OhInputFileDouble.html">OhInputFileDouble</a>
    /// Ȃǂpč쐬Ă܂BvO[UA
    /// <a href ="OhymosNet.OhInputFile.html">OhInputFile</a>pāA[UƎ̓͒[qNX
    /// 쐬邱Ƃł܂B<p/>
    /// 
    /// [UƎ̓͒[qf̒ۃNXhč쐬
    /// Ƃ́AΉf[^pbN^`ƂƂɁA̒
    /// NX<p/>
    /// 
    /// <code>
    /// public abstract String DataType{ get; };
    /// public abstract OhDataPack makeDataPack(String);
    /// </code>
    /// 
    /// 2̒ۃ\bhÃf[^pbN^ɑΉĒ`
    /// B<br/><br/>
    /// 
    /// An object of "OhInputFile" class has
	/// 
	/// <dl>
	/// <dt> (Object) _objectName</dt>
	/// <dd> input file name</dd>
	/// <dt> _fileTimeAtInput</dt>
	/// <dd> final update time of the input file</dd>
	/// </dl>
	/// 
	/// and
	/// 
	/// <dl><dd>
	/// function to read in data and send them as datapacks to the
	/// send port.
	/// </dd></dl>
    /// </remarks>
	public abstract class OhInputFile:OhInputPort
	{
		// @begin {instance variable or static variable} -------------
		
		/// <summary> 
        /// ̓t@C̍ŏIXVłB<p/>
        /// Final update time of the input file.
        /// </summary>
        /// <remarks>
		/// Vf[^擾ꂽƂɁAO̓̓t@Ct@
		/// Cɒǉ邩A㏑邩Iׂ悤ɑz肵Ă
		/// ܂B̂߂ɁAf[^ǂݍ񂾂ƂɁAt@C̍ŏI
		/// XVL^ĂAɃt@CǂޕKvƂ
		/// ɁAt@C̍ŏIXVāAt@CXVꂽ
		/// 悤ɂ܂B
        /// </remarks>
		protected internal long _fileTimeAtInput;
		
		// @end {instance variable or static variable} ---------------
		
		
		// @begin {abstract property and methods} ---------------------------------
		
		// 1. connection function
		
		/// <summary> Returns the string showing the type of the sending datapacks.
		/// 
		/// 
		/// <p/>
		/// Mf|^pbN^\Ԃ
		/// 
		/// 
		/// </summary>
		public abstract override string DataType{ get ; }
        
		// 2. sending function of the input file data
		
		/// <summary> 
        /// f[^f[^pbN쐬쐬܂B<p/>
        /// Makes datapacks from string data.
        /// </summary>
		/// <remarks>
		/// f[^f[^pbN쐬܂B
		/// ۃ\bhłB
		/// hNXŎ邱ƂvĂ܂B<p/>
		/// 
		/// Psf[^P̃f[^pbNɕϊ郁\bh
		/// hNXŒ`ĉB<p/>
		/// 
		/// WIȔhNXł́A
		/// Mf[^pbN^ɉĎ̎葱`ĉB<p/>
		/// 
        /// <ul>
        /// <li>(1) Mpf[^pbN̋L̈q[vɐB</li>
        /// <li>(2)  aLineData f[^L^Af[^lǂݎB</li>
        /// <li>(3) (2) œǂݎl (1) ̃f[^pbÑoɑB</li>
        /// <li>(4) (1) ̃f[^pbN Object ^ɃLXgĕԂB</li>
        /// </ul>
		/// 
		/// ۃ\bhȂ̂ŔhNXŕK`ĉB
		/// </remarks>
		/// 
		/// <param name="aLineData">a <var>String</var> value<p/>
        /// ̓f[^Ps̕
		/// </param>
		/// <returns> an <var>OhObject</var> value<p/>
        /// Mf[^pbN OhObject ^ɃLXg
		/// </returns>
		public abstract OhDataPack makeDataPack(String aLineData);
		
		// @end {abstract methods} -----------------------------------
		
		
		// @begin{initialization} ------------------------------------
		
		/// <summary> 
        /// IuWFNg(t@C) aFileNameAIuWFNgԍ 0 ɏ܂B<p/>
        /// Initialization.
        /// </summary>
        /// <remarks>
		/// 
		/// ۂɃt@CI[v̂́Ainit ̒ł͂ȂāA
		/// (Ref "#inputData()") ̒ɂĂłB
        /// </remarks>
		/// 
		/// <param name="aFileName">a <var>String</var> value<p/>
        /// t@C
		/// </param>
		/// <returns> a <var>bool</var> value<p/>
		/// </returns>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public virtual void init(String aFileName)
		{
			base.init(aFileName, 0);
		}

        
		// @end{initialization} --------------------------------------
		
		
		// @begin {constructors} -------------------------------------
		
		/// <summary> 
        /// ftHgRXgN^łB
        /// Creates a new default <var>OhInputFile</var> instance.
		/// 
		/// </summary>
        /// <remarks>ftHgRXgN^
        /// <a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> s܂B
        /// ̌ʁAȉ̐ݒƂȂ܂B<p/>
        /// 
        /// <ul>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// <li> _className         : "OhInputFile"</li>
        /// <li> _fileTimeAtInput   : Long.MinValue (lŏ long l)</li>
        /// </ul>
        /// 
        /// 
        /// <p/>
        /// ̑̃oϐɑ΂鏈͈RXgN^ƓłB
        /// ̏ꍇA<a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> 
        /// păoϐĉB
        /// </remarks>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public OhInputFile():base()
		{
			_fileTimeAtInput = Int64.MinValue;
			_className = "OhInputFile";
		}
		
		/// <summary> 
        /// IuWFNgit@Cjw肵 <var>OhInputFile</var> NX
        /// CX^X𐶐܂B<p/>
        /// Creates a new <var>OhInputFile</var> instance.
		/// </summary>
        /// <remarks>
        /// NX <a href="OhInputPort.html">OhInputPort</a> ̃ftHgRXgN^pA
        /// <a href="OhInputFile.html#init(String, int)">OhInputFile.init(String, int)</a> pāA
        /// oϐȉ̒ʂ菉܂B<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhInputFile"</li>
        /// <li> _objectName        : aFileName</li>
        /// <li> _objectNumber      : 0</li>
        /// <li> _owner             : null</li>
        /// <li> _bufferList        : ɊmۂB(ڑƗp)</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// <li> _dataPackArray     : null</li>
        /// <li> _numberOfDataPacks : -1</li>
        /// <li> _fileTimeAtInput   : Long.MinValue (lŏ long l)</li>
        /// </dl>
        /// </remarks>
		/// <param name="aFileName">a <var>String</var> value<p/>
        /// t@C
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public OhInputFile(String aFileName):base()
		{
			_fileTimeAtInput = Int64.MinValue;
			_className = "OhInputFile";
			_objectName = aFileName;
			_objectNumber = 0;
		}
		
		// @end {constructors} ---------------------------------------
		
		// @begin {methods} ------------------------------------------
		
		/// <summary> 
        /// [q͌vZ@\Ȃ̂ŁAvZvĂǂ傤ȂB
        /// KO <a href="OhCannotGetDataError.html">OhCannotGetDataError</a> 𓊂܂B
        /// <p/>
        /// When the input port receives a calculation request, throw
		/// (Ref "OhCannotGetDataError"). This will be caught in the catch
		/// block in OhTotalSystem.work().
		/// 
		/// </summary>
		/// <exception cref="OhCannotGetDataError">if an error occurs
		/// </exception>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void calReq()
		{
			String pauseComment = "Cannot get data from " + _objectName;
			
			OhInterface.pause(pauseComment);
			throw new OhCannotGetDataError(pauseComment);
		}
		
		/// <summary> 
        /// ̓̓t@C̏Xg[ <b>aSw</b> ɏo͂܂B
        /// o͂ƂɁAes̍sɁA <b>aHead</b> t
        /// ܂B<p/>
        /// <p/>
        /// 
        /// The information on this input file is printed to the
		/// specified PrintWriter <b>aSw</b> with the preceding String
		/// <b>aHead</b>
		/// </summary>
		/// <param name="aSw">a <var>StreamWriter</var> value<p/>
        /// IuWFNgoXg[
		/// </param>
		/// <param name="aHead">a <var>String</var> value<p/>
        /// esɕt镶
		/// </param>
		public override void  cPrint(StreamWriter aSw, String aHead)
		{
			aSw.WriteLine(aHead + _className + " " + _objectName);
		}
		
		/// <summary> 
        /// ̓t@CǂݎAf[^pbN쐬AM܂B
        /// </summary>
        /// <remarks>
        /// ̓t@CǂݎA
        /// <a href="OhInputFile.html#makeDataPack(String)">OhInputFile.makeDataPack(String)</a> ɂ
        /// f[^pbN쐬A쐬f[^pbN
        /// <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> 
        /// pđM܂(OhInputFile  OhSendPort ̈ł)B<p/>
        /// 
        /// t@CǂݎƂAs # ̏ꍇA
        /// ̍sRgsƌȂ܂B<p/>
        /// 
        /// As # łǍ &amp;ƁAȂ킿 #&amp;Ŏn܂śA
        /// Rgł͂ȂAȃf[^LڂĂ̂Ƃ܂B<p/>
        /// 
        /// ݁i2004/01/16ĵƂA
        /// <a href="DataConnectionType.html">DataConnectionType</a> Vector w肷ꍇ邾łB
        /// 
        /// <pre><code>
        /// # DataConnectionType = {0:BROKENLINE, 1:STEP, 2:PULSE, 3:NOINTP}
        /// #&amp; dctvec 2 0 1
        /// </code></pre>
        ///  
        /// ̂悤ɏĂƁA<a href="DataConnectionType.html">DataConnectionType</a> Vector 
        /// w肷̂ƌȂ܂B
        /// ̗́Af[^QāAP BROKENLINE A
        /// Q STEP ł邱ƂӖ܂B
        /// wȂʂ̏ꍇ́A
        /// 
        /// <pre><code>
        /// #&amp; dctvec 1 0
        /// </code></pre>
        /// 
        /// Ƃ̂ƓłB<p/>
        /// 
        /// t@C܂ŏIA
        /// <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> 
        /// p _dataPackArray 쐬܂B<p/>
        /// 
        /// A̓t@C̍ŏIύX _fileTimeAtInput ȑȌꍇ͉ return ܂B
        /// RXgN^ _fileTimeAtInput  Long.MIN_VALUE ɐݒ肳̂
        /// ͕̓͂Ks܂B
        /// </remarks>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
        public override void inputData()
        {

            setDataPackArray();

            reduceDataPackStock(); // Delete the unnecessary datapacks.

            FileInfo f = new FileInfo(_objectName);
            // Checks work directory by "System.getProperty("user.dir")".

            if (File.Exists(f.FullName) != true)
            {
                throw new OhError("Cannot find the file \"" + _objectName + "\"");
            }

            if (f.LastWriteTime.Ticks <= _fileTimeAtInput)
                return;

            _fileTimeAtInput = f.LastWriteTime.Ticks;


            StreamReader sr = null;
            String lineData = null;
            //Regex regex = new Regex("(\\w+) ");
            //Match m = regex.Match(lineData);
            // pattern to match a keyword

            try
            {
                sr = new StreamReader(_objectName, Encoding.Default);
                //; Do not use OhBufferedReader(fr, true).

                while ((lineData = sr.ReadLine()) != null)
                {

                    if (lineData.Length <= 0)
                        continue;

                    if (lineData.Substring(0, 1).Equals(" ") || lineData.Substring(0, 1).Equals("\t"))
                    {
                        continue;
                    }

                    if (lineData.Substring(0, 1).Equals("#"))
                    {
                        //  start "if"
                        if (lineData.Length <= 1)
                            continue;

                        if (lineData.Substring(1, 1).Equals("&"))
                        {
                            parseSpecLine(lineData);
                        }

                        continue; // Skip comment lines.
                    } // --------------------------------------- end "if"

                    dataPackStock(makeDataPack(lineData));
                } // ----------------------------------------- end "while"
            }
            catch (IOException e)
            {
                e = new IOException("Cannot open " + _objectName + "." + NEWLINE + oPrintString("\t"));
                throw e;
            }
            finally
            {
                if (sr != null)
                {
                    try
                    {
                        sr.Close();
                    }
                    catch (IOException e)
                    {
                        e = new IOException("Cannot close " + _objectName + "." + NEWLINE + oPrintString("\t"));
                        throw e;
                    }
                }
            }

            setDataPackArray(); //; Set "_numberOfDataPacks = _dataPackArray.size();"

            if (NumberOfDataPacks > 0)
            {
                IOhTimePointImmutable tp = NewestDataTime;

                StringWriter sw = new StringWriter();
                sw.WriteLine("File: " + _objectName + "  newest data time : "
                    + tp.Year + " " + tp.Tsec + " " + tp.Frac);

                String str = sw.ToString();

                OhObject top = (OhObject)TopOwner;
                if (top != null && top is OhTotalSystem)
                {
                    // (Ref "OhTotalSystem#addNewestDataTime()")
                    ((OhTotalSystem)top).addNewestDataTime(str);
                }
            }
        }
       
		/// <summary> 
        /// ̓̓t@C̏Xg[ <b>aSw</b> ɏo͂܂B
        /// o͂ƂɁAes̍sɁA <b>aHead</b> t
        /// ܂B
        /// <p/>
        /// 
        /// The information on this input file is printed to the
		/// specified PrintWriter <b>aSw</b> with preceding String
		/// <b>aHead</b>
		/// </summary>
		/// <param name="aSw">a <var>StreamWriter</var> value<p/>
        /// ̓t@C̏ރXg[
		/// </param>
		/// <param name="aHead">a <var>String</var> value<p/>
		/// </param>
		public override void  oPrint(StreamWriter aSw, String aHead)
		{
			cPrint(aSw, aHead);
		}
		
		/// <summary> 
        /// ŗ^ꂽA"#&amp;" Ŏn܂s͂āAΉĒ`
        /// ꂽ OhInputFile IuWFNgɑ΂Ďs܂B
        /// </summary>
        /// <remarks>
        /// ǂ̂悤ȑ삪s邩́Ã\bh̎Ɉˑ
        /// ܂BOHyMoS.NET ̃vO~O[ÚAOhInputFile p
        /// TuNXŁÃ\bhĒ`āÃTuNX
        /// Ǝ̑`邱Ƃł܂B<p/>
        /// 
        /// ̃NXł́Af[^Ԍ^z
        /// ǂݍޑ삾`Ă܂B
        /// 
        /// 
        /// The parseSpecLine method analyzes the line which starts
		/// with "#&amp;" and executes the specified operation by the
		/// line. What operation will be done is specified in this
		/// method. You can redifine this method in subclasses derived
		/// from this OhInputFile class.<p/>
		/// 
		/// In this default implementation of the parseSpecLine()
		/// method, only the dctvec line as <p/>
		/// 
		/// <pre>
		/// #&amp; dctvec 2 1 0
		/// </pre>
		/// 
		/// are analyzed. This exmaple line specifies the
		/// DataConnectionType vector Vector[2] =
		/// { DataConnectionType.STEP, DataConnectionType.BROKENLINE }
		/// <p/>
        /// </remarks>
		/// <param name="aSpecLine">a <var>String</var> value<p/>
        /// ǂݍޓʂȍs
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public virtual void  parseSpecLine(String aSpecLine)
		{
			// aSpecLine sample
			// #& dctvec 2 1 0
			Regex regex = new Regex("(\\w+) ");
            // #&̕ǂݎ
			Match m = regex.Match(aSpecLine.Substring(3, aSpecLine.Length - 3)); 
			if (m.Success)
			{
				String keyword = m.Groups[1].ToString();
				String info = aSpecLine.Substring(3 + keyword.Length);
				if (keyword.ToLower().Equals("dctvec"))
				{
					DctVec = DataConnectionTypeOption.parseDctVec(info);
				}
				else
				{
					throw new OhError("OhError wad detected." + NEWLINE + 
                        " File: " + _objectName + "." + NEWLINE + 
                        " in:   " + aSpecLine + NEWLINE + 
                        " Unrecognized option: " + keyword + NEWLINE);
				}
			}
			else
			{
				throw new OhError("OhError wad detected." + NEWLINE + 
                    " File: " + _objectName + "." + NEWLINE 
                    + " Illegal option line: " + aSpecLine + NEWLINE);
			}
		}
		
		// @end {methods} --------------------------------------------
	} // OhInputFile
}