/// <(#)>  OhInputFileDouble: Derived from OhInputFile. </(#)>
/// Modified and Tested by Naoto Ohshima. Date:2007/12/01 2:05:06
/// Last update : 2008/02/26 19:42:19
using System;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
    /// t@Cf[^l double ^łf[^pbNǂݎ͒[q\NXłB
    /// ̃NX<a href ="OhymosNet.OhInputFile.html">OhInputFile</a>
    /// pĂ܂B<p/>
    /// 
    /// The OhInputFileDouble class is a class of input port which
	/// reads in datapacks of type double from an input file and
	/// sends them. 
    /// It extends <a href ="OhymosNet.OhInputFile.html">OhInputFile</a> class.<p/>
    /// </summary>
    /// <remarks>
    /// [qڑɂ́Aڑe[q̃f[^^ĂKv܂B
    /// āAOhInputFileDouble
    /// <ul>
    /// <li><a href="OhymosNet.OhReceiveDouble.html">OhReceiveDouble</a></li>
    /// <li><a href="OhymosNet.OhRelayDouble.html">OhRelayDouble</a></li>
    /// <li><a href="OhymosNet.OhOutputFileDouble.html">OhOutputFileDouble</a></li>
    /// </ul>
    /// Ɛڑ邱Ƃł܂B<p/>
    /// 
    /// ǂݎf[^͂P̃f[^pbNɂ1sƂɏA
    /// ̗̂悤ɁAXy[X܂̓^uŋ؂Ă̂Ƃ܂B
    ///  <code>
    /// 2003 120 0.3 1.2 2.3 4.5
    /// </code>
    /// Ⴄ`œǂݎꍇɂ́ÃNXpNX쐬A
    /// <a href="OhymosNet.OhInputFileDouble.makeDataPack.html">makeDataPack(string)</a>
    /// Ē`ĂB
    /// <br/><br/>
	/// OhInputFileDouble can be connected with
    /// <a href="OhymosNet.OhReceiveDouble.html">OhReceiveDouble</a>,
    /// <a href="OhymosNet.OhRelayDouble.html">OhRelayDouble</a>, and 
    /// <a href="OhymosNet.OhOutputFileDouble.html">OhOutputFileDouble</a>.
	/// 
	/// OhInputFileDouble reads an input file using "simple data
	/// format for input and output." <p/>
	/// 
	/// "Simple data format for input and output" is a format in
	/// which
	/// 
	/// .....
	/// all the data items for a datapack should be written in one
	/// line, and data items should be separated with a number of
	/// spaces or tabs.
	/// <code>
	/// 2003 120 0.3 1.2 2.3 4.5
    /// </code>
    /// ....
	/// 
	/// If you need different format, please make a derived class of
    /// OhInputFileDouble and redefine "<a href="OhymosNet.OhInputFileDouble.makeDataPack.html">makeDataPack(string)</a>".
    /// </remarks>
	public class OhInputFileDouble:OhInputFile
	{
		
		/// <summary> 
        /// ̓t@Cǂݎlɏ悸li搔jłB<p/>
        /// Multiplication factor to the value which is read in from
		/// the input file. 
        /// </summary>
        /// 
        /// <remarks>
        /// Ⴆ
        /// <br/><br/>
        /// _mulfactor = 2.0
        /// <br/><br/>
        /// ƂĂƁA̓t@Cǂݎl3.2Ƃ
        /// f[^l6.4Ɛݒ肳܂B<p/>
        /// ftHgł1.0i̓t@Cǂݎl̂܂܎gjƂĂ܂B<p/>
        /// 
        /// For example, assume that _mulfactor is
		/// equal to 2.0 and a numerical value 3.2 is read in from the
		/// input file, then the data value is set to 6.4.
        /// </remarks>
		protected internal double _mulfactor = 1.0; //; {
		
		// @begin{initialization} ------------------------------------
		
		/// <summary> 
        /// IuWFNg(t@C) aFileNameAIuWFNgԍ 0 ɏ܂B
        /// <p/>
        /// Initializ the object. This method assigns the argument
		/// aFileName to the instance variable
		/// (Ref "OhObject._className").
		/// 
		/// </summary>
		/// <param name="aFileName">a <var>String</var> value<p/>
        /// ̓t@C
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void init(String aFileName)
		{
			base.init(aFileName);
		}
		
		// @end{initialization} --------------------------------------
		
		
		// @begin {constructors} -------------------------------------
		
		/// <summary> 
        /// ftHgRXgN^łB
        /// <p/>
        /// Creates a new default <var>OhInputFileDouble</var>
		/// instance.
		/// </summary>
        /// <remarks>
        /// ftHgRXgN^
        /// <a href="OhInputFileDouble.html#init(java.lang.String, int)">OhInputFileDouble.init(String, int)</a>
        /// s܂B
        /// ̌ʁAȉ̐ݒƂȂ܂B<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// ̑̃oϐɑ΂鏈͈RXgN^ƓłB
        /// ̏ꍇA
        /// <a href="OhInputFileDouble.html#init(java.lang.String, int)">OhInputFileDouble.init(String, int)</a>
        /// păoϐĉB
        /// </remarks>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public OhInputFileDouble():base()
		{
			_className = "OhInputFileDouble";
		}
		
		
		/// <summary> Creates a new <var>OhInputFileDouble</var> instance.
		/// 
		/// </summary>
        /// <remarks>
        /// NX̃RXgN^
        /// <a href="OhInputFile.html#OhInputFile(java.lang.String)">OhInputFile.OhInputFile(String)</a>
        /// pāA
        /// oϐȉ̒ʂ菉܂B<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhInputFileDouble"</li>
        /// <li> _objectName        : aFileName</li>
        /// <li> _objectNumber      : 0</li>
        /// <li> _owner             : null</li>
        /// <li> _bufferList        : ɊmۂB(ڑƗp)</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// <li> _dataPackArray     : null</li>
        /// <li> _numberOfDataPacks : -1</li>
        /// <li> _fileTimeAtInput   : Long.MIN_VALUE (lŏ long l)</li>
        /// </dl>
        /// </remarks>
        /// 
		/// <param name="aFileName">a <var>String</var> value<p/>
        /// ̓t@C
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public OhInputFileDouble(String aFileName):base(aFileName)
		{
			_className = "OhInputFileDouble";
		}
		
		// @end {constructors} ---------------------------------------
		
		
		// @begin {property}-----------------------------------------------
		// 1. connection function.
		
		/// <summary> 
        /// ̓f[^pbÑf[^ľ^ double 𕶎 "double" 擾܂B<p/>
        /// 
        /// Gets the data pack type which expresses the type of data
		/// to be received.
		/// 
		/// </summary>
		public override String DataType
		{
            get { return "double"; }
		}
        // @end {property}-----------------------------------------------

        // @begin {methods} ------------------------------------------
		
		// 2. data input function
		
		/// <summary> 
        ///  aLineData Ɋi[Ăf[^pbN񂩂f[^
        /// pbN쐬AOhDataPack ^ƂĖ߂܂B<p/>
        /// 
        /// Reads string datapacks of type double, creates datapacks of
		/// type double, and returns it.
        /// </summary>
        /// 
        /// <remarks>
		/// ȉ̗ႾƁAf[^ OhTimePoint(2003, 12, 0.3)AɃf[^
		/// l 3.5 f[^pbN쐬܂B
        /// </remarks>
		/// <example>
        /// OhDataPack dataPack = makeDataPack("2003 12 0.3 3.5");<p/>
        /// </example>
        /// 
		/// <param name="aDataStr">a <var>String</var> value<p/>
        /// f[^pbN
		/// </param>
		/// <returns> an <var>OhDataPack</var> value<p/>
		/// </returns>
		public override OhDataPack makeDataPack(String aDataStr)
		{
			OhDataPackDouble dp = new OhDataPackDouble(aDataStr);
			dp.multiply(_mulfactor);
			return dp;
		}
		
		/// <summary> 
        /// ŗ^ꂽA"#&amp; " Ŏn܂s͂āAΉĒ`
        /// ꂽs܂B
        /// 
        /// The parseSpecLine method analyzes the line which starts
        /// with "#&amp;" and executes the specified operation by the
        /// line.
        /// </summary>
        /// <remarks>
        /// ̃\bh̒`́AX[p[NXŁA^Ă܂A
        /// ̃NXŃI[o[ChĂ܂B(Q: (Ref "OhInputFile#partsSpecLine()") ).
        /// <p/>
        /// 
        /// (Ref "OhInputFile#parseSpecLine()") Œ`Ăf[^
        /// Ԍ^zǂݍޑɉāAf[^l萔{鑀
        /// `Ă܂B"#&amp; " ̌ multiply ƏśA
        /// ĂāAt@Cǎf[^lɂ̏搔
        /// |̂f[^lƂč̗p܂B<p/>
        /// <br/>
        /// This method is implemented in the super class
		/// (See: (Ref "OhInputFile#parseSpecLine()") ), but is
		/// overrided in this OhInputFileDouble class.<p/>
		/// 
		/// In this implementation of the parseSpecLine()
		/// method, the following multiplication specification<p/>
		/// 
		/// <code>
		/// #&amp; multiply 2.0
        /// </code>
		/// 
		/// is alse analyzed. This exmaple line specifies the
		/// multiplication factor to 2.0, that is, the data which are
		/// read in fromt this file are multiplied with 2.0.
		/// <p/>
        /// </remarks>
		/// <param name="aSpecLine">a <var>String</var> value<p/>
        /// ǂݎʂȍs
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  parseSpecLine(String aSpecLine)
		{
			// aSpecLine samples
			// #& dctvec 2 1 0
			// #& multiply 2.0
            Regex regex = new Regex("(\\w+) ");
            Match m = regex.Match(aSpecLine.Substring(3, aSpecLine.Length - 3));
            if (m.Success)
            {
                String keyword = m.Groups[1].ToString();
                String info = aSpecLine.Substring(3 + keyword.Length);
				if (keyword.ToLower().Equals("dctvec"))
				{
					DctVec = DataConnectionTypeOption.parseDctVec(info);
				}
				else if (keyword.ToLower().Equals("multiply"))
				{
					_mulfactor = Double.Parse(info);
				}
				else
				{
					throw new OhError("OhError wad detected." + NEWLINE
                        + " File: " + _objectName + "." + NEWLINE 
                        + " in:   " + aSpecLine + NEWLINE 
                        + " Unrecognized option: " + keyword + NEWLINE);
				}
			}
			else
			{
				throw new OhError("OhError wad detected." + NEWLINE 
                    + " File: " + _objectName + "." + NEWLINE 
                    + " Illegal option line: " + aSpecLine + NEWLINE);
			}
		}
		
		/// <summary> 
        ///   aStr Ɋi[Ăf[^pbN񂩂f[^pb
        /// N쐬A͒[qɎ^܂B<p/>
        /// 
        /// Stocks the datapack of type double which is stored in the
		/// form of string.
        /// </summary>
		/// <remarks>
        /// \bh̒` <a href="OhSendPort.html#dataPackStock()">OhSendPort.dataPackStock()</a> 
        /// gpĂ܂B<p/>
        /// </remarks>
		/// <param name="aDataStr">a <var>String</var> value<p/>
        /// f[^pbN
		/// </param>
		public override void  sendByString(String aDataStr)
		{
			OhDataPackDouble dp = new OhDataPackDouble(aDataStr);
			dp.multiply(_mulfactor);
			dataPackStock(dp);
		}
		
		// @end {methods} --------------------------------------------

        
	} // OhInputFileDouble
}