﻿/// <(#)>  OhOutputFileLong </(#)>
using System;
using System.IO;
using System.Text;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> 
    /// データ値が long 型であるデータパックをファイルに記録する出力端子を表すクラスです。 <p/>
    /// このクラスは<a href = "OhymosNet.OhOutputFile.html">OhOutputFile</a>を
    /// 継承しています。
    /// <p/>
    /// OhOutputFileLong;
    /// 
    /// OhOutputFileLong is a class of output ports
    /// which receive datapacks of type long and print them in a file. 
    /// 
    /// It extends <a href ="OhymosNet.OhOutputFile.html">OhOutputFile</a> class.<p/>
    /// </summary>
    /// <remarks>
    /// OhOutputFileLongはlong型のデータを受信するので、
    /// <a href="OhymosNet.OhSendLong.html">OhSendLong</a>,
    /// <a href="OhymosNet.OhRelayLong.html">OhRelayLong</a>, 
    /// <a href="OhymosNet.OhInputFileLong.html">OhInputFileLong</a>.
    /// と接続することができます。
    /// <br/><br/>
    /// OhOutputFileLong receives long data.
    /// OhOutputFileLong can be connected with
    /// <a href="OhymosNet.OhSendLong.html">OhSendLong</a>,
    /// <a href="OhymosNet.OhRelayLong.html">OhRelayLong</a>, and 
    /// <a href="OhymosNet.OhInputFileLong.html">OhInputFileLong</a>.
    /// </remarks>
    public class OhOutputFileLong : OhOutputFile
    {
        // @begin{initialization} ------------------------------------

        // @end{initialization} --------------------------------------


        // @begin{constructors} --------------------------------------

        /// <summary> 
        /// デフォルトコンストラクタです。
        /// Creates a new default <var>OhOutputFileLong</var> instance.
        /// 
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// 基底クラス <a href="OhOutputFile.html">OhOutputFile</a> のデフォルトコンストラクタしか実行しません。
        /// その結果、以下の設定となります。<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// <li> _lineLength        : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// その他のメンバ変数に対する処理は引数つきコンストラクタと同じです。
        /// この場合、
        /// <a href="OhOutputFileLong.html#init(java.lang.String, int)">OhOutputFileLong.init(String, int)</a> 
        /// を用いてメンバ変数を初期化して下さい。
        /// </remarks>
        public OhOutputFileLong()
            : base()
        {
            _className = "OhOutputFileLong";
        }


        /// <summary> 
        /// オブジェクト（出力ファイル）名、出力データ文字列の最大長さ(_lineLength)を指定し、
        /// 新しい<var>OhOutputFileLong</var>インスタンスを生成します。<p/> 
        /// Creates a new <var>OhOutputFileLong</var> instance.
        /// 
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhOutputFile.html#OhOutputFile(java.lang.String, int)">OhOutputFile.OhOutputFile(String, int)</a>
        /// を用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhOutputFileLong"</li>
        /// <li> _objectName        : aFileName</li>
        /// <li> _objectNumber      : 0</li>
        /// <li> _lineLength        : aLineLength</li>
        /// <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a></li>
        /// <li> _commentString     : null</li>
        /// </dl>
        /// </remarks>
        /// <param name="aFileName">a <var>String</var> value<p/>
        /// </param>
        /// <param name="aLineLength">an <var>int</var> value<p/>
        /// </param>
        public OhOutputFileLong(String aFileName, int aLineLength)
            : base(aFileName, aLineLength)
        {
            _className = "OhOutputFileLong";
        }

        /// <summary> 
        /// オブジェクト（出力ファイル）名を指定し、
        /// 新しい<var>OhOutputFileLong</var>インスタンスを生成します。 <p/>
        /// Creates a new <var>OhOutputFileLong</var> instance.
        /// 
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhOutputFile.html#OhOutputFile(java.lang.String, int)">OhOutputFile.OhOutputFile(String, int)</a>
        /// を用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhOutputFileLong"</li>
        /// <li> _objectName        : aFileName</li>
        /// <li> _objectNumber      : 0</li>
        /// <li> _lineLength        : 128</li>
        /// <li> _lastDataTime      : <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a></li>
        /// <li> _commentString     : null</li>
        /// </dl>
        /// </remarks>
        /// <param name="aFileName">a <var>String</var> value<p/>
        /// </param>
        public OhOutputFileLong(String aFileName)
            : base(aFileName, 128)
        {
            _className = "OhOutputFileLong";
        }

        // @end{constructors} ----------------------------------------


        // @begin{methods} -------------------------------------------

        // 1. Connection function.

        /// <summary> Returns the datapack type, "long", which expresses the
        /// type of data to be received.
        /// 
        /// </summary>
        /// <returns> a <var>String</var> value<p/>
        /// </returns>
        public override String DataType
        {
            get { return "long"; }
        }


        // 2. The write-in function of output files.

        /// <summary> Nothing is treated.
        /// 
        /// </summary>
        /// <param name="aStr">a <var>String</var> value<p/>
        /// </param>
        /// <param name="aDataPack">an <var>Object</var> value<p/>
        /// </param>
        //public override void  makeStringData(String aStr, OhDataPack aDataPack)
        //{
        //}

        // 再定義する必要ある？

        /// <summary> Though this method is defined in the base class
        /// (Ref "OhOutputFile")
        /// it is redifined in
        /// (Ref "OhOutputFileLong")
        /// 
        /// 
        /// <p/>
        /// 出力データの書き込み <p/>
        /// 
        /// まず、出力ファイル _objectName を追加書きモードでオープンし、
        /// オープンできない場合は return します。<p/>
        /// 
        /// 次に、受信データパック配列 _dataPackArray の先頭に格納され
        /// ているデータパックから順番に
        /// (Ref "OhDataPackLong#dpack2Ofile(PrintWriter)") を用いて
        /// 出力ファイルに記入します。<p/> ※注意 <p/>
        /// 
        /// 受信データパックはこのメソッドの中で取得されます。モデル 
        /// OhOutputFile は受信端子の派生モデルであって、データパックを
        /// ストックする機能はありません。
        /// 
        /// 
        /// </summary>
        /// <exception cref="IOException">if an error occurs
        /// </exception>
        //public override void  outputData()
        //{

        //    OhArrayList dpArray = new OhArrayList();
        //    OhArrayList[] dpArrayArray = new OhArrayList[]{dpArray}; //; For work.

        //    int numOfDps = 0;
        //    int[] numOfDpsArray = new int[]{numOfDps}; //; For work.

        //    getDataPackArray(dpArrayArray, numOfDpsArray);

        //    dpArray = dpArrayArray[0];
        //    numOfDps = numOfDpsArray[0];


        //    // _dataPackArray is not updated.
        //    //
        //    if (((OhDataPack) dpArray[numOfDps - 1]).getDataTime().compareTo(_lastDataTime) <= 0)
        //        return ;


        //    /* first time output */

        //    if (_lastDataTime == OhTimePoint.OhTimePointMin)
        //        writeHeader();

        //    ////////////////////////////////////////////////
        //    // Additional writing.
        //    //
        //    // 
        //    // 書き込み用出力ファイルの追加書きオープン。
        //    // 
        //    //
        //    StreamWriter bw = null;

        //    try
        //    {
        //        //UPGRADE_WARNING: 少なくとも 1 つの式が、ターゲット コードで 2 回以上使用されました。 "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1181'"
        //        //UPGRADE_TODO: コンストラクタ 'java.io.FileWriter.FileWriter' は、動作の異なる 'System.IO.StreamWriter' に変換されました。 "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javaioFileWriterFileWriter_javalangString_boolean'"
        //        //UPGRADE_TODO: Class 'java.io.FileWriter' は、動作の異なる 'System.IO.StreamWriter' に変換されました。 "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1073_javaioFileWriter'"
        //        bw = new StreamWriter(new StreamWriter(_objectName, true, Text.Encoding.Default).BaseStream, new StreamWriter(_objectName, true, Text.Encoding.Default).Encoding);
        //    }
        //    catch (IOException e)
        //    {
        //        throw e;
        //    }


        //    StreamWriter pw = new StreamWriter(bw.BaseStream, bw.Encoding);

        //    if (((OhDataPack) dpArray[0]).getDataTime().compareTo(OhTimePoint.OhTimePointMin) == 0)
        //    {
        //        ((OhDataPack) dpArray[0]).dpack2Ofile(pw);
        //        pw.Write(NEWLINE);
        //    }


        //    int k = 0;

        //    // Skip the data which have been output.
        //    //
        //    for (k = 0; k < numOfDps; k++)
        //    {
        //        if (((OhDataPack) dpArray[k]).getDataTime().compareTo(_lastDataTime) > 0)
        //            break;
        //    }


        //    /* output the data */

        //    for (; k < numOfDps; k++)
        //    {
        //        ((OhDataPack) dpArray[k]).dpack2Ofile(pw);
        //        pw.Write(NEWLINE);
        //    }

        //    //UPGRADE_NOTE: メソッド 'java.io.PrintWriter.close' に相当する .NET の処理よってスローされた例外は、異なる可能性があります。 "ms-help://MS.VSCC.v80/dv_commoner/local/redirect.htm?index='!DefaultContextWindowIndex'&keyword='jlca1099'"
        //    pw.Close();

        //    _lastDataTime = ((OhDataPack) dpArray[numOfDps - 1]).getDataTime();

        //    reduceDataPackStock();
        //}

        /// <summary>
        /// クラス名、データ型などの基本情報を文字列として返します。
        /// </summary>
        /// <returns>
        /// a <var>String</var> value<p/>
        /// クラスの基本情報
        /// </returns>
        public override String basicInfo()
        {
            return "classname: " + _className + NEWLINE
                 + "not an abstact class" + NEWLINE
                 + "datatype it records: " + DataType;
        }

        // @end{methods} ---------------------------------------------
    } // OhOutputFileLong
}