﻿// OhReceiveLong; class to express a port which receive long data. </(#)>
//                      Made from "OhReceiveLong.java" by Ohshima.
//                         Last update : 2008/02/27 11:47:33
//
using System;
using System.IO;
using System.Collections;
using OhymosNet;

namespace OhymosNet
{


    /// <summary> 
    /// データ値が long 型のデータパックを受信する受信端子を表すクラスです。
    /// 
    /// このクラスは<a href="OhymosNet.OhReceivePort.html">OhReceivePort</a> を継承しています。
    /// <p/>
    /// 
    /// OhReceiveLong;
    /// 
    /// OhReceiveLong is the base class to express a port which receive long data.
    /// It extends <a href="OhymosNet.OhReceivePort.html">OhReceivePort</a> class.
    /// </summary>
    public class OhReceiveLong : OhReceivePort
    {
        // @begin{initialization} ----------------------------------------------

        /// <summary> 
        /// オブジェクト（受信端子）名、オブジェクト（受信端子）番号を初期化します。
        /// 
        /// <p/>
        /// Initialization.
        /// 
        /// </summary>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 受信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 受信端子番号
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void init(String aPortName, int aPortNum)
        {
            base.init(aPortName, aPortNum);
        }

        // @end{initialization} ------------------------------------------------

        // @begin {constructors} -----------------------------------------------

        /// <summary> 
        /// デフォルトコンストラクタです。<p/>
        /// Creates a new <var>OhReceiveLong</var> instance.
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// <a href="OhReceiveLong.html#init(String, int)">OhReceiveLong.init(String, int)</a> を実行しません。
        /// その結果、以下の設定となります。<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// その他のメンバ変数に対する処理は引数つきコンストラクタと同じです。
        /// この場合、<a href="OhReceiveLong.html#init(String, int)">OhReceiveLong.init(String, int)</a> を
        /// 用いてメンバ変数を初期化して下さい。
        /// </remarks>
        public OhReceiveLong()
            : base()
        {
            _className = "OhReceiveLong";
        }

        /// <summary>
        /// 受信端子名と受信端子番号を指定して、
        /// <var>OhReceiveLong</var> クラスのインスタンスを生成します。<p/>
        /// Creates a new <var>OhReceiveLong</var> port instance by
        /// specifying the port name and the port number.
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhReceivePort.html#OhReceivePort.OhReceivePort(String, int)">OhReceivePort.OhReceivePort(String, int)</a>
        /// のコンストラクタを用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhReceiveLong"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _owner             : null</li>
        /// <li> _sender            : null</li>
        /// </dl>
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 受信端子名
        /// </param>
        /// <param name="aPortNumber">an <var>int</var> value<p/>
        /// 受信端子名
        /// </param>
        public OhReceiveLong(String aPortName, int aPortNumber)
            : base(aPortName, aPortNumber)
        {
            _className = "OhReceiveLong";
        }

        // @end {constructors} -------------------------------------------------

        // @begin {property} -------------------------------------------------
        /// <summary> 
        /// 受信するデータパックのデータ値の型 long を表す文字列 "long" を返します。
        /// 
        /// <p/>
        /// Returns the data pack type which expresses the type of data to be received.
        /// 
        /// </summary>
        /// <returns> a <var>String</var> value<p/>
        /// </returns>
        public override String DataType
        {
            //; (*5)
            get { return "long"; }
        }
        // @end {property} -------------------------------------------------


        // @begin {methods} ----------------------------------------------------


        /// <summary> 
        /// データ補間型を指定して、時刻 aT におけるデータ値を取得します。
        /// 
        /// <p/>
        /// Returns data value<p/> at time t.
        /// This data connection type is brokenline or step or pulse.
        /// 
        /// </summary>
        /// <param name="aT">an <var>OhTimePoint</var> value<p/>
        /// 取得するデータ値のデータ記録時刻
        /// </param>
        /// <param name="aDctVec">a <var>ArryaList</var> value<p/>
        /// データ補間配列リスト
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// </returns>
        public long getData(OhTimePoint aT, ArrayList aDctVec)
        {
            //; (*1-1)
            long dataValue = 0L;

            OhArrayList dataPackArray = null;
            int numberOfDataPacks = 0;

            OhArrayList[] dataPackArrayArray = new OhArrayList[] { dataPackArray }; //; Array for work.
            int[] numberOfDataPacksArray = new int[] { numberOfDataPacks };

            int ip = 0;

            getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

            dataPackArray = dataPackArrayArray[0];
            numberOfDataPacks = numberOfDataPacksArray[0];

            if (numberOfDataPacks <= 0)
            {
                throw new OhError("Error in Get_Data", oPrintString(" >"));
            }

            OhTimePoint ta = new OhTimePoint();
            ta.copy(((OhDataPackLong)dataPackArray[0]).DataTime);
            OhTimePoint tb = new OhTimePoint();
            tb.copy(((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataTime);

            // たぶんいらない

            //if (aT.CompareTo(ta) <= 0)
            //{
            //    // aT <= ta ?
            //    return ((OhDataPackLong) dataPackArray[0]).DataValue;
            //}

            //if (aT.CompareTo(tb) >= 0)
            //{
            //    // aT >= tb ?
            //    return ((OhDataPackLong) dataPackArray[numberOfDataPacks - 1]).DataValue;
            //}

            DataConnectionType dataConnectionType = (DataConnectionType)aDctVec[0];

            if (dataConnectionType == DataConnectionType.BROKENLINE)
            {
                if (aT.CompareTo(ta) <= 0)
                {
                    dataValue = ((OhDataPackLong)dataPackArray[0]).DataValue;
                }
                else if (aT.CompareTo(tb) >= 0)
                {
                    dataValue = ((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataValue;
                }
                else
                {
                    int ip1 = 0;
                    int ip2 = numberOfDataPacks - 1;

                    OhTimePoint t1 = new OhTimePoint();
                    t1.copy(((OhDataPackLong)dataPackArray[ip1]).DataTime);

                    OhTimePoint t2 = new OhTimePoint();
                    t2.copy(((OhDataPackLong)dataPackArray[ip2]).DataTime);

                    while (ip2 - ip1 > 1)
                    {
                        ip = (ip2 + ip1) / 2;
                        OhTimePoint tp = new OhTimePoint();
                        tp.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);

                        if (tp.CompareTo(aT) < 0)
                        {
                            ip1 = ip;
                            t1.copy(tp);
                        }
                        else if (tp.CompareTo(aT) > 0)
                        {
                            ip2 = ip;
                            t2.copy(tp);
                        }
                        else
                        {
                            return ((OhDataPackLong)dataPackArray[ip]).DataValue;
                        }
                    }

                    long y1 = ((OhDataPackLong)dataPackArray[ip1]).DataValue;

                    if (ip1 == ip2)
                    {
                        return y1;
                    }

                    long y2 = ((OhDataPackLong)dataPackArray[ip2]).DataValue;

                    OhTimePoint aTBackUp = new OhTimePoint();
                    aTBackUp.copy(aT);
                    //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
                    dataValue = (long)((OhTimePoint.sub(aTBackUp,t1).toDouble() * y2 + OhTimePoint.sub(t2,aTBackUp).toDouble() * y1) 
                                / OhTimePoint.sub(t2,t1).toDouble());
                }
            }
            else if (dataConnectionType == DataConnectionType.STEP)
            {
                if (aT.CompareTo(ta) <= 0)
                {
                    dataValue = ((OhDataPackLong)dataPackArray[0]).DataValue;
                }
                else if (aT.CompareTo(tb) >= 0)
                {
                    dataValue = ((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataValue;
                }
                else
                {
                    int ip1 = 0;
                    int ip2 = numberOfDataPacks - 1;

                    OhTimePoint t1 = new OhTimePoint();
                    t1.copy(((OhDataPackLong)dataPackArray[ip1]).DataTime);

                    OhTimePoint t2 = new OhTimePoint();
                    t2.copy(((OhDataPackLong)dataPackArray[ip2]).DataTime);

                    while (ip2 - ip1 > 1)
                    {
                        ip = (ip2 + ip1) / 2;

                        OhTimePoint tp = new OhTimePoint();
                        tp.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);

                        if (tp.CompareTo(aT) < 0)
                        {
                            ip1 = ip;
                            t1.copy(tp);
                        }
                        else if (tp.CompareTo(aT) > 0)
                        {
                            ip2 = ip;
                            t2.copy(tp);
                        }
                        else
                        {
                            return ((OhDataPackLong)dataPackArray[ip]).DataValue;
                        }
                    }
                    dataValue = ((OhDataPackLong)dataPackArray[ip2]).DataValue;
                }
            }
            else if (dataConnectionType == DataConnectionType.PULSE)
            {
                if (aT.CompareTo(ta) <= 0)
                {
                    dataValue = 0L;
                }
                else if (aT.CompareTo(tb) >= 0)
                {
                    dataValue = 0L;
                }
                else
                {
                    int ip1 = 0;
                    int ip2 = numberOfDataPacks - 1;

                    OhTimePoint t1 = new OhTimePoint();
                    t1.copy(((OhDataPackLong)dataPackArray[ip1]).DataTime);

                    OhTimePoint t2 = new OhTimePoint();
                    t2.copy(((OhDataPackLong)dataPackArray[ip2]).DataTime);

                    while (ip2 - ip1 > 1)
                    {
                        ip = (ip2 + ip1) / 2;

                        OhTimePoint tp = new OhTimePoint();
                        tp.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);

                        if (tp.CompareTo(aT) < 0)
                        {
                            ip1 = ip;
                            t1.copy(tp);
                        }
                        else if (tp.CompareTo(aT) > 0)
                        {
                            ip2 = ip;
                            t2.copy(tp);
                        }
                        else
                        {
                            return ((OhDataPackLong)dataPackArray[ip]).DataValue;
                        }
                    }
                    if (aT.CompareTo(t1) == 0)
                    {
                        dataValue = ((OhDataPackLong)dataPackArray[ip1]).DataValue;
                    }
                    else if (aT.CompareTo(t2) == 0)
                    {
                        dataValue = ((OhDataPackLong)dataPackArray[ip2]).DataValue;
                    }
                    else
                    {
                        dataValue = 0L;
                    }
                }
            }
            else
            {
                throw new OhError("Illegal data_connection_type.", oPrintString("> "));
            }

            return dataValue;
        }




        /// <summary> 
        /// 時刻 aT におけるデータ値を取得します。
        /// データ値の補間は折れ線とします。
        /// 
        /// <p/>
        /// Returns the data value<p/> at time t.
        /// This data connection type is brokenline.
        /// 
        /// </summary>
        /// <param name="aT">an <var>OhTimePoint</var> value<p/>
        /// 取得するデータ値のデータ記録時刻
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// データ値
        /// </returns>
        public long getData(OhTimePoint aT)
        {
            //; (*1-2)
            long dataValue = 0L;

            OhArrayList dataPackArray = null;
            int numberOfDataPacks = 0;

            OhArrayList[] dataPackArrayArray = new OhArrayList[] { dataPackArray }; //; Array for work.
            int[] numberOfDataPacksArray = new int[] { numberOfDataPacks };

            int ip = 0;

            getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

            dataPackArray = dataPackArrayArray[0];
            numberOfDataPacks = numberOfDataPacksArray[0];


            if (numberOfDataPacks <= 0)
            {
                throw new OhError("Error in Get_Data", oPrintString(" >"));
            }

            OhTimePoint ta = new OhTimePoint();
            ta.copy(((OhDataPackLong)dataPackArray[0]).DataTime);

            OhTimePoint tb = new OhTimePoint();
            tb.copy(((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataTime);


            if (aT.CompareTo(ta) <= 0)
            {
                // aT <= ta ?
                return ((OhDataPackLong)dataPackArray[0]).DataValue;
            }

            if (aT.CompareTo(tb) >= 0)
            {
                // aT >= tb ?
                return ((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataValue;
            }

            int ip1 = 0;
            int ip2 = numberOfDataPacks - 1;

            OhTimePoint t1 = new OhTimePoint();
            t1.copy(((OhDataPackLong)dataPackArray[ip1]).DataTime);

            OhTimePoint t2 = new OhTimePoint();
            t2.copy(((OhDataPackLong)dataPackArray[ip2]).DataTime);

            while (ip2 - ip1 > 1)
            {
                ip = (ip2 + ip1) / 2;

                OhTimePoint tp = new OhTimePoint();
                tp.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);

                if (tp.CompareTo(aT) < 0)
                {
                    ip1 = ip;
                    t1.copy(tp);
                }
                else if (tp.CompareTo(aT) > 0)
                {
                    ip2 = ip;
                    t2.copy(tp);
                }
                else
                {
                    return ((OhDataPackLong)dataPackArray[ip]).DataValue;
                }
            }

            long y1 = ((OhDataPackLong)dataPackArray[ip1]).DataValue;

            if (ip1 == ip2)
            {
                return y1;
            }

            long y2 = ((OhDataPackLong)dataPackArray[ip2]).DataValue;

            OhTimePoint aTBackUp = new OhTimePoint();
            aTBackUp.copy(aT);
            //;     v = ((aT-t1)*y2                  + (t2-aT)*y1)                  / (t2-t1);
            dataValue = (long) (((OhTimePoint.sub(aTBackUp, t1).toDouble() * y2 + OhTimePoint.sub(t2, aTBackUp).toDouble() * y1)) 
                        / OhTimePoint.sub(t2, t1).toDouble());
            

            return dataValue;
        }



        /// <summary> 
        /// データ補間型を指定して、時刻 aT におけるデータ値を取得し、long 型配列 aDataValue の第一要素に格納します。
        /// 
        /// <p/>
        /// Set the data value<p/> at time t to reference aDataValue[0].
        /// 
        /// </summary>
        /// <param name="aT">an <var>OhTimePoint</var> value<p/>
        /// 取得するデータ値のデータ記録時刻
        /// </param>
        /// <param name="aDataValue">a <var>long[]</var> value<p/>
        /// データ値を格納するlong型配列
        /// </param>
        /// <param name="aDctVec">a <var>ArrayList</var> value<p/>
        /// データ補間配列リスト
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public void setData(OhTimePoint aT, long[] aDataValue, ArrayList aDctVec)
        {
            //; (*4-1)
            aDataValue[0] = getData(aT, aDctVec);
        }


        /// <summary> 
        /// 時刻 aT におけるデータ値を取得し、long 型配列 aDataValue の第一要素に格納します。
        /// データ値の補間は折れ線とします。
        /// 
        /// <p/>
        /// Set the data value<p/> at time t to reference aDataValue[0].
        /// This data connection type is brokenline.
        /// 
        /// </summary>
        /// <param name="aT">an <var>OhTimePoint</var> value<p/>
        /// 取得するデータ値のデータ記録時刻
        /// </param>
        /// <param name="aDataValue">a <var>long[]</var> value<p/>
        /// データ値を格納するlong型配列
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public void setData(OhTimePoint aT, long[] aDataValue)
        {
            //; (*4-2)
            aDataValue[0] = getData(aT);
        }



        /// <summary> 
        /// データ補間型を指定して、時刻 aT1 から時刻 aT2 までの積分値を返します。
        /// 
        /// <p/>
        /// Returns the integrated value<p/> from time aT1 till time aT2.
        /// aT1 should be less than or equal to t2. <p/>
        /// </summary>
        /// <remarks>
        /// When data connection type is Pulse, the value<p/> summed from aT1 till
        /// aT2 will be returned. The pulse at time aT1 is not contained in the
        /// sum, and the pulse at time aT2 is contained in the sum.
        /// </remarks>
        /// 
        /// <param name="aT1">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aT2">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aDctVec">a <var>Vector</var> value<p/>
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// </returns>
        public long getDataSum(OhTimePoint aT1, OhTimePoint aT2, ArrayList aDctVec)
        {
            //(*3-1)
            long sum = 0L;

            if (aT1.CompareTo(aT2) == 0)
            {
                return sum;
            }

            if (aT1.CompareTo(aT2) > 0)
            {
                OhTimePoint aT3 = aT1;
                aT1 = aT2;
                aT2 = aT3;
            }


            OhArrayList dataPackArray = null;
            int numberOfDataPacks = 0;

            OhArrayList[] dataPackArrayArray = new OhArrayList[] { dataPackArray }; //; Array for work.
            int[] numberOfDataPacksArray = new int[] { numberOfDataPacks };

            int ip = 0;

            getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

            dataPackArray = dataPackArrayArray[0];
            numberOfDataPacks = numberOfDataPacksArray[0];

            if (numberOfDataPacks <= 1)
            {
                throw new OhError("Error in Get_Data", oPrintString(" >"));
            }


            IOhTimePointImmutable ta = ((OhDataPackLong)dataPackArray[0]).DataTime;
            IOhTimePointImmutable tb = ((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataTime;

            if (aT1.CompareTo(ta) < 0 || aT2.CompareTo(tb) > 0)
            {
                // ta <= aT1 and aT2 <= tb, right?
                throw new ArgumentException("Illegal aT1 or aT2.");
            }

            DataConnectionType dataConnectionType = (DataConnectionType)aDctVec[0];

            if (dataConnectionType == DataConnectionType.BROKENLINE)
            {
                long y = 0L;
                long ya = 0L;
                long yb = 0L;
                int ipa = 0;
                int ipb = numberOfDataPacks - 1;

                while (ipb - ipa > 1)
                {
                    ip = (ipa + ipb) / 2;

                    IOhTimePointImmutable tp = ((OhDataPackLong)dataPackArray[ip]).DataTime;

                    if (tp.CompareTo(aT1) < 0)
                    {
                        ipa = ip;
                        ta = tp;
                    }
                    else if (tp.CompareTo(aT1) > 0)
                    {
                        ipb = ip;
                        tb = tp;
                    }
                    else
                    {
                        ipb = ip;
                        ipa = ip - 1;
                        tb = tp;

                        ta = ((OhDataPackLong)dataPackArray[ipa]).DataTime;
                        break;
                    }
                }


                ya = ((OhDataPackLong)dataPackArray[ipa]).DataValue;
                yb = ((OhDataPackLong)dataPackArray[ipb]).DataValue;

                if (ipa == ipb)
                {
                    y = ya;
                }
                else
                {
                    y = (long)((OhTimePoint.sub(aT1,ta).toDouble() * yb + (OhTimePoint.sub(tb,aT1).toDouble() * ya)) 
                        / (OhTimePoint.sub(tb,ta).toDouble()));
                }

                IOhTimePointImmutable t = aT1;
                ip = ipb;
                IOhTimePointImmutable ts = tb;
                long ys = yb;

                while (ts.CompareTo(aT2) < 0)
                {
                    sum += (long)((y + ys) * OhTimePoint.sub(ts,t).toDouble() / 2);
                    t = ts;
                    y = ys;
                    ip++;

                    ts = ((OhDataPackLong)dataPackArray[ip]).DataTime;
                    ys = ((OhDataPackLong)dataPackArray[ip]).DataValue;
                }

                ys = (long)((y * OhTimePoint.sub(ts, aT2).toDouble() + ys * OhTimePoint.sub(aT2, t).toDouble()) 
                        / OhTimePoint.sub(ts,t).toDouble());
                sum += (long)((y + ys) * OhTimePoint.sub(aT2,t).toDouble() / 2);
            }
            else if (dataConnectionType == DataConnectionType.STEP)
            {
                int ipa = 0;
                int ipb = numberOfDataPacks - 1;

                while (ipb - ipa > 1)
                {
                    ip = (ipa + ipb) / 2;

                    IOhTimePointImmutable tp = ((OhDataPackLong)dataPackArray[ip]).DataTime;

                    if (tp.CompareTo(aT1) < 0)
                    {
                        ipa = ip;
                        ta = tp;
                    }
                    else if (tp.CompareTo(aT1) > 0)
                    {
                        ipb = ip;
                        tb = tp;
                    }
                    else
                    {
                        ipb = ip;
                        ipa = ip - 1;
                        tb = tp;
                        break;
                    }
                }

                IOhTimePointImmutable tn = tb;
                ip = ipb;

                long yn = ((OhDataPackLong)dataPackArray[ip]).DataValue;
                IOhTimePointImmutable ts = aT1;

                while (tn.CompareTo(aT2) < 0)
                {
                    sum += (long)(yn * OhTimePoint.sub(tn,ts).toDouble());
                    ts = tn;
                    ip++;

                    tn = ((OhDataPackLong)dataPackArray[ip]).DataTime;
                    yn = ((OhDataPackLong)dataPackArray[ip]).DataValue;
                }

                if (aT2.CompareTo(tn) <= 0)
                {
                    sum += (long)(yn * OhTimePoint.sub(aT2,ts).toDouble());
                }
            }
            else if (dataConnectionType == DataConnectionType.PULSE)
            {
                int ipa = 0;
                int ipb = numberOfDataPacks - 1;

                while (ipb - ipa > 1)
                {
                    ip = (ipa + ipb) / 2;

                    IOhTimePointImmutable tp = ((OhDataPackLong)dataPackArray[ip]).DataTime;

                    if (tp.CompareTo(aT1) < 0)
                    {
                        ipa = ip;
                        ta = tp;
                    }
                    else if (tp.CompareTo(aT1) > 0)
                    {
                        ipb = ip;
                        tb = tp;
                    }
                    else
                    {
                        ipb = ip + 1;
                        break;
                    }
                }

                IOhTimePointImmutable tn;
                long yn = 0L;

                for (; ip < numberOfDataPacks; ip++)
                {
                    tn = ((OhDataPackLong)dataPackArray[ip]).DataTime;
                    yn = ((OhDataPackLong)dataPackArray[ip]).DataValue;

                    if (tn.CompareTo(aT2) > 0)
                        break;

                    sum += yn;
                }
            }
            else
            {
                throw new ArgumentException("Illegal data_connection_type.");
            }

            return sum;
        }


        /// <summary> 
        /// 時刻 aT1 から時刻 aT2 までの積分値を返します。
        /// データ値の補間は折れ線とします。
        /// 
        /// <p/>
        /// Gets sum data whose data connection type is brokenline.
        /// 
        /// </summary>
        /// <param name="aT1">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aT2">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// </returns>
        public long getDataSum(OhTimePoint aT1, OhTimePoint aT2)
        {
            //(*3-2)
            long sum = 0L;

            if (aT1.CompareTo(aT2) == 0)
            {
                return sum;
            }

            if (aT1.CompareTo(aT2) > 0)
            {
                OhTimePoint aT3 = aT1;
                aT1 = aT2;
                aT2 = aT3;
            }


            OhArrayList dataPackArray = null;
            int numberOfDataPacks = 0;

            OhArrayList[] dataPackArrayArray = new OhArrayList[] { dataPackArray }; //; Array for work.
            int[] numberOfDataPacksArray = new int[] { numberOfDataPacks };

            int ip = 0;

            getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

            dataPackArray = dataPackArrayArray[0];
            numberOfDataPacks = numberOfDataPacksArray[0];

            if (numberOfDataPacks <= 1)
            {
                throw new OhError("Error in Get_Data", oPrintString(" >"));
            }

            OhTimePoint ta = new OhTimePoint();
            ta.copy(((OhDataPackLong)dataPackArray[0]).DataTime);

            OhTimePoint tb = new OhTimePoint();
            tb.copy(((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataTime);

            if (aT1.CompareTo(ta) < 0 || aT2.CompareTo(tb) > 0)
            {
                // ta <= aT1 and aT2 <= tb, right?
                throw new ArgumentException("Illegal aT1 or aT2.");
            }

            long y = 0L;
            long ya = 0L;
            long yb = 0L;
            int ipa = 0;
            int ipb = numberOfDataPacks - 1;

            while (ipb - ipa > 1)
            {
                ip = (ipa + ipb) / 2;

                OhTimePoint tp = new OhTimePoint();
                tp.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);

                if (tp.CompareTo(aT1) < 0)
                {
                    ipa = ip;
                    ta.copy(tp);
                }
                else if (tp.CompareTo(aT1) > 0)
                {
                    ipb = ip;
                    tb.copy(tp);
                }
                else
                {
                    ipb = ip;
                    ipa = ip - 1;
                    tb.copy(tp);

                    ta.copy(((OhDataPackLong)dataPackArray[ipa]).DataTime);

                    break;
                }
            }

            ya = ((OhDataPackLong)dataPackArray[ipa]).DataValue;
            yb = ((OhDataPackLong)dataPackArray[ipb]).DataValue;

            if (ipa == ipb)
            {
                y = ya;
            }
            else
            {
                y = (long)((OhTimePoint.sub(aT1,ta).toDouble() * yb + OhTimePoint.sub(tb,aT1).toDouble() * ya) 
                    / OhTimePoint.sub(tb,ta).toDouble());
            }

            OhTimePoint t = new OhTimePoint();
            t.copy(aT1);

            ip = ipb;

            OhTimePoint ts = new OhTimePoint();
            ts.copy(tb);

            long ys = yb;

            while (ts.CompareTo(aT2) < 0)
            {
                sum += (long)((y + ys) * OhTimePoint.sub(ts,t).toDouble() / 2);

                t.copy(ts);

                y = ys;
                ip++;

                ts.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);
                ys = ((OhDataPackLong)dataPackArray[ip]).DataValue;
            }

            ys = (long)((y * OhTimePoint.sub(ts,aT2).toDouble() + ys * OhTimePoint.sub(aT2,t).toDouble()) 
                / OhTimePoint.sub(ts,t).toDouble());
            sum += (long)((y + ys) * OhTimePoint.sub(aT2,t).toDouble() / 2);

            return sum;
        }




        /// <summary> 
        /// 時刻 aT におけるパルスデータ値を long 型配列 aDataValue に格納します。
        /// 
        /// <p/>
        /// Gets pulse data.
        /// 
        /// </summary>
        /// <param name="aT">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aDataValue">a <var>long[]</var> value<p/>
        /// </param>
        /// <returns> a <var>boolean</var> value<p/>
        /// </returns>
        public bool canYouGetPulseData(OhTimePoint aT, long[] aDataValue)
        {
            //(*2)

            OhArrayList dataPackArray = null;
            int numberOfDataPacks = 0;

            OhArrayList[] dataPackArrayArray = new OhArrayList[] { dataPackArray }; //; Array for work.
            int[] numberOfDataPacksArray = new int[] { numberOfDataPacks };

            int ip = 0;

            getDataPackArray(dataPackArrayArray, numberOfDataPacksArray);

            dataPackArray = dataPackArrayArray[0];
            numberOfDataPacks = numberOfDataPacksArray[0];


            if (numberOfDataPacks <= 0)
                return false;

            OhTimePoint ta = new OhTimePoint();
            ta.copy(((OhDataPackLong)dataPackArray[0]).DataTime);

            OhTimePoint tb = new OhTimePoint();
            tb.copy(((OhDataPackLong)dataPackArray[numberOfDataPacks - 1]).DataTime);


            if (aT.CompareTo(ta) < 0 || aT.CompareTo(tb) > 0)
                return false;

            OhTimePoint tn = new OhTimePoint();
            OhTimePoint tp = new OhTimePoint();
            long yn = 0L;
            long yp = 0L;

            tn.copy(ta);
            yn = ((OhDataPackLong)dataPackArray[0]).DataValue;

            if (aT.CompareTo(tn) == 0)
            {
                aDataValue[0] = yn;
                return true;
            }

            bool hit = false;

            for (ip = 1; ip < numberOfDataPacks; ip++)
            {
                tp.copy(tn);
                yp = yn;

                tn.copy(((OhDataPackLong)dataPackArray[ip]).DataTime);
                yn = ((OhDataPackLong)dataPackArray[ip]).DataValue;

                if (tn.CompareTo(aT) > 0)
                {
                    hit = false;
                    break;
                }
                else if (tn.CompareTo(aT) == 0)
                {
                    aDataValue[0] = yn;
                    hit = true;
                    break;
                }
            }
            return hit;
        }




        /// <summary> 
        /// データ補間型を指定して、時刻 aT1 におけるデータ値と時刻 aT2 におけるデータ値の平均値を返します。
        /// 
        /// <p/>
        /// Gets mean data.
        /// 
        /// </summary>
        /// <param name="aT1">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aT2">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <param name="aDctVec">a <var>Vector</var> value<p/>
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public long getMeanData(OhTimePoint aT1, OhTimePoint aT2, ArrayList aDctVec)
        {
            //(*6)
            if (aT2.CompareTo(aT1) == 0)
            {
                return getData(aT2, aDctVec);
            }
            else
            {
                return getDataSum(aT1, aT2, aDctVec) / (long)(OhTimePoint.sub(aT2,aT1).toDouble());
            }
        }




        /// <summary> 
        /// 時刻 aT1 におけるデータ値と時刻 aT2 におけるデータ値の平均値を返します。
        /// データ値の補間は折れ線とします。
        /// 
        /// <p/>
        /// Gets mean data whose data connection type is brokenline.
        /// 
        /// </summary>
        /// <param name="aT1">an <var>OhTimePoint</var> value<p/><p/>
        /// 時刻 aT1 におけるデータ値
        /// </param>
        /// <param name="aT2">an <var>OhTimePoint</var> value<p/><p/>
        /// 時刻 aT2 におけるデータ値
        /// </param>
        /// <returns> a <var>long</var> value<p/>
        /// </returns>
        public long getMeanData(OhTimePoint aT1, OhTimePoint aT2)
        {
            //(*6)
            if (aT2.CompareTo(aT1) == 0)
            {
                return getData(aT2);
            }
            else
            {
                return getDataSum(aT1, aT2) / (long)(OhTimePoint.sub(aT2,aT1).toDouble());
            }
        }

        // @end {methods} ------------------------------------------------------
    } // OhReceiveLong
}