/// <(#)>  OhReceivePort; class to express a receive port </(#)>
/// <summary>                     Make "OhReceivePort.cs" from OhReceivePort.java" by Naoto Ohshima.
using System;
using System.IO;
using OhymosNet;

namespace OhymosNet
{

    /// <summary>
    /// {^M[qf\NXłB
    /// ۃNXłB
    /// ̃NX<a href ="OhymosNet.OhPort.html">OhPort</a>pĂ܂B<p/>
    /// 
    /// Class for models of receive ports. 
    /// It extends <a href="OhymosNet.OhPort.html">OhPort</a> class.
    /// </summary>
    /// <remarks>
    /// M[qɂ͑M[q<b>P</b>ȂƂł܂B<p/>
    /// 
    /// NX OhReceivePort ł́A
    /// <ul>
    /// <li> f[^pbNM[q : _sender</li>
    /// </ul>
    /// LA
    /// <ul>
    /// <li> M[qƂ̐ڑ@\</li>
    /// <li> _sender ̃f[^pbNM@\</li>
    /// </ul>
    /// `܂B
    /// 
    /// <p/>
    /// This class OhReceivePort has a instance variable for storing a sender
    /// which sends datapacks to the current receive port. <p/>
    /// 
    /// Methods for
    /// <ul>
    /// <li>  connecting the current receive port to a corresponding sender</li>
    /// <li>  receiving datapacks from the sender</li>
    /// </ul>
    /// are defined. <p/>
    /// 
    /// NOTE:<p/> 
    /// 
    /// Only one send port can be connected to an object of class OhReceivePort.
    /// </remarks>
    /// 
    /// 
    /// <author>  <a href="mailto:satoh@wr.kuciv.kyoto-u.ac.jp">satoh</a>
    /// </author>
    /// <version>  $Revision: 1.45 $ $Date: 2004/02/11 22:52:00 $
    /// </version>
    public abstract class OhReceivePort : OhPort
    {
        // @begin {field} ------------------------------------------------------

        /// <summary> 
        /// <a href="OhSenderPort.html">OhSenderPort</a> ^ɃLXgꂽM[qłB
        /// 
        /// <p/>
        /// A sender which sends datapacks to the current object.
        /// 
        /// </summary>
        protected internal OhSenderPort _sender;

        // @end {field} --------------------------------------------------------

        // @begin{initialization} ----------------------------------------------

        /// <summary> 
        /// IuWFNgAIuWFNgԍ܂B
        /// 
        /// <p/>
        /// Initialization with object name "aPortName" and object number "aPortNum".
        /// 
        /// </summary>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// </param>
        public override void init(String aPortName, int aPortNum)
        {
            base.init(aPortName, aPortNum);
        }

        
        // @end{initialization} ------------------------------------------------


        // @begin {constructors} -----------------------------------------------

        /// <summary> 
        /// ftHgRXgN^łB<p/>
        /// Creates a new default <var>OhReceivePort</var> instance.
        /// </summary>
        /// <remarks>
        /// ftHgRXgN^
        /// <a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> s܂B
        /// ̌ʁAȉ̐ݒƂȂ܂B<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// ̑̃oϐɑ΂鏈͈RXgN^ƓłB
        /// ̏ꍇA<a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> 
        /// păoϐĉB
        /// <p/>
        /// </remarks>
        public OhReceivePort()
            : base()
        {
            _sender = null;

            _className = "OhReceivePort";
        }


        /// <summary> 
        /// IuWFNgiM[qj AIuWFNgiM[qjԍw肵āA
        /// <a href="OhReceivePort.html">OhReceivePort</a> NX̃CX^X𐶐܂B<p/>
        /// Creates a new default <var>OhReceivePort</var> instance.
        /// </summary>
        /// <remarks>
        /// NX <a href="OhPort.html">OhPort</a> ̃ftHgRXgN^pA
        /// <a href="OhReceivePort.html#init(String, int)">OhReceivePort.init(String, int)</a> pāA
        /// oϐȉ̒ʂ菉܂B<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhReceivePort"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _sender            : null</li>
        /// </dl>
        /// 
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// M[q
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// M[qԍ
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhReceivePort(String aPortName, int aPortNum)
            : base()
        {
            _sender = null;

            _className = "OhReceivePort";

            init(aPortName, aPortNum);
        }

        // @end {constructors} -------------------------------------------------

        // @begin {properties} ------------------------------------------------------

        /// <summary>
        /// ŌẪf[^̋L^擾܂B
        /// 
        /// Returns the data time of the oldest data pack in the data stored in the sender.
        /// </summary>
        /// <remarks>
        /// M[q(_sender)ɍŌẪf[^̋L^擾悤w܂B
        /// </remarks>
        ///
        /// <returns> an <var>OhTimePointImmutable</var> value<p/>
        /// ŌẪf[^L^
        /// </returns>
        public override IOhTimePointImmutable OldestDataTime
        {
            get
            {
                return _sender.OldestDataTime;
            }

        }
        /// <summary> 
        /// ŐṼf[^̋L^擾܂B<p/>
        /// Returns the data time of the newest data pack in the data stored in the sender.
        /// </summary>
        /// <remarks>
        /// M[q(_sender)ɍŐṼf[^̋L^擾悤w܂B
        /// </remarks>
        /// 
        /// <returns> an <var>OhTimePointImmutable</var> value<p/>
        /// ŐṼf[^L^
        /// </returns>
        public override IOhTimePointImmutable NewestDataTime
        {
            get
            {
                return _sender.NewestDataTime;
            }

        }
        /// <summary> 
        /// f[^pbŇ(_numberOfDataPacks)擾܂B <p/>
        /// Returns the number of datapacks stored in the sender.
        /// </summary>
        /// <remarks>
        /// M[q(_sender)Ƀf[^pbŇ(_numberOfDataPacks)擾悤w܂B
        /// </remarks>
        /// <returns> an <var>int</var> value<p/>
        /// f[^pbŇ
        /// </returns>
        public override int NumberOfDataPacks
        {
            get
            {
                return _sender.NumberOfDataPacks;
            }

        }
        /// <summary> 
        /// f[^Kv擾܂B<p/>
        /// 
        /// Gets the "NecessaryTimeFrom" (this) of the owner of the current receive port.
        /// You can redefine it in derived classes.
        /// </summary>
        /// <remarks>
        /// M̑M[q NecessaryTimeFrom@ȍ~ NecessaryTimeTo ȑÕf[^pbN
        /// Ȃ悤Ɏw邽߂ɒ`܂B<p/>
        /// 
        /// _owner 
        /// <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a>
        /// Ԃ܂B
        /// _owner  null ̏ꍇ
        /// <a href="OhTimePoint.html#OhTimePointMin">OhTimePoint.OhTimePointMin</a>
        /// Ԃ܂B
        /// 
        /// <p/>
        /// </remarks>
        /// <returns> an <var>OhTimePointImmutable</var> value<p/>
        /// </returns>
        public override IOhTimePointImmutable NecessaryTimeFrom
        {
            get
            {
                //; (*2)
                return (_owner != null) ? ((OhElement)_owner).NecessaryTimeFrom(this) : OhTimePoint.OhTimePointMin;
            }

        }
        /// <summary> 
        /// f[^Kv擾܂B<p/>
        /// Gets the "NecessaryTimeTo" (this) of the owner of the current receive port.
        /// You can redefine it in derived classes.
        /// </summary>
        /// <remarks>
        /// M̑M[q NecessaryTimeFrom ȍ~ NecessaryTimeTo ȑÕf[^pbN
        /// Ȃ悤Ɏw邽߂ɒ`܂B<p/>
        /// 
        /// _owner 
        /// <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a>
        /// Ԃ܂B
        /// _owner  null ̏ꍇ
        /// <a href="OhTimePoint.html#OhTimePointMax">OhTimePoint.OhTimePointMax</a>
        /// Ԃ܂B<p/>
        /// 
        /// <p/>
        /// </remarks>
        /// <returns> an <var>OhTimePointImmutable</var> value<p/>
        /// </returns>
        public override IOhTimePointImmutable NecessaryTimeTo
        {
            get
            {
                //; (*3)
                return (_owner != null) ? ((OhElement)_owner).NecessaryTimeTo(this) : OhTimePoint.OhTimePointMax;
            }

        }
        // 1. connection function.

        /// <summary> 
        /// Mf[^pbÑf[^ľ^\Ԃ܂B<p/>
        /// Returns a string which expresses the type of datapacks received by the current object.
        /// </summary>
        /// <remarks>
        /// NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
        ///  OhReceivePort łۃ\bhƂĂ܂B
        /// OhReceivePort ̔hNXŎMf[^pbN^\Ԃ悤ɕK`ĉB
        /// 
        /// Ⴆ΁AMf[^pbN^ "double" ^łhNXł́A
        /// 
        /// <pre><var>
        /// public String DataType
        /// {
        ///     get { return "double";}
        /// }
        /// </var></pre>
        /// 
        /// ̂悤ɒ`܂B<p/>
        /// 
        /// ڑɂ́A̒[qƑM[q̃f[^pbN^v邱ƂmF܂B
        /// 
        /// <p/>
        /// This is a abstract method.
        /// This function must be defined in derived classes.
        /// </remarks>
        /// <returns> a <var>String</var> value<p/>
        /// Mf[^pbN^\
        /// </returns>
        public abstract override String DataType { get; }
        // @end {properties} ------------------------------------------------------


        // @begin {methods} ----------------------------------------------------

        // 1. connection function.

        /// <summary> 
        /// M[qݒ肵܂B<p/>
        /// Sets "_sender" to "aSender".
        /// </summary>
        /// <remarks>
        /// NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB
        /// M[q _sender  aSender ɐݒ肵܂B<p/>
        /// 
        /// L̏ꍇ̓G[bZ[W\܂B<p/>
        /// 
        /// <ul>
        /// <li> aSender  null ̏ꍇB</li>
        /// <li> ̒[q̑M[q _sender ݒς݂̏ꍇB</li>
        /// <li> ̒[q aSender Ńf[^pbN^قȂꍇB</li>
        /// </ul>
        /// 
        /// [ACCIDENT]
        /// EaSender  <a href="OhPort.html">OhPort</a> ^ϐi[ĂȂꍇB
        /// <p/>
        /// 
        /// 
        /// When one of the following cases occurs <p/>
        /// <ul>
        /// <li> "aSender" is NULL.</li>
        /// <li> "_sender" has been set already.</li>
        /// <li> Data pack types between the current object and the sender are different.</li>
        /// </ul>
        /// <p/>
        /// this function prints an error message and returns "false".
        /// </remarks>
        /// <param name="aSender">an <var>OhPort</var> value<p/>
        /// M[q
        /// </param>
        public override void setSender(OhPort aSender)
        {
            //; (*1)
            if (aSender == null)
            {
                throw new ArgumentException("Cannot set \"sender\"." + oPrintString(" > "));
            }

            if (_sender != null)
            {
                StringWriter sw = new StringWriter();
                _sender.cPrint(sw, " >    connected to  : ");
                aSender.cPrint(sw, " >    connecting to  : ");
                oPrint(sw, " > ");
                String str = sw.ToString();
                sw.Close();
                throw new OhError("\"sender\" has already been set. Doubly setting?", str);
            }

            String sendType = aSender.DataType;
            String receiveType = DataType;

            if (!(sendType.Equals(receiveType)))
            {
                throw new OhError("'dataPackType' of this port and \"sendType\"(arg) are different" + NEWLINE 
                    + "sendType is " + sendType + " while receiveType is " + receiveType + NEWLINE 
                    + aSender.oPrintString(" >    s > ") + oPrintString(" > "));
            }

            _sender = (OhSenderPort)aSender;
        }


        /// <summary> 
        /// ڑƌ̏sȂ܂B<p/>
        /// </summary>
        /// 
        /// <remarks>
        /// OhReceivePort ł͐ڑƌɍsȂׂ͓ɂ܂B
        /// <p/>
        /// Because no corresponding work is needed in the <var>OhReceivePort</var> class,
        /// Switch of the <var>OhReceivePort</var> class simply returns "false". <p/>
        /// 
        /// Note: 
        /// 
        /// doSwitch() of the <var>OhSenderPort</var> class is defined differently.
        /// </remarks>
        public override void doSwitch()
        {
        }


        /// <summary> 
        /// vZƑO̊mFsȂ܂B<p/>
        /// Answers whether the current receive port has been connected to some sender or not.
        /// </summary>
        /// <remarks>
        /// NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhĂ܂B<br/>
        /// f[^pbNM邱ƂłԂł邱ƂmF܂B
        /// ܂A_sender ݒ肳Ă邩`FbN܂B
        /// _sender ݒ肳Ă΁A_sender f[^pbNM邱Ƃł܂B
        /// 
        /// 
        /// <p/>
        /// </remarks>
        /// <returns> a <var>bool</var> value<p/>
        /// <ul><pre>
        /// Etrue    _sender ݒς݂̏ꍇB
        /// Efalse   _sender ݒ̏ꍇB
        /// </pre></ul>
        /// </returns>
        public override bool areYouReady()
        {
            return (_sender != null) ? true : false;
        }


        // 2. function for receiving datapacks.

        /// <summary> 
        /// f|^pbN̔z쐬܂B<p/>
        /// Creates a datapack array.
        /// </summary>
        /// <remarks>
        /// NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB
        /// _sender Ƀf[^pbNz쐬悤߂܂B
        /// 
        /// <p/>
        /// Orders the sender to create a datapack array.
        /// </remarks>
        public override void setDataPackArray()
        {
            _sender.setDataPackArray();
        }


        /// <summary> 
        /// 擾\f[^pbN̊mFs܂B<p/>
        /// Tells whether the data from "aTFrom" to "aTTo" can be obtained or not. <p/>
        /// </summary>
        /// <remarks>
        /// 擾\f[^pbN̊mFif[^^̊mFjs܂B<p/>
        /// 
        /// NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB<p/>
        /// 
        /// _sender ɖ₢킹 aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNM\mF܂B
        /// M\Ȃ true ԂAMs\Ȃ false Ԃ܂B<p/>
        /// 
        /// ̎M[q _owner ŌvZ̊mFƂɗp܂B
        /// 
        /// 
        /// [ACCIDENT]
        /// EڑOŁA_sender  null ̏ꍇB
        /// ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
        /// E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
        /// <p/>
        /// 
        /// Accident: 
        /// 
        /// "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
        /// </remarks>
        /// <param name="aTFrom">an <var>OhTimePointImmutable</var> value<p/>
        /// aTFrom ȑOf[^pbNM\ł邱ƂmFB
        /// </param>
        /// <param name="aTTo">an <var>OhTimePointImmutable</var> value<p/>
        /// aTTo ȍ~܂Ńf[^pbNM\ł邱ƂmFB
        /// </param>
        /// <returns> a <var>bool</var> value<p/>
        /// <pre>
        /// Etrue      aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNM\ȏꍇB
        /// Efalse     aTFrom ȑO aTTo ȍ~܂ł̃f[^pbNMs\ȏꍇB
        /// </pre>
        /// </returns>
        public override bool canYouGetData(IOhTimePointImmutable aTFrom, IOhTimePointImmutable aTTo)
        {
            return _sender.canYouGetData(aTFrom, aTTo);
        }


        /// <summary> 
        /// 擾\f[^pbN̊mFsȂ܂B<p/>
        /// 
        /// <p/>
        /// Tells whether the data at "aTFromTo" can be obtained or not. <p/>
        /// </summary>
        /// 
        /// Accident: 
        /// 
        /// "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
        /// 
        /// <param name="aTFromTo">an <var>OhTimePoint</var> value<p/>
        /// </param>
        /// <returns> a <var>bool</var> value<p/>
        /// </returns>
        public override bool canYouGetData(OhTimePoint aTFromTo)
        {
            return _sender.canYouGetData(aTFromTo, aTFromTo);
        }


        /// <summary> 
        /// f[^pbN̔z񃊃Xg擾(M)܂B<p/>
        /// Get a data pack array from the sender.
        /// </summary>
        /// <remarks>
        /// NX <a href="OhPort.html">OhPort</a> ̒ۃ\bhłB<p/>
        /// 
        /// M[q _sender f[^pbNzM܂B<p/>
        /// 
        ///  aDpArray ̑vfɂ Object ^ɃLXgꂽ
        /// <a href="OhDataPack.html">OhDataPack</a> ^ϐ̔z񂪊i[A
        /// aNumOfDp ̑vfɂ aDpArray ̗vfi[܂B<p/>
        /// 
        /// aDpArray ̎Mf[^pbÑf[^L^AĂ邱Ƃ
        /// <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a>
        /// ɂۏ؂܂B<p/>
        /// 
        /// Kvȃf[^pbNM\ۂ
        /// canYouGetData(OhTimePoint, OhTimePoint) pĊmFł܂B<p/>
        /// 
        /// [ACCIDENT]
        /// EڑO _sender  null ̏ꍇB
        /// ĩ\bh͌vZɕpɂɗp\̂Ŋ _sender  null łȂƂmF܂łBj
        /// E_sender  <a href="OhPort.html">OhPort</a> ^ȊO̕ϐi[ĂꍇB
        /// <p/>
        /// After the execution of this function, "aDpArray" stores the data pack array
        /// and "aNumOfDp" stores the dimention of "aDpArray". <p/>
        /// 
        /// Accident: 
        /// 
        /// "_sender" is "null" or not <a href="OhPort.html">OhPort</a> type.
        /// </remarks>
        /// <param name="aDpArray">an <var>OhArrayList[]</var> value<p/>
        /// Mf[^pbNz1vfɊi[z񃊃Xg
        /// </param>
        /// <param name="aNumOfDp">an <var>int[]</var> value<p/>
        /// aDpArray ̗vfvfɊi[ int ^z 
        /// </param>
        public override void getDataPackArray(OhArrayList[] aDpArray, int[] aNumOfDp)
        {
            _sender.getDataPackArray(aDpArray, aNumOfDp);
        }


        /// <summary> 
        /// wɂf[^pbN擾(M)܂B<p/>
        /// Get a data pack at specified time "_dataTime".
        /// </summary>
        /// <remarks>
        /// NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p/>
        /// 
        /// w肵 _sender f[^pbNM܂B<p/>
        /// 
        /// aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<br/>
        /// 
        /// aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<br/>
        /// 
        /// aFlag ͎擾̃[hI邽߂̃IvVŁAL̒ʂłB
        /// 
        /// <pre>
        /// E"b"       aFormer, aLater ̗KMB
        ///             MłȂꍇ́AƂ null ɐݒ肷B
        /// 
        /// E"f"       aLater MłȂĂAaFormer ̂ݎMB
        ///             aLater MłȂꍇAnull ɐݒ肷B
        /// 
        /// E"l"       aFormer MłȂĂAaLater ̂ݎ擾B
        ///             aFormer MłȂꍇAnull ɐݒ肷B
        /// 
        /// Ȇ    aFormer, aLater ̂ȂƂ擾B
        ///             MłȂ̂ null ɐݒ肷B
        /// </pre>
        /// 
        /// aFlag ̃ftHgĺA "b" łB
        /// ܂A  aLater nȂꍇ́A aFlag  f Ƃ aFormer ̂ݎMA
        ///  aFormer nȂꍇ́A aFlag  l Ƃ aLater ̂ݎM܂B
        /// 
        /// f[^L^x _dataTime łf[^pbN^Ăꍇ́A
        /// aFormer, aLater Ƃi[܂B
        /// 
        /// 𖞂f[^pbN擾łȂꍇAaFormer, aLater  null ɐݒ肳܂B
        /// 
        /// f[^pbN擾łꍇ́Atrue Ԃ܂B
        /// 
        /// 
        /// 
        /// <p/>
        /// 
        /// The results are stored in "aFormer" and "aLater".
        /// 
        /// <ul>
        /// <li> "aFormer" is the newest data before "_dataTime"</li>
        /// <li> "aLater" is the pointer to the oldest data after "_dataTime"</li>
        /// </ul>
        /// 
        /// When a data pack just at "_dataTime" exists, "aFormer" and "aLater" point to that data pack.
        /// <p/>
        /// "aFlag" specifies the search option.
        /// 
        /// <dl>
        /// <dt> "b"</dt>
        /// <dd>  Receive both of "former" and "later".</dd>
        /// 
        /// <dt> "f"</dt>
        /// <dd>   Receive "aFormer". Even if "aLater" cannot be obtained, the function returns "true".</dd>
        /// 
        /// <dt> "l"</dt>
        /// <dd>  Receive "aLater". Even if "aFormer" cannot be obtained, the function returns "true".</dd>
        /// 
        /// <dt> other character</dt>
        /// <dd>  Receive at lease one of "aFormer" and "aLater".</dd>
        /// </dl>
        /// </remarks>
        /// <param name="aFormer">an <var>Object[]</var> value<p/>
        /// _dataTime ȑO _dataTime ɍł߂^f[^pbN1vfɊi[Object^z
        /// </param>
        /// <param name="_dataTime">an <var>OhTimePoint</var> value<p/>
        /// w莞<p/>
        /// ̎ɍł߂f[^L^^f[^pbNM܂B
        /// </param>
        /// <param name="aLater">an <var>Object[]</var> value<p/>
        /// _dataTime ȍ~ _dataTime ɍł߂^f[^pbN1vfɊi[Object^z
        /// </param>
        /// <param name="aMode">a <var>char</var> value<p/>
        /// f[^pbN̎Mw肷tO
        /// </param>
        /// <returns> a <var>bool</var> value<p/>
        /// <pre>
        /// Etrue      w̃f[^pbNMłꍇB
        /// Efalse     _dataPackArray  null ̏ꍇB
        ///             w̃f[^pbNMłȂꍇB
        /// </pre>
        /// </returns>
        public override bool getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater, char aMode)
        {
            return _sender.getDataPack(aFormer, _dataTime, aLater, aMode);
        }


        /// <summary> 
        /// wɂf[^pbN擾(M)܂B<p/>
        /// By all means, Get a data pack at specified time "_dataTime".
        /// The results are stored in "aFormer" and "aLater".
        /// </summary>
        /// <remarks>
        /// NX <a href="OhSenderPort.html">OhSenderPort</a> ̒ۃ\bhłB<p/>
        /// 
        /// w肵 _sender f[^pbNM܂B<p/>
        /// 
        /// aFormer ɂ́Af[^L^ _dataTime ȑO _dataTime ɍł߂^f[^pbNMAi[܂B<br/>
        /// 
        /// aLater ɂ́Af[^L^ _dataTime ȍ~ _dataTime ɍł߂^f[^pbNMAi[܂B<br/>
        /// 
        /// aFormer, aLater ̗KM܂B
        /// MłȂꍇ́AƂ null ɐݒ肳܂B
        /// 
        /// f[^L^x _dataTime łf[^pbN^Ăꍇ́A
        /// aFormer, aLater Ƃi[܂B
        /// 
        /// f[^pbN擾łꍇ́Atrue Ԃ܂B
        /// </remarks>
        /// <param name="aFormer">an <var>Object[]</var> value<p/>
        /// _dataTime ȑO _dataTime ɍł߂^f[^pbN1vfɊi[Object^z
        /// </param>
        /// <param name="_dataTime">an <var>OhTimePoint</var> value<p/>
        /// w莞<p/>
        /// ̎ɍł߂f[^L^^f[^pbNM܂B
        /// </param>
        /// <param name="aLater">an <var>Object[]</var> value<p/>
        /// _dataTime ȍ~ _dataTime ɍł߂^f[^pbN1vfɊi[Object^z
        /// </param>
        /// <returns> a <var>bool</var> value<p/>
        /// <pre>
        /// Etrue      w̃f[^pbNMłꍇB
        /// Efalse     _dataPackArray  null ̏ꍇB
        ///             w̃f[^pbNMłȂꍇB
        /// </pre>
        /// </returns>
        public virtual bool getDataPack(Object[] aFormer, OhTimePoint _dataTime, Object[] aLater)
        {
            return _sender.getDataPack(aFormer, _dataTime, aLater, 'b');
        }


        /// <summary> 
        /// M[qւ̌vZv͑M[qɓ]܂B
        /// 
        /// <p/>
        /// Relay the calculation request to the sender.
        /// 
        /// </summary>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void calReq()
        {
            _sender.calReq();
        }


        /// <summary> 
        /// svf[^pbN܂B
        /// Remove unnecessary data packs from the memory.
        /// </summary>
        /// <remarks>
        /// M[q _sender ɕsvf[^pbN悤w܂B<p/>
        /// 
        /// </remarks>
        public override void reduceDataPackStock()
        {
            _sender.reduceDataPackStock();
        }


        // 3. Information output function.

        /// <summary> 
        /// ڑo͂܂B
        /// 
        /// <p/>
        /// Print the information about the connection to "aSw" with the string "aHead".
        /// 
        /// </summary>
        /// <param name="aSw">a <var>StreamWriter</var> value<p/>
        /// ڑo͂Xg[
        /// </param>
        /// <param name="aHead">a <var>String</var> value<p/>
        /// </param>
        public override void printConnection(StreamWriter aSw, String aHead)
        {
            //; (*4)
            oPrint(aSw, aHead);

            if (_sender == null)
            {
                aSw.Write(aHead + "\tNot connected." + NEWLINE);
                return;
            }

           _sender.oPrint(aSw, aHead + "Connected > ");
        }

        // @end {methods} ------------------------------------------------------
    } // OhReceivePort
}