﻿/// <(#)>  OhRelayDouble </(#)>
/// <summary> 
/// </summary>
using System;
using System.IO;
using System.Text;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> 
    /// double型のデータパックを中継する端子モデルを表すクラスです。
    /// このクラスは<a href ="OhymosNet.OhRelayPort.html">OhRelayPort</a>を継承しています。
    /// <p/>
    /// The OhRelayDouble class is a class of port which relays
    /// datapacks of type double.
    /// It extends <a href ="OhymosNet.OhRelayPort.html">OhRelayPort</a> class.
    /// 
    /// </summary>
    public class OhRelayDouble : OhRelayPort
    {
        // @begin{initialization} ------------------------------------

        /// <summary> 
        /// オブジェクト（中継端子）名、オブジェクト（中継端子）番号を初期化します。
        /// 
        /// Initialization.
        /// 
        /// </summary>
        /// <param name="aPortName">a <code>String</code> value
        /// 中継端子名
        /// </param>
        /// <param name="aPortNum">an <code>int</code> value
        /// 中継端子番号
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void init(String aPortName, int aPortNum)
        {
            base.init(aPortName, aPortNum);
        }

        // @end{initialization} --------------------------------------


        // @begin {constructors} -------------------------------------

        /// <summary> 
        /// デフォルトコンストラクタです。
        /// Creates a new default <code>OhRelayDouble</code> instance.
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// <a href="OhRelayDouble.html#init(String, int)">OhRelayDouble.init(String, int)</a> を実行しません。
        /// その結果、以下の設定となります。<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// </dl>
        /// 
        /// <p/>
        /// その他のメンバ変数に対する処理は引数つきコンストラクタと同じです。
        /// この場合、<a href="OhRelayDouble.html#init(String, int)">OhRelayDouble.init(String, int)</a> を
        /// 用いてメンバ変数を初期化して下さい。<p/>
        /// </remarks>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhRelayDouble()
            : base()
        {
            _className = "OhRelayDouble";
        }

        /// <summary> 
        /// オブジェクト（中継端子）名、オブジェクト（中継端子）番号を指定して、
        /// <var>OhRelayDouble</var>クラスの新しいインスタンスを生成します。
        /// Creates a new <var>OhRelayDouble</var> instance.
        /// </summary>
        /// <remarks>
        /// 基底クラス <a href="OhSenderPort.html">OhSenderPort</a> のデフォルトコンストラクタを基本とし、
        /// <a href="OhRelayPort.html#init(String, int)">OhRelayPort(String, int)</a> を用いて、
        /// メンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhRelayDouble"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _owner             : null</li>
        /// <li> _sender            : null</li>
        /// <li> _bufferList        : メモリに確保する。（接続作業用）</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// </dl>
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 中継端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 中継端子番号
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhRelayDouble(String aPortName, int aPortNum)
            : base(aPortName, aPortNum)
        {
            _className = "OhRelayDouble";
        }

        // @end {constructors} ---------------------------------------


        // @begin {property} ------------------------------------------

        /// <summary> 
        /// 中継するデータパックのデータ値の型 double を表す文字列 "double" を取得します。
        /// 
        /// Gets the data pack type which expresses the type of data to be received.
        /// 
        /// </summary>
        /// <returns> a <code>String</code> value
        /// </returns>
        public override String DataType
        {
            get { return "double"; }
        }

        // @end {property} --------------------------------------------
    } // OhRelayDouble
}