/// <(#)>  OhScfSubSystem </(#)>
using System;
using System.IO;
using System.Text;
using System.Reflection;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
	/// \`t@Cgpč\镔n\NXłB
    /// ̃NX<a href ="OhymosNet.OhSubSystem.html">OhSubSystem</a>pĂ܂B
	/// <p/>
    /// Subsystem class configured from a "structure configuration
    /// file".
    /// It extends <a href ="OhymosNet.OhSubSystem.html">OhSubSystem</a> class.
    /// </summary>
    /// <remarks>
    /// ̃NX́A[q\vf̍\t@C "Structure
	/// Configuration File" ǂ݂Ƃč肠镔ñNX
	/// B<p/>
    /// .NET̃tNV@\<a href = "OhymosNet.OhAssembly.html">OhAssembly</a>
    /// pāA\`t@C^NXƂɁA
    /// V~[VsɓIɃIuWFNg𐶐An\܂B
	/// <p/>
    /// </remarks>
	public class OhScfSubSystem:OhSubSystem
	{
		// @begin {instance variable or static variable} -------------
		
		/// <summary> The array where relay ports for receiveing data are stored.
		/// 
		/// </summary>
		protected internal OhPort[] _receivePorts;
		
		/// <summary> Number of relay ports for receiving data.
		/// 
		/// </summary>
		protected internal int _numOfReceivePorts;
		
		/// <summary> The array where relay ports for sending data are stored.
		/// 
		/// </summary>
		protected internal OhPort[] _sendPorts;
		
		/// <summary> Number of relay ports for sending data.
		/// 
		/// </summary>
		protected internal int _numOfSendPorts;
		
		/// <summary> Usual (Non-iteratetive) Components.
		/// 
		/// </summary>
		protected internal OhElement[] _elements;
		
		/// <summary> Number of usual components.
		/// 
		/// </summary>
		protected internal int _numOfElements;
		
		/// <summary> Iterative Components.
		/// 
		/// </summary>
		protected internal OhElement[] _iterators;
		
		/// <summary> Number of iterative components.
		/// 
		/// </summary>
		protected internal int _numOfIterators;
		
		/// <summary> "Structure Configuration File" Name.
		/// 
		/// </summary>
		protected internal String _scfFile;
		
		// @end {instance variable or static variable} ---------------
		
		// @begin {initialization} -----------------------------------
		
		/// <summary> Initialize the element with the information obtained from
		/// the file.
		/// 
		/// <pre>
		/// [in] objectname -- object name
		/// [in] objnum     -- object number
		/// [in] fp         -- file pointer
		/// </pre>
		/// 
		/// </summary>
		/// <param name="aObjName">a <code>String</code> value
		/// </param>
		/// <param name="aObjNum">an <code>int</code> value
		/// </param>
		/// <param name="aSr">an <code>OhStreamReader</code> value
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  init(String aObjName, int aObjNum, OhStreamReader aSr)
		{
			
			// OhObject.init(aObjName, aObjNum);
			//
			if (aObjName == null)
			{
				throw new ArgumentException(oPrintString("\t"));
			}
			
			if (_objectName != null)
			{
				throw new OhError("\taObjName(arg) = \"" + aObjName + "\", aObjNum(arg) = " + aObjNum + NEWLINE + oPrintString("\t"));
			}
			
			_objectName = aObjName;
			_objectNumber = aObjNum;
			
			
			// Read SCF name from "aBr".
			//
			String fileName = null;
			try
			{
				aSr.skipWhites();
				fileName = aSr.readWord();
			}
			catch (Exception e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			_scfFile = fileName;
			
			
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
                sr = new OhStreamReader(_scfFile, true);
			}
			catch (FileNotFoundException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			// Read Numbers part.
			//
			try
			{
				sr.skipWhites();
				_numOfReceivePorts = Int32.Parse(sr.readWord());
				if (_numOfReceivePorts > 0)
				{
					_receivePorts = new OhPort[_numOfReceivePorts];
				}
				
				sr.skipWhites();
				_numOfElements = Int32.Parse(sr.readWord());
				if (_numOfElements > 0)
				{
					_elements = new OhElement[_numOfElements];
				}
				
				sr.skipWhites();
				_numOfIterators = Int32.Parse(sr.readWord());
				if (_numOfIterators > 0)
				{
					_iterators = new OhElement[_numOfIterators];
				}
				
				sr.skipWhites();
				_numberOfIteratorSets = Int32.Parse(sr.readWord());
				
				sr.skipWhites();
				_numOfSendPorts = Int32.Parse(sr.readWord());
				if (_numOfSendPorts > 0)
				{
					_sendPorts = new OhPort[_numOfSendPorts];
				}
			}
			catch (Exception e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			// Read Memory allocation part.
			//
			String header;
			String objectName;
			String className;
			String portType;
			int objectNumber;

            // Register OHyMoS.NET assemblies.
            OhAssembly ohymosAssembly = OhAssembly.Default;
			
			// Read [ReceivePorts] section.
			//
            try
            {
                sr.skipWhites();
                header = sr.readWord();

                if (header.Equals("[ReceivePorts]"))
                {
                    int id;
                    for (int i = 0; i < _numOfReceivePorts; i++)
                    {
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        objectNumber = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF data error in [ReceivePorts] section. " + "The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }

                        Assembly a;
                        if (ohymosAssembly.PortAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _receivePorts[i] = OhPort.create(a.GetType("OhymosNet." + className, true));
                            _receivePorts[i].init(objectName, objectNumber);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" +" does not exist in OHyMoS.NET.");
                    }
                }
                else
                {
                    String str = "SCF data error: no [ReceivePorts] section." + NEWLINE;
                    throw new OhError(str);
                }
            }

            catch (Exception e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }
			
			// Read [Components] section.
			//
			try
			{
				sr.skipWhites();
				header = sr.readWord();
				
				if (header.Equals("[Components]"))
				{
					for (int i = 0; i < _numOfElements; i++)
					{
						int id;
						sr.skipWhites();
						id = Int32.Parse(sr.readWord());
						sr.skipWhites();
						objectName = sr.readWord();
						sr.skipWhites();
						objectNumber = Int32.Parse(sr.readWord());
						sr.skipWhites();
						className = sr.readWord();
						
						if (id != i)
						{
							StringWriter sw = new StringWriter();
							sw.Write("SCF data error in [Components] section. " + "The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
							String str = sw.ToString();
							sw.Close();
							throw new OhError(str);
						}

                        Assembly a;
                        if ( ohymosAssembly.ElementAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _elements[id] = OhElement.create(a.GetType("OhymosNet."+ className,true));
                            _elements[id].init(objectName, objectNumber, sr);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");
                    }
				}
				else
				{
					String str = "SCF data error: no [Components] section." + NEWLINE;
					throw new OhError(str);
				}
			}
            catch (TypeLoadException e)
            {
                throw e;
            }
			catch (Exception e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			// Read [Iterators] section.
			//
			try
			{
				sr.skipWhites();
				header = sr.readWord();
				
				if (header.Equals("[Iterators]"))
				{
                    for (int i = 0; i < _numOfIterators; i++)
                    {
                        int id;
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        objectNumber = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF data error in [Iterators] section. " + "The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }

                        Assembly a;
                        if (ohymosAssembly.ElementAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _iterators[id] = OhElement.create(a.GetType("OhymosNet." + className, true));
                            _iterators[id].init(objectName, objectNumber, sr);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");

                    }
				}
				else
				{
					String str = "SCF data error: no [Iterators] section." + NEWLINE;
					throw new OhError(str);
				}
			}

            catch (TypeLoadException e)
            {
                throw e;
            }
			catch (Exception e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			// Read [SendPorts] section.
			//
			try
			{
				sr.skipWhites();
				header = sr.readWord();
				
				if (header.Equals("[SendPorts]"))
				{
                    for (int i = 0; i < _numOfSendPorts; i++)
                    {
                        int id;
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        objectNumber = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();
                        sr.skipWhites();
                        portType = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF data error in [SendPorts] section. " + "The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }

                        Assembly a;
                        if ( ohymosAssembly.PortAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _sendPorts[i] = OhPort.create(a.GetType("OhymosNet."+ className,true));
                            _sendPorts[i].init(objectName, objectNumber);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");
                    }
				}
				else
				{
					String str = "SCF data error: no [SendPorts] section." + NEWLINE;
					throw new OhError(str);
				}
			}

            catch (TypeLoadException e)
            {
                throw e;
            }
			catch (Exception e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
		}
		
		// @end {initialization} -------------------------------------
		
		// @begin {constructors} -------------------------------------
		
		/// <summary> <japanese>
		/// ftHgRXgN^łB
		/// </japanese>
		/// 
		/// </summary>
		public OhScfSubSystem():base()
		{
			_receivePorts = null;
			_sendPorts = null;
			_elements = null;
			_iterators = null;
			_scfFile = null;
			_className = "OhScfSubSystem";
		}
		
		// @end {constructors} ---------------------------------------
		
		// @begin {methods} ------------------------------------------
		
		/// <summary> <japanese>
		/// Mp[qo^܂B
		/// hNXŒ`܂B
		/// (Ref "OhSubSystem#construct()")
		/// Ă΂܂B
		/// o^ĂȂMp[q͐ڑł܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerReceivePorts()
		{
			for (int i = 0; i < _numOfReceivePorts; i++)
			{
				register((OhObject) _receivePorts[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// Mp[qo^܂B
		/// hNXŒ`܂B
		/// (Ref "OhSubSystem#construct()")
		/// Ă΂܂B
		/// o^ĂȂMp[q͐ڑł܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerSendPorts()
		{
			for (int i = 0; i < _numOfSendPorts; i++)
			{
				register((OhObject) _sendPorts[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// S\vf̂AvZsȂȂ\IuWFNg̃A
		/// hX _componentArray ɓo^܂B<p/>
		/// 
		/// o^ĂȂ\IuWFNgɂ́AvZs߂𑗂邱
		/// Ƃł܂B<p/>
		/// 
		/// Ⴆ΁AhNXō\IuWFNg abc oϐƂ
		/// `ꍇÂ悤 abc o^܂B
		/// 
		/// <pre>
		/// public void registerComponents() {
		/// register((Object)abc);
		/// }
		/// </pre>
		/// 
		/// ̎AK abc  Object ^ɃLXgĉB<p/>
		/// 
		/// ɂ abc ́AꎞI _objectList Ɏ^ꂽA
		/// ŏII Object ^z _componentArray ɋL܂B
		/// (Ref "OhSubSystem#construct()") QƁj
		/// 
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerComponents()
		{
			//; (*3)
			for (int i = 0; i < _numOfElements; i++)
			{
				register((OhObject) _elements[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// ̃\bh́AS\vf̂AvZsȂvf̃A
		/// hX _iteratorArray ɓo^܂BvZsȂ
		/// nfVɍ쐬vO}Ã\bh`
		/// ΂Ȃ܂B<p/>
		/// 
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerIterators()
		{
			//; (*4)
			if (_numberOfIteratorSets <= 0)
				return ;
			
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
				sr = new OhStreamReader(_scfFile, true);
			}
			catch (FileNotFoundException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			String strLine = null;
			bool foundP = false;
			
			try
			{
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[RegisterIterators]"))
					{
						foundP = true;
						break;
					}
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			if (foundP == false)
			{
				throw new OhError("");
			}
			
			
			try
			{
				for (int m = 0; m < _numberOfIteratorSets; m++)
				{
					int num;
					sr.skipWhites();
					num = Int32.Parse(sr.readWord());
					
					//; _iteratorArray[i] == null ƂȂĂ i TB
					//;  exit B
					//;
					int i;
					bool flag = false;
					
					for (i = 0; i < _numberOfIteratorSets; i++)
					{
						if (_iteratorArray[i] == null)
						{
							flag = true;
							break;
						}
					}
					
					if (flag == false)
					{
						throw new OhError(oPrintString("    "));
					}
					
					for (int k = 0; k < num; k++)
					{
						int s;
						sr.skipWhites();
						s = Int32.Parse(sr.readWord());
						register(_iterators[s]);
					}
					
					int[] numberOfIteratorsArray = new int[]{_numberOfIterators[i]};
					
					_iteratorArray[i] = makeObjectArray(numberOfIteratorsArray);
					_numberOfIterators[i] = numberOfIteratorsArray[0];
					
					_objectList.Clear();
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
		}
		
		
		/// <summary> <japanese>
		/// nf̒[q̐ڑ
		/// (Ref "OhSubSystem#connect()")
		/// pĒ`܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  connectPorts()
		{
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
				sr = new OhStreamReader(_scfFile, true);
			}
			catch (FileNotFoundException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			String strLine = null;
			bool foundP = false;
			
			try
			{
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[ConnectPorts]"))
					{
						foundP = true;
						break;
					}
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			if (foundP == false)
			{
				throw new OhError("");
			}
			
			String objNameS = null;
			String objNameR = null;
			int objNumS = 0;
			int objNumR = 0;
			
			try
			{
				while (true)
				{
					// -------------- starts while(true) loop.
					String str;
					sr.skipWhites();
					str = sr.readWord();
					
					if (str[0] == 'z')
					{
						break;
					}
					
					// -- There are nine cases. ---
					
					if (str[0] == 'r')
					{
						// The sender is a receive port.
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord()); // Receive port ID.
						
						if (i >= _numOfReceivePorts)
						{
							throw new OhError(_scfFile + ": Illegal receive port number: r " + i);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal element number: c " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect((OhPort) _receivePorts[i], (OhElement) _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal iterator element number: i " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect((OhPort) _receivePorts[i], (OhElement) _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal output port number: s " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_receivePorts[i], _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else if (str[0] == 'c')
					{
						// The sender is a usual component.
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord());
						sr.skipWhites();
						objNameS = sr.readWord();
						sr.skipWhites();
						objNumS = Int32.Parse(sr.readWord());
						
						if (i >= _numOfElements)
						{
							throw new OhError(_scfFile + ": Illegal element number: c " + i);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									throw new OhError(_scfFile + ": Illegal element number: c " + j);
								}
								
								connect((OhElement) _elements[i], objNameS, objNumS, (OhElement) _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									throw new OhError(_scfFile + ": Illegal iterator element number: i " + j);
								}
								
								connect((OhElement) _elements[i], objNameS, objNumS, (OhElement) _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									throw new OhError(_scfFile + ": Illegal output port number: s " + j);
								}
								
								connect((OhElement) _elements[i], objNameS, objNumS, (OhPort) _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else if (str[0] == 'i')
					{
						// The sender is an
						// iterative component.
						
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord());
						sr.skipWhites();
						objNameS = sr.readWord();
						sr.skipWhites();
						objNumS = Int32.Parse(sr.readWord());
						
						if (i >= _numOfIterators)
						{
							throw new OhError(_scfFile + ": Illegal iterator element number: i " + i);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									throw new OhError(_scfFile + ": Illegal element number: c " + j);
								}
								
								connect((OhElement) _iterators[i], objNameS, objNumS, (OhElement) _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									throw new OhError(_scfFile + ": Illegal iterator element number: i " + j);
								}
								
								connect((OhElement) _iterators[i], objNameS, objNumS, (OhElement) _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									throw new OhError(_scfFile + ": Illegal output port number: s " + j);
								}
								
								connect((OhElement) _iterators[i], objNameS, objNumS, (OhPort) _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else
					{
						throw new OhError("illegal character in SCF file.");
					}
				} // ----------------------------- ends while(true) loop.
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
		}
		
		
		/// <summary> <japanese>
		/// nf̒ڒʐMڑ
		/// (Ref "OhElement#register(Object, int)")
		/// pĒ`܂B
		/// 
		/// <pre>
		/// [ConnectComponents]
		/// c 2 c 3 0 # mElmnt[2]->Register((void*)mElmnt[3], 0)
		/// i 0 i 1 2 # mIte[0]->Register((void*)mIte[1], 2);
		/// z # end mark
		/// </pre>
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  connectComponents()
		{
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
                sr = new OhStreamReader(_scfFile, true);
			}
			catch (FileNotFoundException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			
			String strLine = null;
			bool foundP = false;
			
			try
			{
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[ConnectComponents]"))
					{
						foundP = true;
						break;
					}
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			if (foundP == false)
			{
				throw new OhError("");
			}
			
			try
			{
				while (true)
				{
					String strCom1;
					String strCom2;
					int i1;
					int i2;
					int j;
					
					sr.skipWhites();
					strCom1 = sr.readWord();
					
					if (strCom1[0] == 'z')
					{
						break;
					}
					
					sr.skipWhites();
					i1 = Int32.Parse(sr.readWord());
					sr.skipWhites();
					strCom2 = sr.readWord();
					sr.skipWhites();
					i2 = Int32.Parse(sr.readWord());
					sr.skipWhites();
					j = Int32.Parse(sr.readWord());
					
					OhElement elm1;
					OhElement elm2;
					
					elm1 = (strCom1[0] == 'c')?_elements[i1]:_iterators[i1];
					elm2 = (strCom2[0] == 'c')?_elements[i2]:_iterators[i2];
					
					elm1.register(elm2, j);
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
		}
		
        /// <summary>
        /// ̃NX̊{łB
        /// </summary>
        /// <returns></returns>
		public override String basicInfo()
		{
			return "Sub-system model which uses a \"structure configuration file\"";
		}
		
		// @end {methods} --------------------------------------------
	} // OhScfSubSystem
}