/// <(#)>  total system created from a structure configuration file. </(#)>
using System;
using System.Text;
using System.IO;
using System.Collections;
using System.Reflection;
//using System.Text.RegularExpressions;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> 
    /// \`t@Cgpč\Šn\NXłB
    /// ̃NX<a href ="OhymosNet.OhTotalSystem.html">OhTotalSystem</a>
    /// <p/>
    /// totalsystem class configured from a "structure configuration
    /// file".
    /// It extends <a href ="OhymosNet.OhTotalSystem.html">OhTotalSystem</a>
    /// </summary>
    /// <remarks>
    /// ̃NX́A[q\vf̍\t@C "Structure
    /// Configuration File" ǂ݂Ƃč肠ŠñNX
    /// B<p/>
    /// .NET̃tNV@\<a href = "OhymosNet.OhAssembly.html">OhAssembly</a>
    /// pāA\`t@C^NXƂɁA
    /// V~[VsɓIɃIuWFNg𐶐AŠn\܂B
    /// <p/>
    /// </remarks>
	public class OhScfTotalSystem:OhTotalSystem
	{
		// @begin {instance variable or static variable} -------------
		
		/// <summary> The array where relay ports for receiveing data are stored.
		/// 
		/// </summary>
		protected internal OhPort[] _receivePorts;
		
		/// <summary> Number of relay ports for receiving data.
		/// 
		/// </summary>
		protected internal int _numOfReceivePorts;
		
		/// <summary> The array where relay ports for sending data are stored.
		/// 
		/// </summary>
		protected internal OhPort[] _sendPorts;
		
		/// <summary> Number of relay ports for sending data.
		/// 
		/// </summary>
		protected internal int _numOfSendPorts;
		
		/// <summary> Usual(Non-iterate) Components.
		/// 
		/// </summary>
		protected internal OhElement[] _elements;
		
		/// <summary> Number of usual components.
		/// 
		/// </summary>
		protected internal int _numOfElements;
		
		/// <summary> Iterative Components.
		/// 
		/// </summary>
		protected internal OhElement[] _iterators;
		
		/// <summary> Number of iterative components.
		/// 
		/// </summary>
		protected internal int _numOfIterators;
		
		/// <summary> "Structure Configuration File" Name.
		/// 
		/// </summary>
		protected internal String _scfFile;
		
		// @end {instance variable or static variable} ---------------
		
		
		// @begin {constructors} -------------------------------------
		
		/// <summary> <japanese>
		/// ftHgRXgN^łB
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aFileName">a <code>String</code> value
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public OhScfTotalSystem(String aFileName):base()
		{
			_receivePorts = null;
			_sendPorts = null;
			_elements = null;
			_iterators = null;
			_scfFile = null;
			_className = "OhScfTotalSystem";
			
			if (_objectName != null)
			{
				throw new OhError("\t\"OhScfTotalSystem\", aObjNum(arg) = 0" + NEWLINE + oPrintString("\t"));
			}
			
			_objectName = "totalSystem";
			_objectNumber = 0;
			
			// Read SCF name from "aBr".
			//
			_scfFile = aFileName;
			
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;

            try
            {
                sr = new OhStreamReader(_scfFile, true);

                // Read Numbers part.
                //

                sr.skipWhites();
                _numOfReceivePorts = Int32.Parse(sr.readWord());
                if (_numOfReceivePorts > 0)
                {
                    _receivePorts = new OhPort[_numOfReceivePorts];
                }

                sr.skipWhites();
                _numOfElements = Int32.Parse(sr.readWord());
                if (_numOfElements > 0)
                {
                    _elements = new OhElement[_numOfElements];
                }

                sr.skipWhites();
                _numOfIterators = Int32.Parse(sr.readWord());
                if (_numOfIterators > 0)
                {
                    _iterators = new OhElement[_numOfIterators];
                }

                sr.skipWhites();
                _numberOfIteratorSets = Int32.Parse(sr.readWord());

                sr.skipWhites();
                _numOfSendPorts = Int32.Parse(sr.readWord());
                if (_numOfSendPorts > 0)
                {
                    _sendPorts = new OhPort[_numOfSendPorts];
                }

                // out.println("complete read numbers part.");

                // registerAssembly
                OhAssembly ohymosAssembly = OhAssembly.Default;

                // Read Memory allocation part.
                //

                String header;
                String objectName;
                String className;
                int objectNumber;

                // Read [InputPorts] section.
                //

                sr.skipWhites();
                header = sr.readWord();

                if (header.Equals("[InputPorts]"))
                {
                    int id;
                    for (int i = 0; i < _numOfReceivePorts; i++)
                    {
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();

                        objectName = sr.readWord();
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF Total data error in [InputPorts] section. The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }
                        Assembly a;
                        if (ohymosAssembly.PortAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _receivePorts[i] = OhPort.create(a.GetType("OhymosNet." + className, true));
                            _receivePorts[i].init(objectName, 0, sr);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " doesn't exist in OHyMoS.NET.");
                    }
                }
                else
                {
                    String str = "SCF Total data error: no [InputPorts] section." + NEWLINE;
                    throw new OhError(str);
                }

                // out.println("complete inputports part.");

                // Read [Components] section.
                //

                sr.skipWhites();
                header = sr.readWord();

                if (header.Equals("[Components]"))
                {
                    for (int i = 0; i < _numOfElements; i++)
                    {
                        int id;
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        objectNumber = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF Total data error in [Components] section. " + "The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }
                        Assembly a;
                        if (ohymosAssembly.ElementAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _elements[id] = OhElement.create(a.GetType("OhymosNet." + className, true));
                            _elements[id].init(objectName, objectNumber, sr);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");
                    }
                }
                else
                {
                    String str = "SCF Total data error: " + "no [Components] section." + NEWLINE;
                    throw new OhError(str);
                }

                // out.println("complete components part.");

                // Read [Iterators] section.
                //

                sr.skipWhites();
                header = sr.readWord();

                if (header.Equals("[Iterators]"))
                {
                    for (int i = 0; i < _numOfIterators; i++)
                    {
                        int id;
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        objectNumber = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF Total data error in [Iterators] " + "section. The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }
                        Assembly a;
                        if ( ohymosAssembly.ElementAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _iterators[id] = OhElement.create(a.GetType("OhymosNet." + className, true));
                            _iterators[id].init(objectName, objectNumber, sr);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");

                    }
                }
                else
                {
                    String str = "SCF Total data error: " + "no [Iterators] section." + NEWLINE;

                    throw new OhError(str);
                }

                // out.println("complete iterators part.");

                // Read [OutputPorts] section.
                //

                sr.skipWhites();
                header = sr.readWord();

                if (header.Equals("[OutputPorts]"))
                {
                    for (int i = 0; i < _numOfSendPorts; i++)
                    {
                        int id;
                        int lineLength;
                        sr.skipWhites();
                        id = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        objectName = sr.readWord();
                        sr.skipWhites();
                        lineLength = Int32.Parse(sr.readWord());
                        sr.skipWhites();
                        className = sr.readWord();

                        if (id != i)
                        {
                            StringWriter sw = new StringWriter();
                            sw.Write("SCF Total data error in [SendPorts] " + "section. The id number " + i + " was expected, but it was " + id + "." + NEWLINE);
                            String str = sw.ToString();
                            sw.Close();
                            throw new OhError(str);
                        }
                        Assembly a;
                        if ( ohymosAssembly.PortAssemblyDic.TryGetValue("OhymosNet." + className, out a))
                        {
                            _sendPorts[i] = OhPort.create(a.GetType("OhymosNet." + className, true));
                            _sendPorts[i].init(objectName, lineLength);
                        }
                        else
                            throw new TypeLoadException("\"" + className + "\"" + " does not exist in OHyMoS.NET.");
                    }
                }
                else
                {
                    String str = "SCF Total data error: " + "no [SendPorts] section." + NEWLINE;
                    throw new OhError(str);
                }

                // out.println("complete outputports part.");
            }
            catch (FileNotFoundException e)
            {
                // System.out.println("File not found: " + _scfFile);
                // e.printStackTrace();
                throw new OhError("Fine not found: " + _scfFile);
            }
            catch (TypeLoadException e)
            {
                throw new OhError(e.Message);
            }
            catch (OhError e)
            {
                throw new OhError("SCF Total data error in " + aFileName + NEWLINE + e.Message);
            }
            catch (Exception e)
            {
                throw new OhError("Error when reading " + _scfFile + NEWLINE + e.StackTrace);
            }
			finally
			{
				if (sr != null)
				{
					try
					{
						sr.Close();
					}
					catch (Exception e)
					{
					}
				}
			}
		}
		
		// @end {constructors} ---------------------------------------
		
		// @begin {methods} ------------------------------------------
		
		/// <summary> <japanese>
		/// Mp[qo^܂B
		/// (Ref "OhSubSystem.construct()")
		/// Ă΂܂B
		/// o^ĂȂMp[q͐ڑł܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerInputPorts()
		{
			//; (*7)
			for (int i = 0; i < _numOfReceivePorts; i++)
			{
				register(_receivePorts[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// OhScfTotalSystem.registerOutputPorts()
		/// s܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerReceivePorts()
		{
			//; (*1)
			registerOutputPorts();
		}
		
		
		
		/// <summary> <japanese>
		/// Mp[qo^܂B
		/// OhSubSystem.construct()
		/// Ă΂܂B
		/// o^ĂȂMp[q͐ڑł܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerOutputPorts()
		{
			//; (*8)
			for (int i = 0; i < _numOfSendPorts; i++)
			{
				register(_sendPorts[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// OhScfTotalSystem.registerInputPorts()
		/// s܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerSendPorts()
		{
			//; (*2)
			registerInputPorts();
		}
		
		
		/// <summary> <japanese>
		/// S\vf̂AvZsȂȂ\IuWFNg̃AhX _componentArray ɓo^܂B<p/>
		/// 
		/// o^ĂȂ\IuWFNgɂ́AvZs߂𑗂邱Ƃł܂B<p/>
		/// 
		/// Ⴆ΁AhNXō\IuWFNg abc oϐƂĒ`ꍇA
		/// ̂悤 abc o^܂B
		/// 
		/// <pre>
		/// public void registerComponents() {
		/// register((Object)abc);
		/// }
		/// </pre>
		/// 
		/// ̎AK abc  Object ^ɃLXgĉB<p/>
		/// 
		/// ɂ abc ́AꎞI _objectList Ɏ^ꂽA
		/// ŏII Object ^z _componentArray ɋL܂B
		/// iOhSubSystem.construct() QƁj
		/// 
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerComponents()
		{
			//; (*3)
			for (int i = 0; i < _numOfElements; i++)
			{
				register(_elements[i]);
			}
		}
		
		
		/// <summary> <japanese>
		/// ̃\bh́AS\vf̂AvZsȂvf̃AhX _iteratorArray ɓo^܂B
		/// vZsȂnfVɍ쐬vO}Ã\bh`Ȃ΂Ȃ܂B<p/>
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  registerIterators()
		{
			//; (*4)
			if (_numberOfIteratorSets <= 0)
				return ;
			
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
				sr = new OhStreamReader(_scfFile, true);
			}
			catch (FileNotFoundException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
			
			String strLine = null;
			bool foundP = false;
			
			try
			{
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[RegisterIterators]"))
					{
						foundP = true;
						break;
					}
				}
			}
			catch (IOException e)
			{
			     OhError.WriteStackTrace(e, Console.Error);
			}
			
			if (foundP == false)
			{
				throw new OhError("");
			}
			
			
			try
			{
				for (int m = 0; m < _numberOfIteratorSets; m++)
				{
					int num;
					sr.skipWhites();
					num = Int32.Parse(sr.readWord());
					
					//; _iteratorArray[i] == null ƂȂĂ i TB exit B
					//;
					int i;
					bool flag = false;
					
					for (i = 0; i < _numberOfIteratorSets; i++)
					{
						if (_iteratorArray[i] == null)
						{
							flag = true;
							break;
						}
					}
					
					if (flag == false)
					{
						throw new OhError(oPrintString("    "));
					}
					
					for (int k = 0; k < num; k++)
					{
						int s;
						sr.skipWhites();
						s = Int32.Parse(sr.readWord());
						register(_iterators[s]);
					}
					
					int[] numberOfIteratorsArray = new int[]{_numberOfIterators[i]};
					
					_iteratorArray[i] = makeObjectArray(numberOfIteratorsArray); //; OhElement (*4)
					_numberOfIterators[i] = numberOfIteratorsArray[0];
					
					_objectList.Clear();
				}
			}
			catch (IOException e)
			{
				OhError.WriteStackTrace(e, Console.Error);
			}
		}
		
		
		/// <summary> <japanese>
		/// Šnf̒[q̐ڑ
		/// OhSubSystem.connect()
		/// pĒ`܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  connectPorts()
		{
			//; (*5)
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
				
				sr = new OhStreamReader(_scfFile, true);
				
				String strLine = null;
				bool foundP = false;
				
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[ConnectPorts]"))
					{
						foundP = true;
						break;
					}
				}
				
				if (foundP == false)
				{
					throw new OhError("No [ConnectPorts] section.");
				}
				
				String objNameS = null;
				String objNameR = null;
				int objNumS = 0;
				int objNumR = 0;
				
				while (true)
				{
					// ----------- starts while(true) loop.
					String str;
					sr.skipWhites();
					str = sr.readWord();
					
					if (str[0] == 'z')
					{
						break;
					}
					
					// -- There are nine cases. ---
					
					if (str[0] == 'r')
					{
						// The sender is a receive port.
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord()); // Receive port ID.
						
						if (i >= _numOfReceivePorts)
						{
							StringWriter sw = new StringWriter();
							sw.Write(_scfFile + ": Illegal receive port number: r " + i);
							String errMsg = sw.ToString();
							sw.Close();
							throw new OhError(errMsg);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal element number: c " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_receivePorts[i], _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal iterator element number: i " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_receivePorts[i], _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal output port number: s " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_receivePorts[i], _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else if (str[0] == 'c')
					{
						// The sender is a usual component.
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord());
						sr.skipWhites();
						objNameS = sr.readWord();
						sr.skipWhites();
						objNumS = Int32.Parse(sr.readWord());
						
						if (i >= _numOfElements)
						{
							StringWriter sw = new StringWriter();
							sw.Write(_scfFile + ": Illegal element number: c " + i);
							String errMsg = sw.ToString();
							sw.Close();
							throw new OhError(errMsg);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal element number: c " + i); //; j ̊ԈႢł́HHH
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_elements[i], objNameS, objNumS, _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal iterator element number: i " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_elements[i], objNameS, objNumS, _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal output port number: s " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_elements[i], objNameS, objNumS, _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else if (str[0] == 'i')
					{
						// The sender is an iterative component.
						int i;
						sr.skipWhites();
						i = Int32.Parse(sr.readWord());
						sr.skipWhites();
						objNameS = sr.readWord();
						sr.skipWhites();
						objNumS = Int32.Parse(sr.readWord());
						
						if (i >= _numOfIterators)
						{
							StringWriter sw = new StringWriter();
							sw.Write(_scfFile + ": Illegal iterator element number: i " + i);
							String errMsg = sw.ToString();
							sw.Close();
							throw new OhError(errMsg);
						}
						
						// The receiver.
						//
						String strR;
						sr.skipWhites();
						strR = sr.readWord();
						
						int j;
						
						char r = strR[0];
						
						switch (r)
						{
							
							
							case 'c':  // The receiver is a usual component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfElements)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal element number: c " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_iterators[i], objNameS, objNumS, _elements[j], objNameR, objNumR);
								
								break;
							
							
							case 'i':  // The receiver is an iterative component.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								sr.skipWhites();
								objNameR = sr.readWord();
								sr.skipWhites();
								objNumR = Int32.Parse(sr.readWord());
								
								if (j >= _numOfIterators)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal iterator element number: i " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect(_iterators[i], objNameS, objNumS, _iterators[j], objNameR, objNumR);
								
								break;
							
							
							case 's':  // The receiver is a send port.
								sr.skipWhites();
								j = Int32.Parse(sr.readWord());
								
								if (j >= _numOfSendPorts)
								{
									StringWriter sw = new StringWriter();
									sw.Write(_scfFile + ": Illegal output port number: s " + j);
									String errMsg = sw.ToString();
									sw.Close();
									throw new OhError(errMsg);
								}
								
								connect((OhElement) _iterators[i], objNameS, objNumS, (OhPort) _sendPorts[j]);
								
								break;
							
							
							default: 
								throw new OhError("");
							
						}
					}
					else
					{
						throw new OhError("");
					}
				} // -------------------------------------------------------------- ends while(true) loop.
			}
			catch (OhError e)
			{
				throw e;
			}
			catch (FileNotFoundException e)
			{
				throw new OhError(e.StackTrace);
			}
			catch (IOException e)
			{
				throw new OhError(e.StackTrace);
			}
			finally
			{
				if (sr != null)
				{
					try
					{
						sr.Close();
					}
					catch (Exception e)
					{
					}
				}
			}
		}
		
		
		/// <summary> <japanese>
		/// nf̒ڒʐMڑ
		/// OhElement.register(Object, int)
		/// pĒ`܂B
		/// 
		/// <pre>
		/// [ConnectComponents]
		/// c 2 c 3 0 # mElmnt[2]->Register((void*)mElmnt[3], 0)
		/// i 0 i 1 2 # mIte[0]->Register((void*)mIte[1], 2);
		/// z # end mark
		/// </pre>
		/// </japanese>
		/// 
		/// </summary>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  connectComponents()
		{
			//; (*6)
			// Open the SCF. Comment strings are removed.
			//
			OhStreamReader sr = null;
			
			try
			{
				sr = new OhStreamReader(_scfFile, true);
				
				String strLine = null;
				bool foundP = false;
				
				sr.skipWhites();
				while ((strLine = sr.readWord()) != null)
				{
					if (strLine.Equals("[ConnectComponents]"))
					{
						foundP = true;
						break;
					}
				}
				
				if (foundP == false)
				{
					throw new OhError("No [ConnectComponents] section.");
				}
				
				while (true)
				{
					String strCom1;
					String strCom2;
					int i1;
					int i2;
					int j;
					
					sr.skipWhites();
					strCom1 = sr.readWord();
					
					if (strCom1[0] == 'z')
					{
						break;
					}
					
					sr.skipWhites();
					i1 = Int32.Parse(sr.readWord());
					sr.skipWhites();
					strCom2 = sr.readWord();
					sr.skipWhites();
					i2 = Int32.Parse(sr.readWord());
					sr.skipWhites();
					j = Int32.Parse(sr.readWord());
					
					OhElement elm1;
					OhElement elm2;
					
					elm1 = (strCom1[0] == 'c')?_elements[i1]:_iterators[i1];
					elm2 = (strCom2[0] == 'c')?_elements[i2]:_iterators[i2];
					
					elm1.register(elm2, j);
				}
			}
			catch (FileNotFoundException e)
			{
                
				throw new OhError(e.StackTrace,"SCF file does not found.");
			}
			catch (IOException e)
			{
				throw new OhError(e.StackTrace,"An error occurs.");
			}
			finally
			{
				if (sr != null)
				{
					try
					{
						sr.Close();
					}
					catch (Exception e)
					{
					}
				}
			}
		}
		
		
		/// <summary> <japanese>
		/// ̓Xg[ aBr  _currentTime ǂݍ݂܂B
		/// t@C _currentTime ǂݎꍇقƂǂƎv܂B
		/// </japanese>
		/// <p/>
		/// Set initial state by reading a file.
		/// 
		/// </summary>
		/// <param name="aSr">an <code>OhStreamReader</code> value
		/// </param>
		/// <exception cref="OhError">if an error occurs
		/// </exception>
		public override void  setInitialState(OhStreamReader aSr)
		{
			//; (*9)
			int year = 0;
			long tsec = 0L;
			double frac = 0.0;
			
			try
			{
				aSr.skipWhites();
				year = Int32.Parse(aSr.readWord());
				tsec = Int64.Parse(aSr.readWord());
				frac = Double.Parse(aSr.readWord());
			}
			catch (IOException e)
			{
				throw new OhError("Error in OhScfTotalSystem.setInitialState().", oPrintString("\t"));
			}
			catch (System.FormatException e)
			{
				throw new OhError("Error in OhScfTotalSystem.setInitialState().", oPrintString("\t"));
			}
			
			if (year > 0)
			{
				_currentTime = new OhTimePoint(year, tsec);
			}
		}
		
		// @end {methods} --------------------------------------------
	} // OhScfSubSystem
}