﻿/// <(#)>  OhSendInt; a port which sends datapacks of type int. </(#)>
using System;
using System.IO;
using System.Collections;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> 
    /// int 型のデータパックを送信する送信端子のクラスです。 <p/>
    /// このクラスは<a href="OhymosNet.OhSendPort.html">OhSendPort</a>を
    /// 継承しています。<p/>
    /// The OhSendInt class is a base class to express ports which
    /// send int data.
    /// It extends <a href="OhymosNet.OhSendPort.html">OhSendPort</a> class.
    /// </summary>
    public class OhSendInt : OhSendPort
    {

        // @begin{initialization} ------------------------------------

        /// <summary> <japanese>
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号を初期化し、
        /// データの補間は折れ線に設定します。
        /// 具体的には、
        /// _dctVec の第一要素に
        /// <a href="DataConnectionType.html#BROKENLINE">DataConnectionType.BROKENLINE</a> を格納します。
        /// </japanese>
        /// <p/>
        /// Initialization.
        /// 
        /// </summary>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <exception cref="OhError">すでに初期化している場合
        /// </exception>
        public override void init(String aPortName, int aPortNum)
        {
            base.init(aPortName, aPortNum);
        }

        /// <summary> <japanese>
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号、
        /// データ補間型配列リストを初期化します。
        /// </japanese>
        /// <p/>
        /// Initialization.
        /// 
        /// </summary>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <param name="aDctVec">a <var>ArrayList</var> value<p/>
        /// データ補間型配列リスト
        /// </param>
        /// <exception cref="OhError">すでに初期化している場合
        /// </exception>
        public override void init(String aPortName, int aPortNum, ArrayList aDctVec)
        {
            base.init(aPortName, aPortNum, aDctVec);
        }

        /// <summary> 
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号を初期化し、
        /// データ補間型指定してデータ補間型配列リストを初期化します。
        /// 
        /// Initialization.
        /// 
        /// </summary>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <param name="aDct">a <var>DataConnectionType</var> value<p/>
        /// データ補間型
        /// </param>
        /// <exception cref="OhError">すでに初期化している場合
        /// </exception>
        public void init(String aPortName, int aPortNum, DataConnectionType aDct)
        {
            base.init(aPortName, aPortNum);
            _dctVec = ArrayList.Synchronized(new ArrayList());
            _dctVec.Add(aDct);
        }

        // @end{initialization} --------------------------------------


        // @begin {constructors} -------------------------------------

        /// <summary> 
        /// デフォルトコンストラクタです。<p/>
        /// Creates a new default <var>OhSendInt</var> instance.
        /// </summary>
        /// <remarks>
        /// デフォルトコンストラクタは
        /// <a href="OhSendInt.html#init(String, int)">OhSendInt.init(String, int)</a> を実行しません。
        /// その結果、以下の設定となります。<p/>
        /// 
        /// <dl>
        /// <li> _objectName        : null</li>
        /// <li> _objectNumber      : -1</li>
        /// <li> _dctVec            : null</li>
        /// </dl>
        /// 
        /// <p/>
        /// その他のメンバ変数に対する処理は引数つきコンストラクタと同じです。
        /// この場合、<a href="OhSendInt.html#init(String, int)">OhSendInt.init(String, int)</a> を
        /// 用いてメンバ変数を初期化して下さい。
        /// </remarks>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhSendInt()
            : base()
        {
            _className = "OhSendInt";
        }

        /// <summary> 
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号を指定して、
        /// OhSendInt クラスのインスタンスを生成します。<p/>
        /// Creates a new <var>OhSendInt</var> instance.
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhSendPort.html#OhSendPort.OhSendPort(String, int)">OhSendPort.OhSendPort(String, int)</a>
        /// のコンストラクタを用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhSendInt"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _owner             : null</li>
        /// <li> _bufferList        : メモリに確保する。（接続作業用）</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// <li> _dataPackArray     : null</li>
        /// <li> _numberOfDataPacks : -1</li>
        /// <li> _dctVec            : null</li>
        /// </dl>
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhSendInt(String aPortName, int aPortNum)
            : base(aPortName, aPortNum)
        {
            _className = "OhSendInt";
        }

        /// <summary> 
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号、
        /// データ補間型配列リストを指定して、
        /// OhSendInt クラスのインスタンスを生成します。<p/>
        /// Creates a new <var>OhSendInt</var> instance.
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhSendPort.html#OhSendPort.OhSendPort(String, int)">OhSendPort.OhSendPort(String, int)</a>
        /// のコンストラクタを用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhSendInt"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _owner             : null</li>
        /// <li> _bufferList        : メモリに確保する。（接続作業用）</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// <li> _dataPackArray     : null</li>
        /// <li> _numberOfDataPacks : -1</li>
        /// <li> _dctVec            : aDctVec</li>
        /// </dl>
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <param name="aDctVec">a <var>ArrayList</var> value<p/>
        /// データ補間型配列リスト
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public OhSendInt(String aPortName, int aPortNum, ArrayList aDctVec)
            : base(aPortName, aPortNum, aDctVec)
        {
            _className = "OhSendInt";
        }

        /// <summary> 
        /// オブジェクト（送信端子）名、オブジェクト（送信端子）番号、
        /// データ補間型を指定して、
        /// OhSendInt クラスのインスタンスを生成します。<p/>
        /// Creates a new <var>OhSendInt</var> instance.
        /// </summary>
        /// <remarks>
        /// 基底クラスのコンストラクタ
        /// <a href="OhSendPort.html#OhSendPort.OhSendPort(String, int)">OhSendPort.OhSendPort(String, int)</a>
        /// のコンストラクタを用いてメンバ変数を以下の通り初期化します。<p/>
        /// 
        /// <dl>
        /// <li> _className         : "OhSendInt"</li>
        /// <li> _objectName        : aPortName</li>
        /// <li> _objectNumber      : aPortNum</li>
        /// <li> _owner             : null</li>
        /// <li> _bufferList        : メモリに確保する。（接続作業用）</li>
        /// <li> _receiverArray     : null</li>
        /// <li> _numberOfReceivers : -1</li>
        /// <li> _dataPackArray     : null</li>
        /// <li> _numberOfDataPacks : -1</li>
        /// <li> _dctVec            : aDctで指定した補間型の配列リスト</li>
        /// </dl>
        /// </remarks>
        /// <param name="aPortName">a <var>String</var> value<p/>
        /// 送信端子名
        /// </param>
        /// <param name="aPortNum">an <var>int</var> value<p/>
        /// 送信端子番号
        /// </param>
        /// <param name="aDct">a <var>DataConnectionTyper</var> value<p/>
        /// データ補間型
        /// </param>
        public OhSendInt(String aPortName, int aPortNum, DataConnectionType aDct)
            : base(aPortName, aPortNum)
        {
            _dctVec = ArrayList.Synchronized(new ArrayList());
            _dctVec.Add(aDct);
            _className = "OhSendInt";
        }

        // @end {constructors} ---------------------------------------

        // @begin {property}----------------------------------------------------
        /// <summary> 
        /// データ補間型を指定します。<p/>
        /// Sets the data connection type.
        /// </summary>
        /// <remarks>
        /// データ補間型を指定します。データ補間型配列を指定する必要は
        /// ありません。引数で指定したデータ補間型オブジェクトを持つ配
        /// 列をメソッド内部で生成して記憶します。<p/>
        /// 
        /// データ補間型はコンストラクタやinitメソッドでも指定できます。<br/>
        /// 
        /// Sets the data connection type. You can specify the data
        /// connection type directly and do not need give a data
        /// connection type vector.<p/>
        /// </remarks>
        public virtual DataConnectionType DataConnectionType
        {
            set
            {
                _dctVec = ArrayList.Synchronized(new ArrayList());
                _dctVec.Add(value);
            }

        }
        // @end {property}----------------------------------------------------

        // @begin {methods} ------------------------------------------

        // 1. function for sending data.

        /// <summary> 
        /// 送信するデータパックのデータ値の型 int を表す文字列 "int" を取得します。
        /// <p/>
        /// Gets the data pack type which expresses the type of data to be received.
        /// 
        /// </summary>
        public override String DataType
        {
            get { return "int"; }
        }


        /// <summary> 
        /// 引数にとった aT, aValue から
        /// <a href="OhDataPackInt.html">OhDataPackInt</a> 型データパックを作成し、それを
        /// <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> を用いて
        /// 送信端子に送信します。<p/>
        /// 
        /// Send data whose data time and data value are <b>aT</b> and
        /// <b>aValue</b>, respectively.
        /// </summary>
        /// 
        /// <param name="aT">an <var>OhTimePointImmutable</var> value<p/>
        /// データ時刻
        /// </param>
        /// <param name="aValue">a <var>int</var> value<p/>
        /// データ値
        /// </param>
        public virtual void sendData(IOhTimePointImmutable aT, int aValue)
        {
            dataPackStock(new OhDataPackInt(aT, aValue));
        }

        /// <summary> 
        /// 引数にとった文字列 aDataStr で表されたデータパックから
        /// <a href="OhDataPackInt.html">OhDataPackInt</a> 型データパックを作成し、それを
        /// <a href="OhSendPort.html#dataPackStock(Object)">OhSendPort.dataPackStock(Object)</a> を用いて
        /// 送信端子に送信します。<p/>
        /// Send data of type int whose value<p/> is stored in the
        /// String specified.
        /// </summary>
        /// <example><code>
        /// sendByString("2003 10 0.0 6");
        /// </code></example>
        /// 
        /// <param name="aDataStr">a <var>String</var> value<p/>
        /// データ文字列
        /// </param>
        public override void sendByString(String aDataStr)
        {
            dataPackStock(new OhDataPackInt(aDataStr));
        }

        /// <summary> 
        /// このポートのクラス名とデータ型に関する情報を文字列にして返
        /// します。<p/>
        /// Returns the basic info about this port class such as
        /// classname and datatype it carries.
        /// </summary>
        /// <remarks>
        /// 複数行文字列ですが、最後の行には行末コードがありま
        /// せん。
        /// 
        /// The string returned contains multiple lines, but the last line does not contain 
        /// line-end character.
        /// </remarks>
        /// 
        /// <returns> a <var>String</var> value<p/>
        /// このポートに関する情報
        /// </returns>
        public override String basicInfo()
        {
            return "classname: " + _className + NEWLINE + "datatype it carries: " + DataType;
        }

        // @end {methods} --------------------------------------------
    } // OhSendInt
}
