/// @(#) class OhTime
///
/// created: 2004/01/04
/// Modified and Tested by Naoto Ohshima. Date:2007/10/30 20:06:18 
/// Development of the OhTime classs<p/>
using System;
using OhymosNet;

namespace OhymosNet
{
	
	/// <summary> 
    /// bPʂ̎Ԃ\NXłB
    /// </summary>
    /// <remarks>
    /// <p/>
    /// OhTime IuWFNǵAԂbPʂŕ\킵܂BOhTime ́A
    /// Ԃ\킷߂ <b>sec</b>  <b>frac</b> 2̃tB[
    /// hgp܂B<b>sec</b> ́AAfrac ́A
    /// \킵܂Bfrac ́A-0.5 ȏA0.5 Ƃ܂B<p/>
    /// 
    /// Ⴆ΁A10.7 b́A<b>sec</b> = 11, <b>frac</b> = -0.3 ƕ\
    /// ܂B10.3 b́A<b>sec</b> = 10, <b>frac</b> = 0.3 ƕ\
    /// 킳܂B<p/><br/>
    /// 
    /// An OhTime object stands for an amount of time in seconds. An
	/// OhTime object has two field variables: <b>sec</b> and
	/// <b>frac</b>. <b>sec</b> is the integer part of the time, and
	/// <b>frac</b> is the fractional part of the time. <b>frac</b> is
	/// greater than or equal to -0.5 and less than 0.5.  In other
	/// words, <b>sec</b> is the closest long to <b>sec</b> +
	/// <b>frac</b>. <p/>
	/// 
	/// For example, 10.7 seconds are stood for by <b>sec</b> =
	/// 11 and <b>frac</b> = -0.3 and 10.4 seconds are stood for
	/// by <b>sec</b> = 10 and <b>frac</b> = 0.4<p/>
	/// 
    /// </remarks>
	public class OhTime : ICloneable, IOhTimeImmutable
	{
		/// <summary> integer part of time.
		/// 
		/// <japanese>
		/// <p/>
		/// ԂbPʂŕ\Ƃ̐łB
		/// </japanese>
		/// </summary>
		private long _sec;
		
		/// <summary> Negligibly small time amount whose default value is 0.0001
		/// sec.  When the absolute value of an amount of time is less
		/// than this value, the time is regarded as negligibly small.
		/// 
		/// <japanese>
		/// <p/>
		/// ł鎞ԁBΒl̒l菬Ԃ́Ał
		/// 炢Ƃ܂B10000 1bɂ܂B
		/// </japanese>
		/// </summary>
		public readonly static double NEGLIGIBLY_SMALL_TIME = 0.0001;
		// on the second time scale
		
		/// <summary> OhTime object with sec = 0 and frac = 0.
		/// 
		/// <japanese>
		/// sec  frac  0  OhTime IuWFNgB
		/// </japanese>
		/// 
		/// </summary>
		public static OhTime ZERO = new OhTime(0, 0.0);
		
		
		/// <summary> fraction part of time
		/// 
		/// <japanese>
		/// <p/>
		/// ԂbPʂŕ\ƂȀB-0.5 ȏA0.5 
		/// Ƃ܂B
		/// </japanese>
		/// </summary>
		private double _frac;
		
		/// <summary> Constructs a new OhTime object with <b>sec</b> 0 and
		/// <b>frac</b> 0.0.
		/// 
		/// <japanese>
		/// <p/>
		/// OhTime IuWFNg𐶐܂BԂ0ɐݒ肳܂B
		/// </japanese>
		/// </summary>
		public OhTime()
		{
			_sec = 0; _frac = 0.0;
		}
		
		/// <summary> Constructs a new OhTime object by setting <b>sec</b>
		/// to <b>aSec</b> and <b>frac</b> to <b>aFrac</b>.<p/>
		/// 
		/// <japanese>
		/// <p/>
		/// w肵 <b>sec</b>, <b>frac</b>  OhTime IuWFNg
		/// 𐶐܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aSec">  integer part of time
		/// </param>
		/// <param name="aFrac"> fraction part of time
		/// </param>
		public OhTime(long aSec, double aFrac)
		{
			_sec = aSec;
			_frac = aFrac;
			long n = (long) Math.Round(_frac);
			_sec += n;
			_frac = _frac - n;
		}
		
		/// <summary> Constructs a new OhTime object which is the same time
		/// as <b>aTime</b>
		/// 
		/// </summary>
		/// <param name="aTime">an <var>IOhTimeImmutable</var> value
		/// </param>
		public OhTime(IOhTimeImmutable aTime)
		{
			_sec = aTime.Sec;
			_frac = aTime.Frac;
		}
		
		/// <summary> Constructs a new <var>OhTime</var> instance from a
		/// double.
		/// 
		/// <japanese>
		/// <p/>
		/// double ̒l aTime  OhTime IuWFNg𐶐܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTime">a <var>double</var> value
		/// </param>
		public OhTime(double aTime)
		{
			_sec = (long) Math.Round(aTime);
			_frac = aTime - _sec;
		}
		
		/// <summary> 
        /// Set <b>sec</b> (integer part of time) to aSec.
		/// 
		/// <japanese>
		/// <p/>
		///  <b>sec</b> 擾Eݒ肵܂B
		/// </japanese>
		/// 
		/// </summary>
		public long Sec
		{
            get { return _sec;}
			set { _sec = value;}
		}
		
		/// <summary> Set <b>frac</b> (fraction part of time) to aFrac.
		/// 
		/// <japanese>
		/// <p/>
		/// [ <b>frac</b> 擾Eݒ肵܂B
		/// </japanese>
		/// 
		/// </summary>
		public double Frac
		{
			get { return _frac;}
            set { _frac = value; }
		}
		
		
		/// <summary> Adjust <b>sec</b> and <b>frac</b> so that they have compatible signs
		/// and the absolute value of <b>frac</b> is less than 1.
		/// 
		/// <japanese>
		/// <p/>
		/// <b>sec</b>  <b>frac</b> ̕܂ĂāA[ <b>frac</b> 
		/// -0.5 ȏA0.5 ɂȂ悤ɒ܂B
		/// </japanese>
		/// </summary>
		public  void  adjust()
		{
			long n = (long) Math.Round(_frac);
			_sec += n;
			_frac = _frac - n;
		}
		
		/// <summary> Returns an OhTime object whose time value is equal to
		/// aSec + aFrac.<p/>
		/// 
		/// aFrac need not to be greater than or equal to -0.5 and less
		/// than 0.5.<p/>
		/// 
		/// <japanese>
		/// <p/>
		/// aSec + aFrac ɓԂ OhTime IuWFNgԂ
		/// B aFrac ́A-0.5 ȏA0.5 łȂĂ\܂
		/// B̃\bh̒(OhTimeRXgN^)Œ܂B<p/>
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aSec">a <var>long</var> value
		/// </param>
		/// <param name="aFrac">a <var>double</var> value
		/// </param>
		/// <returns> an <var>OhTime</var> value
		/// </returns>
		public static OhTime adjust(long aSec, double aFrac)
		{
			return new OhTime(aSec, aFrac);
		}
		
		/// <summary> Add the current OhTime object with aTime seconds.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNgɁAaTime b܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTime">a <var>double</var> value
		/// </param>
		public  void  add(double aTime)
		{
			long sec = (long) Math.Round(aTime);
			double frac = aTime - sec;
			_sec += sec;
			_frac += frac;
			this.adjust();
		}
		
		/// <summary> Add the current OhTime object with aT.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNg OhTime IuWFNg aT 
		/// ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aT">an <var>OhTime</var> value
		/// </param>
		public  void  add(OhTime aT)
		{
			_sec += aT.Sec;
			_frac += aT.Frac;
			this.adjust();
		}
		
		/// <summary> Add the current OhTime object with (aSec + aFrac) seconds.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNgɁA(aSec + aFrac) b܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aSec">a <var>long</var> value
		/// </param>
		/// <param name="aFrac">a <var>double</var> value
		/// </param>
		public  void  add(long aSec, double aFrac)
		{
			_sec += aSec;
			_frac += aFrac;
			this.adjust();
		}
		
		/// <summary> Returns an OhTime object by adding two OhTime objects aTa
		/// and aTb.
		/// 
		/// <japanese>
		/// <p/>
		///  OhTime IuWFNg aTa  aTb 𑫂 OhTime IuWFNg 
		/// ĕԂ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTa">an <var>OhTime</var> value
		/// </param>
		/// <param name="aTb">an <var>OhTime</var> value
		/// </param>
		/// <returns> an <var>OhTime</var> value
		/// </returns>
		public static OhTime add(OhTime aTa, OhTime aTb)
		{
			long sec = aTa.Sec + aTb.Sec;
			double frac = aTa.Frac + aTb.Frac;
			return adjust(sec, frac);
		}
		
		/// <summary> Subtract aTime seconds fromt the current OhTime object.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNgAaTime b܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTime">a <var>double</var> value
		/// </param>
		public  void  sub(double aTime)
		{
			long sec = (long) Math.Round(aTime);
			double frac = aTime - sec;
			_sec -= sec;
			_frac -= frac;
			this.adjust();
		}
		
		/// <summary> Subtract an OhObject aT from the current OhObject.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNg OhTime IuWFNg aT 
		/// ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aT">an <var>OhTime</var> value
		/// </param>
		public  void  sub(OhTime aT)
		{
			_sec -= aT.Sec;
			_frac -= aT.Frac;
			this.adjust();
		}
		
		/// <summary> Returns an OhTime object by subtracting an OhTime object aTb
		/// from an OhTime object aTa.
		/// 
		/// <japanese>
		/// <p/>
		/// OhTime IuWFNg aTa  OhTime IuWFNg aTb ̍ł
		/// OhTime IuWFNgԂ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTa">an <var>OhTime</var> value
		/// </param>
		/// <param name="aTb">an <var>OhTime</var> value
		/// </param>
		/// <returns> an <var>OhTime</var> value
		/// </returns>
		public static OhTime sub(OhTime aTa, OhTime aTb)
		{
			long sec = aTa.Sec - aTb.Sec;
			double frac = aTa.Frac - aTb.Frac;
			return adjust(sec, frac);
		}
		
		/// <summary> Multiplies the current OhTime object by aI.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNg aI {܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aI">an <var>int</var> value
		/// </param>
		public  void  mul(int aI)
		{
			_sec = _sec * aI;
			_frac = _frac * aI;
			this.adjust();
		}
		
		/// <summary> Returns the product of an OhTime object aTime and an
		/// integer aI.
		/// 
		/// <japanese>
		/// <p/>
		/// OhTime IuWFNg aTime Ɛ aI ̐ς OhTime IuWF
		/// NgƂĕԂ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTime">an <var>OhTime</var> value
		/// </param>
		/// <param name="aI">an <var>int</var> value
		/// </param>
		/// <returns> an <var>OhTime</var> value
		/// </returns>
		public static OhTime mul(OhTime aTime, int aI)
		{
			long sec = aTime.Sec * aI;
			double frac = aTime.Frac * aI;
			return adjust(sec, frac);
		}
		
		/// <summary> Copies the field varibles of the source OhTime object into
		/// those of the current OhTime object.
		/// 
		/// <japanese>
		/// <p/>
		/// Ŏw肳ꂽ IOhTimeImmutable IuWFNg̃tB[h
		/// ݂ OhTime IuWFNg̃tB[hϐɕʂ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="aTime">an <var>IOhTimeImmutable</var> value
		/// </param>
		public void  copy(IOhTimeImmutable aTime)
		{
			_sec = aTime.Sec;
			_frac = aTime.Frac;
		}
		
		/// <summary> Returns the double value which is equal to <b>sec</b> +
		/// <b>frac</b>.
		/// 
		/// <japanese>
		/// <p/>
		/// ݂ OhTime IuWFNgɑΉ鎞Ԃ double lԂ
		/// ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <returns> a <var>double</var> value
		/// </returns>
		public  double toDouble()
		{
			return _frac + _sec;
		}
		
		/// <summary> Returns the string expression of the current object.<p/>
		/// 
		/// <japanese>
		/// <p/>
		/// IuWFNg String \Ԃ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <returns> a <var>String</var> value
		/// </returns>
		public override String ToString()
		{
			return _sec.ToString() + " " + _frac.ToString();
		}
		
		/// <summary> Returns a shallow copy of the current OhTime object
		/// 
		/// </summary>
		/// <returns> an <var>Object</var> value
		/// </returns>
		public  Object Clone()
		{
			try
			{
				OhTime ot = (OhTime) base.MemberwiseClone();
				return ot;
			}
			catch (Exception e)
			{
				throw new ApplicationException("This should never happen!");
			}
		}
		
		/// <summary> Check whether the current IOhTimeImmutable object is equal
		/// to <b>anObj</b>.
		/// <p/>
		/// 
		/// <japanese>
		/// Ŏw肳ꂽIuWFNgAIOhTimeImmutable ^ (܂
		/// ̃TuNX) ̃IuWFNgłāA\킵Ă鎞
		/// ỸIuWFNgƓł邩 (true) قȂ邩 (false) 
		/// ܂B
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="anObj">an <var>Object</var> value
		/// </param>
		/// <returns> a <var>boolean</var> value
		/// </returns>
		public new bool Equals(Object anObj)
		{
			if ((anObj != null) && (anObj is IOhTimeImmutable))
			{
				if (((OhTime) anObj)._sec == _sec && ((OhTime) anObj)._frac == _frac)
					return true;
			}
			return false;
		}
		
		/// <summary> Check whether the time difference between the current time
		/// point object and <b>anObj</b> is less than <b>anEps</b>.
		/// <p/>
		/// 
		/// <japanese>
		/// ݂ OhTimePointImmutable IuWFNg <b>anObj</b> 
		/// \킷Ƃ̎ԍ <b>anEps</b> 菬 (true)A
		/// łȂ (false) Ԃ܂B
		/// <p/>
		/// </japanese>
		/// 
		/// </summary>
		/// <param name="anObj">an <var>Object</var> value
		/// </param>
		/// <param name="anEps">a <var>double</var> value
		/// </param>
		/// <returns> a <var>boolean</var> value
		/// </returns>
		public bool equals(Object anObj, double anEps)
		{
			if ((anObj != null) && (anObj is IOhTimeImmutable))
			{
				if (((OhTime) anObj)._sec == _sec && (Math.Abs(((OhTime) anObj)._frac - _frac) < anEps))
					return true;
			}
			return false;
		}

	}
}