/// <(#)>   PowerLawKinematicWave; class to express Power Law Kinematic Wave model for Stream Flow. </(#)>
/// <summary> 
/// </summary>
using System;
using System.IO;
using OhymosNet;

namespace OhymosNet
{


    /// <summary> 
    /// ͓Ԃ̗w@ kinematic wave @ŒǐՂ邽߂̃NXłB<p/>
    /// vffłB<p/>
    /// PowerLawKinematicWave;
    /// Power Law Kinematic Wave model for Stream Flow.
    /// </summary>
    /// <remarks>
    /// ͓Ԃ̗w@ kinematic wave @ŒǐՂ܂B
    /// ŁA͓ԂƂ͂̓ɍ_ȂP{͓̉Ԃ̂Ƃ܂B
    ///  Q ́Aʐfʐ A ̎w֐ a*A^m ŕ\ł̂Ƃ܂B<p/>
    /// </remarks>
    /*
    y 1. NX OhElement pBz*/
    public class PowerLawKinematicWave : OhElement
    {
        // @begin {fields} -----------------------------------------------------

        /*
        y 2. M[qEM[qA܂͎M[qEM[q̔zoϐƂĒ`Bz*/
        /// <summary> Structural variable:
        /// number of inflows from the upstream ends.
        /// _numOfUpIn >= 0.
        /// 
        /// </summary>
        protected internal int _numOfUpIn;

        /// <summary> Structural variable:
        /// number of lateral inflows.
        /// _numOfLateralIn >= 0.
        /// 
        /// </summary>
        protected internal int _numOfLateralIn;

        /// <summary> inflow ports from the upstream end.
        /// 
        /// </summary>
        protected internal OhReceiveDouble[] _rpUpIn;

        /// <summary> lateral inflow ports.
        /// Lateral inflow rate per unit length of stream will be calculated
        /// by multiplying constant _cLateralIn[i] with the value obtained from _rpLateralIn[i]
        /// 
        /// </summary>
        protected internal OhReceiveDouble[] _rpLateralIn;

        /// <summary> Send port.
        /// Runoff discharge (m^3/sec) at the slopeend or the stream end. 
        /// 
        /// <japanese>
        /// <p/>
        /// M[q͂PłB
        /// f[^ <a href="OhDataPackDouble.html">OhDataPackDouble</a> ^łB
        /// f[^Ԍ^
        /// <a href="DataConnectionType.html#BROKENLINE">DataConnectionType.BROKENLINE</a>
        /// łB<p/>
        /// 
        /// ͓[̗oʂ܂B
        /// vZꂽoʂ̎ [m^3/sec] łB
        /// </japanese>
        /// 
        /// </summary>
        protected internal OhSendDouble _sp;

        /*
        y 3. p^Eԗʂ̋LɗpϐoϐƂĒ`Bz*/
        // Parameters.

        /// <summary> Parameter:
        /// kinematic constant <b>a</b> [m/sec] in
        /// 
        /// <pre>
        /// Q = a*A^m
        /// </pre>
        /// 
        /// </summary>
        protected internal double _a;

        /// <summary> Parameter:
        /// kinematic constant <b>m</b> [-] in
        /// 
        /// <pre>
        /// Q = a*A^m
        /// </pre>
        /// 
        /// </summary>
        protected internal double _m;

        /// <summary> Parameter:
        /// length of the stream segment. [m]
        /// 
        /// <japanese>
        /// <p/>
        /// ͓Ԃ̒B[m]
        /// </japanese>
        /// 
        /// </summary>
        protected internal double _lengthOfStream;

        /// <summary> Parameter:
        /// number of divisions of the stream. [-]
        /// 
        /// <japanese>
        /// <p/>
        /// ͓Ԃ𓙕B[-]
        /// </japanese>
        /// 
        /// </summary>
        protected internal int _numOfSepStream;

        /// <summary> Parameter:
        /// time step [sec] for finite difference calculation.
        /// Time step is fixed in this calculation model.
        /// 
        /// <japanese>
        /// <p/>
        /// vZԊԊuB[sec] <p/>
        /// 
        /// ̃fł͌ŒlƂĈ܂B
        /// </japanese>
        /// 
        /// </summary>
        protected internal int _dt;

        /// <summary> Parameter:
        /// time weighting parameter [-].
        /// The standard value of theta is 0.5. 
        /// If theta = 0, the finite difference calculation scheme becomes an explicit one.
        /// 
        /// </summary>
        protected internal double _theta;

        /// <summary> Parameter:
        /// tolerance level of diffrence of two consecutive caluculated discharges to stop the iteration. 
        /// 
        /// </summary>
        protected internal double _epsilon;

        /// <summary> Parameter:
        /// tolerance level of diffrence of two consecutive caluculated discharges to stop the iteration. 
        /// 
        /// </summary>
        protected internal double _qLevel;

        /// <summary> Parameter:
        /// _cUpIn[i] is a multiplication factor [-] to the i-th inflow port from upstream end, _rpUpIn[i].
        /// 
        /// </summary>
        protected internal double[] _cUpIn;

        /// <summary> Parameter:
        /// _cLateralIn[i] is a multiplication factor [-] to the i-th lateral inflow port _rpLateralIn[i].
        /// 
        /// </summary>
        protected internal double[] _cLateralIn;


        // State variable.

        /// <summary> State variable:
        /// discharge Q [m^3/sec] ( = q * w) at current time.
        /// 
        /// </summary>
        protected internal double[] _qCrnt;


        // Working variables.

        /// <summary> Working variable:
        /// length of the division. [m]
        /// 
        /// <pre>
        /// _dx = _lengthOfStream / _numOfSepStream
        /// </pre>
        /// 
        /// </summary>
        protected internal double _dx;

        /// <summary> Working variable:
        /// discharge Q [m^3/sec] at next time.
        /// 
        /// </summary>
        protected internal double[] _qNext;

        /// <summary> <japanese>
        /// ł̗ʂ̕zǂ̂悤ɗ^邩w肷ϐB
        /// ONLY_DOWN_STREAM_END ȂA[ŗʂ^āA͓}B
        /// EVERY_SECTION ȂAx = _dx, 2*_dx, ..., _numOfSepStream * _dx ̕_ŗʂ^B
        /// </japanese>
        /// <p/>
        /// Supplemental variable for setting initial state.
        /// 
        /// </summary>
        protected internal InitialStateDataType _isdt;

        // @end {fields} -------------------------------------------------------


        // @begin {initialization} ---------------------------------------------

        /// <summary> Initialization. <p/>
        /// 
        /// Uses init(String, int, String, int, int).
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void init(String aObjName, int aObjNum, OhStreamReader aSr)
        {
            int numOfUpIn;
            int numOfLateralIn;

            try
            {
                aSr.skipWhites();
                numOfUpIn = Int32.Parse(aSr.readWord());
                aSr.skipWhites();
                numOfLateralIn = Int32.Parse(aSr.readWord());
            }
            catch (IOException e)
            {
                throw new OhError(oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError(oPrintString("\t"));
            }

            init(aObjName, aObjNum, null, numOfUpIn, numOfLateralIn);
        }

        /// <summary> Initialization. <p/>
        /// 
        /// Uses OhElement.init(String, int, int, String).
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aDataStr">a <var>String</var> value
        /// </param>
        /// <param name="aNumOfUpIn">an <var>int</var> value
        /// </param>
        /// <param name="aNumOfLateralIn">an <var>int</var> value
        /// </param>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public virtual void init(String aObjName, int aObjNum, String aDataStr, int aNumOfUpIn, int aNumOfLateralIn)
        {

            _numOfUpIn = aNumOfUpIn;
            _numOfLateralIn = aNumOfLateralIn;

            if (_numOfUpIn < 0)
                _numOfUpIn = 0;

            if (_numOfLateralIn < 0)
                _numOfLateralIn = 0;

            _rpUpIn = new OhReceiveDouble[_numOfUpIn];
            _rpLateralIn = new OhReceiveDouble[_numOfLateralIn];

            _cUpIn = new double[_numOfUpIn];
            _cLateralIn = new double[_numOfLateralIn];

            if (_numOfUpIn > 0)
            {
                for (int i = 0; i < _numOfUpIn; i++)
                {
                    _rpUpIn[i] = new OhReceiveDouble(); //; added by Satoh.
                    _rpUpIn[i].init("rpUpIn", i);
                }
            }

            if (_numOfLateralIn > 0)
            {
                for (int i = 0; i < _numOfLateralIn; i++)
                {
                    _rpLateralIn[i] = new OhReceiveDouble(); //; added by Satoh.
                    _rpLateralIn[i].init("rpLateralIn", i);
                }
            }

            _sp = new OhSendDouble(); //; added by Satoh.
            _sp.init("sp", 0);

            base.init(aObjName, aObjNum, 0, aDataStr);
        }

        // @end {initialization} -----------------------------------------------

        /*
        y 4. RXgN^`Bz*/
        // @begin {constructors} -----------------------------------------------

        /// <summary> Creates a new default <var>PowerLawKinematicWave</var> instance.
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public PowerLawKinematicWave()
            : base()
        {

            _cUpIn = null;
            _cLateralIn = null;
            _qCrnt = null;
            _qNext = null;
            _rpUpIn = null;
            _rpLateralIn = null;

            _className = "PowerLawKinematicWave";
        }

        /// <summary> Creates a new <var>SimpleDam</var> instance.
        /// 
        /// Uses init(String, int, String, int, int).
        /// 
        /// </summary>
        /// <param name="aObjName">a <var>String</var> value
        /// </param>
        /// <param name="aObjNum">an <var>int</var> value
        /// </param>
        /// <param name="aDataStr">a <var>String</var> value
        /// </param>
        /// <param name="aNumOfUpIn">an <var>int</var> value
        /// </param>
        /// <param name="aNumOfLateralIn">an <var>int</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public PowerLawKinematicWave(String aObjName, int aObjNum, String aDataStr, int aNumOfUpIn, int aNumOfLateralIn)
            : base()
        {

            _cUpIn = null;
            _cLateralIn = null;
            _qCrnt = null;
            _qNext = null;
            _rpUpIn = null;
            _rpLateralIn = null;

            _className = "PowerLawKinematicWave";

            init(aObjName, aObjNum, aDataStr, aNumOfUpIn, aNumOfLateralIn);
        }

        // @end {constructors} -------------------------------------------------

        // @begin {methods} ----------------------------------------------------

        /*
        y 5. M[qEM[qo^郁\bh registerReceivePorts(), registerSendPorts() `Bz*/
        /// <summary> <japanese>
        /// SM[q̓o^ <p/>
        /// 
        /// ̃\bh́A̗vffׂĂ̎M[q
        /// <a href="OhElement.html#register(Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
        /// ɂo^܂B<p/>
        /// 
        /// o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void registerReceivePorts()
        {
            //; (*1)
            for (int i = 0; i < _numOfUpIn; i++)
            {
                register((OhObject)_rpUpIn[i]);
            }

            for (int i = 0; i < _numOfLateralIn; i++)
            {
                register((OhObject)_rpLateralIn[i]);
            }
        }

        /// <summary> <japanese>
        /// hNX̑SM[q̓o^ <p/>
        /// 
        /// ̃\bh́A̗vffׂĂ̑M[q
        /// <a href="OhElement.html#register(Object)">OhElement.register(Object)</a>
        /// <a href="OhElement.html#register(Object, int)">OhElement.register(Object, int)</a>
        /// ɂo^܂B<p/>
        /// 
        /// o^ĂȂ[q͐ڑłȂ̂őMɗpł܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void registerSendPorts()
        {
            register((OhObject)_sp);
        }

        /*
        y 6. p^Eԗʂǂݍރ\bh setParameter(), setInitialState() `Bz*/
        /// <summary> <japanese>
        /// p^̐ݒ <p/>
        /// 
        /// ̓Xg[ aSr Ŏw肳t@Cf[^ǂ݂Ƃăp^ݒ肵܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void setParameter(OhStreamReader aSr)
        {
            //; (*2)
            try
            {
                aSr.skipWhites();
                _a = Double.Parse(aSr.readWord());
                _m = Double.Parse(aSr.readWord());
                _lengthOfStream = Double.Parse(aSr.readWord());
                _numOfSepStream = Int32.Parse(aSr.readWord());
                _dt = Int32.Parse(aSr.readWord());
                _theta = Double.Parse(aSr.readWord());
                _qLevel = Double.Parse(aSr.readWord());
                _epsilon = Double.Parse(aSr.readWord());
            }
            catch (IOException e)
            {
                throw new OhError(oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError(oPrintString("\t"));
            }

            _qCrnt = new double[_numOfSepStream + 1];
            _qNext = new double[_numOfSepStream + 1];

            try
            {
                if (_numOfUpIn > 0)
                {
                    //; 㗬[̗ꍇAf[^n̏d݂ǂݎB
                    for (int i = 0; i < _numOfUpIn; i++)
                    {
                        aSr.skipWhites();
                        _cUpIn[i] = Double.Parse(aSr.readWord());
                    }
                }

                if (_numOfLateralIn > 0)
                {
                    //; ̗ꍇAf[^n̏d݂ǂݎB
                    for (int i = 0; i < _numOfLateralIn; i++)
                    {
                        aSr.skipWhites();
                        _cLateralIn[i] = Double.Parse(aSr.readWord());
                    }
                }
            }
            catch (IOException e)
            {
                throw new OhError(oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError(oPrintString("\t"));
            }

            _dx = _lengthOfStream / _numOfSepStream;
        }

        /// <summary> <japanese>
        /// ԗʂ̐ݒ <p/>
        /// 
        /// ̓Xg[ aSr Ŏw肳t@Cf[^ǂ݂Ƃďԗʂݒ肵܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aSr">an <var>OhStreamReader</var> value
        /// </param>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override void setInitialState(OhStreamReader aSr)
        {
            //;(*3)
            /* set initial state */

            int year = 0;
            long tsec = 0L;
            double frac = 0.0;

            try
            {
                aSr.skipWhites();
                year = Int32.Parse(aSr.readWord());
                tsec = Int64.Parse(aSr.readWord());
                frac = Double.Parse(aSr.readWord());
            }
            catch (IOException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }

            if (year > 0)
            {
                _currentTime = new OhTimePoint(year, tsec, frac);
            }

            int dataType; // 0:ONLY_DOWN_STREAM_END 1:EVERY_SECTION

            try
            {
                aSr.skipWhites();
                dataType = Int32.Parse(aSr.readWord());
            }
            catch (IOException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }

            if (dataType == 0)
                _isdt = InitialStateDataType.ONLY_DOWN_STREAM_END;
            else
                _isdt = InitialStateDataType.EVERY_SECTION;

            try
            {
                if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END)
                {
                    aSr.skipWhites();
                    _qCrnt[_numOfSepStream] = Double.Parse(aSr.readWord());
                }
                else
                {
                    for (int i = 1; i <= _numOfSepStream; i++)
                    {
                        _qCrnt[_numOfSepStream] = Double.Parse(aSr.readWord());
                    }
                }
            }
            catch (IOException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }
            catch (FormatException e)
            {
                throw new OhError("Error in setInitialState().", oPrintString("\t"));
            }
        }

        /*
        y 7. o̓\bh initialOutput() `Bz*/
        public override bool initialOutput()
        {
            //; (*5)
            /* initial output */

            if (_numOfUpIn == 0)
            {
                _qCrnt[0] = 0.0;
                if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END)
                {
                    int i = 0;
                    for (i = 1; i < _numOfSepStream; i++)
                    {
                        _qCrnt[i] = _qCrnt[_numOfSepStream] * i / (double)_numOfSepStream;
                    }
                }
            }
            else
            {
                double qIn = 0.0;
                int j;
                for (j = 0; j < _numOfUpIn; j++)
                {
                    if (_rpUpIn[j].canYouGetData(_currentTime) == false)
                    {
                        return false;
                    }
                    qIn += _cUpIn[j] * _rpUpIn[j].getData(_currentTime);
                }

                if (_isdt == InitialStateDataType.ONLY_DOWN_STREAM_END)
                {
                    int i;
                    for (i = 0; i < _numOfSepStream; i++)
                    {
                        _qCrnt[i] = (_qCrnt[_numOfSepStream] * i + qIn * (_numOfSepStream - 1)) / (double)_numOfSepStream;
                    }
                }
            }

            _sp.CommentString = "OhTimePoint.Year OhTimePoint.Tsec(sec) discharge(m**3/sec)";

            _sp.sendData(_currentTime, _qCrnt[_numOfSepStream]);

            return true;
        }

        /*
        y 8. ŏIԗʂރ\bh saveTerminalState() `Bz*/
        /// <summary> <japanese>
        /// ŏIԂۑ郁\bh
        /// </japanese>
        /// 
        /// </summary>
        /// <param name="aSw">a <var>PrintWriter</var> value
        /// </param>
        public override void saveTerminalState(StreamWriter aSw)
        {
            //; (*4)
            /* output terminal state into a file */

            aSw.WriteLine(_currentTime + " # _currentTime");
            aSw.WriteLine("1 # EVERY_SECTION");

            for (int i = 1; i <= _numOfSepStream; i++)
            {
                aSw.WriteLine(_qCrnt[i] + " # Section " + i);
            }
        }

        /*
        y 9. vZԊԊuvZ郁\bh calculateTimeStep() `Bz*/
        /// <summary> <japanese>
        /// vZԊԊu͌Œ肵lƂ܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> a <var>long</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override OhTime calculateTimeStep()
        {
            return new OhTime(_dt); // "_dt" is defined in pmt file.
        }

        /*
        y 10. vZ邱Ƃł邩𔻒f郁\bh canYouCalculate() `Bz*/
        /// <summary> <japanese>
        /// vZ\̔f <p/>
        /// 
        /// <a href="OhElement.html">OhElement</a> NX
        /// \bh <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a>,
        /// <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a> 
        /// `鎞Ԃ̊Ԃ̃f[^擾łΌvZ\Ɣf܂B
        /// <a href="OhElement.html#getNecessaryTimeFrom()">OhElement.getNecessaryTimeFrom()</a> \bh
        /// _currentTime ԂA
        /// <a href="OhElement.html#getNecessaryTimeTo()">OhElement.getNecessaryTimeTo()</a> \bh
        /// _currentTime + _timeStep ̒lԂ܂B<p/>
        /// 
        /// f[^擾s\łƕԎĂM[qɑ΂ČvZvo܂B
        /// ̓Iɂ́AM[q calReq() bZ[W𑗂܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override bool canYouCalculate()
        {
            return canYouCalculate0();
        }

        /*
        y 11. 1Xebv̌vZ郁\bh calculate() `Bz*/
        /// <summary> The solution of power law kimematic wave model by Keith Beven's method.
        /// 
        /// <japanese>
        /// <p/>
        /// Beven @ɂw@ kinematic wave f̉@
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override bool calculate()
        {
            //;(*6)
            double dta;
            dta = _timeStep.toDouble(); //; "_timeStep" equals "_dt";

            /* Lateral inflow */

            double rLin = 0.0;
            int i;

            OhTimePoint crnTime = new OhTimePoint(); //; For work.
            crnTime.copy(_currentTime);
            crnTime.add(_timeStep);
            OhTimePoint nextTime = crnTime;

            for (i = 0; i < _numOfLateralIn; i++)
            {
                rLin += _cLateralIn[i] * _rpLateralIn[i].getMeanData(_currentTime, nextTime) / _lengthOfStream;
            }

            /* upstream end */

            _qNext[0] = 0.0;

            for (i = 0; i < _numOfUpIn; i++)
            {
                _qNext[0] += _cUpIn[i] * _rpUpIn[i].getData(nextTime);
            }

            double qest;
            double cold; //; `dx
            double cnew; //; XVꂽ`dx

            double m1 = (_m - 1.0) / _m;

            for (i = 1; i <= _numOfSepStream; i++)
            {
                double qbase = (_qCrnt[i - 1] + _qCrnt[i] + _qNext[i - 1]) / 3.0;
                qest = _qNext[i - 1] + rLin * _dx;
                if (_qCrnt[i] > qest)
                {
                    qest = _qCrnt[i];
                }
                double qmean = 0.5 * (_qCrnt[i - 1] + _qCrnt[i]);
                cold = _a * _m * Math.Pow(qmean / _a, m1);


                while (true)
                {
                    double qmean1 = 0.5 * (_qNext[i - 1] + qest);
                    cnew = _a * _m * Math.Pow(qmean1 / _a, m1);

                    _qNext[i] = (_qCrnt[i] + _theta * cnew * dta * (_qNext[i - 1] / _dx + rLin) - (1.0 - _theta) * cold * dta * ((_qCrnt[i] - _qCrnt[i - 1]) / _dx - rLin)) / (1.0 + _theta * cnew * dta / _dx);


                    if (_qNext[i] < 0.0)
                        _qNext[i] = 0.0;


                    if (Math.Abs(_qNext[i] - qest) < (_epsilon * (_qLevel + qbase)))
                        break;

                    qest = _qNext[i];
                }

                _qNext[i] = qest;
            }

            /* output */

            OhTimePoint crntTime = new OhTimePoint(); //; For work.
            crntTime.copy(_currentTime);
            crntTime.add(_timeStep);
            OhTimePoint nextT = crntTime;

            _sp.sendData(nextT, _qNext[_numOfSepStream]);

            /* update state variable */

            for (i = 0; i <= _numOfSepStream; i++)
            {
                _qCrnt[i] = _qNext[i];
            }

            return true;
        }

        /*
        y 12. ̈ǍvZEo͍Ƃ郁\bh work() `Bz*/
        /// <summary> <japanese>
        /// <p/>
        /// _timeStep 𓾂Ă calculate() s work0() ̗p܂B
        /// </japanese>
        /// 
        /// </summary>
        /// <returns> a <var>boolean</var> value
        /// </returns>
        /// <exception cref="OhCannotGetDataError">if an error occurs
        /// </exception>
        /// <exception cref="OhError">if an error occurs
        /// </exception>
        public override bool work()
        {
            return work0();
        }
    } // PowerLawKinematicWave
}