﻿/// <(#)>  Main.java </(#)>
/// <summary> 
/// Created: 2004/01/30
/// 
/// </summary>
/// <author>  shiiba
/// </author>
/// <version>  $Revision$
/// 
/// </version>
using System;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using OhymosNet;

namespace OhymosNet
{

    /// <summary> Main class of OHyMoS.NET.<p>
    /// </p></summary>
    public class OhymosNetMain2
    {
        internal static String _version = "1.0";

        [STAThread]
        private static void Main(String[] args)
        {
            Console.WriteLine("OHyMoS.NETの計算準備を開始します");
            Console.WriteLine();
            Console.WriteLine("構造定義ファイルの名前を入力してください");
            Console.WriteLine("OHyMoS.NETに含まれるクラスを表示させたい場合は");
            Console.WriteLine("・要素モデル\t-e");
            Console.WriteLine("・端子モデル\t-p");
            Console.WriteLine("・抽象クラス\t-a");
            Console.WriteLine("・全クラス\t-c");
            Console.WriteLine("を入力してください");
            Console.Write(">");
            
            string str = Console.ReadLine();
            Console.WriteLine();
            
            OhOption op = OhOption.Default;

            bool a_flag = false;
            bool p_flag = false;
            bool e_flag = false;
            bool c_flag = false;
            //String assemblyName = "";
            char c;
            int i = 0;
            if( str[0].Equals('-'))
            {
                //if ((Object)args[i] == (Object)'--')
                  //  break;
                c = str[1];
                switch (c)
                {

                    case 'c':
                        c_flag = true;
                        break;

                    case 'a':
                        a_flag = true;
                        break;

                    case 'e':
                        e_flag = true;
                        break;

                    case 'p':
                        p_flag = true;
                        break;

                    case 'w':
                        op.UseWinP = true;
                        break;

                    default:
                        Console.Error.WriteLine("Unknown option character " + c);
                        usage();
                        Environment.Exit(1);
                        break;

                }
                i++;
            }
            //int nstart = i;

            OhAssembly ohAsm = OhAssembly.Default;

            if (c_flag)
            {
                Console.Out.WriteLine("List of all classes contained in OHyMoS.NET");
                Console.Out.WriteLine("");
                ohAsm.listAllClasses();
            }
            if (a_flag)
            {
                Console.Out.WriteLine("List of abstract classes contained in OHyMoS.NET");
                Console.Out.WriteLine("");
                ohAsm.listAbstractClasses();
            }
            if (e_flag)
            {
                Console.Out.WriteLine("List of element classes contained in OHyMoS.NET");
                Console.Out.WriteLine("");
                ohAsm.listElements();
            }
            if (p_flag)
            {
                Console.Out.WriteLine("List of port classes contained in OHyMoS.NET");
                Console.Out.WriteLine("");
                ohAsm.listPorts();
            }
            if (c_flag || a_flag || e_flag || p_flag)
            {
                Console.WriteLine("表示を終了します。Enterキーを押してください");
                Console.ReadLine();
                Environment.Exit(0);
            }

            Console.WriteLine("パラメタファイル、初期状態量ファイルのベース名を入力してください");
            Console.Write(">");
            string fileBasepart = Console.ReadLine();
            Console.WriteLine();

            Console.WriteLine("OHyMoS.NET " + "version " + _version);

            try
            {

                Console.Out.WriteLine("いまから計算します。Return キーを押してください。");
                StreamReader isr = new StreamReader(Console.OpenStandardInput(), Encoding.Default);
                StreamReader br = new StreamReader(isr.BaseStream, isr.CurrentEncoding);
                String ignore = br.ReadLine();
                Console.Out.WriteLine("Wait. Now, I am working.");

                OhScfTotalSystem totalSystem = new OhScfTotalSystem(str);

                //String fileBasepart = args[nstart + 1];

                totalSystem.setOhymosNetOption();

                totalSystem.construct();

                totalSystem.doSwitch();

                if (totalSystem.areYouReady() != true)
                {
                    Console.Out.Write(NEWLINE);
                    Console.Out.Write("# Failure : areYouReady().");
                    Console.Out.Write(NEWLINE);
                    Environment.Exit(1);
                }

                String pmtFileName = fileBasepart + ".pmt";
                totalSystem.readParameterFile(pmtFileName);

                String istFileName = fileBasepart + ".ist";
                totalSystem.readInitialStateFile(istFileName);

                String isdFileName = fileBasepart + ".isd";
                totalSystem.readSendPortFile(isdFileName);

                //;   totalSystem.doShareInfo();  //; 必要？

                totalSystem.initialWork();

                //;   System.out.println("Finish initialWork().");

                totalSystem.work();

                //      System.out.println("Finish simulation!");  //; 確認用標準出力


                String tstFileName = fileBasepart + ".tst";
                totalSystem.writeTerminalState(tstFileName);

                String tsdFileName = fileBasepart + ".tsd";
                totalSystem.writeSendPortStock(tsdFileName);

                //      out.println("Finish saving!");  //; 確認用標準出力
            }
            //  catch (InterruptedException ignored) {
            //  }
            catch (OhCannotGetDataError e)
            {
                OhError.WriteStackTrace(e, Console.Error);
                Console.WriteLine();
                Console.Out.WriteLine(e.Message);
            }
            catch (OhError e)
            {
                OhError.WriteStackTrace(e, Console.Error);
                Console.WriteLine();
                OhInterface.pause(e.Message);
            }
            catch (IOException e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }
            catch (ArgumentException e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }
            catch (InvalidCastException e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }
            catch (NullReferenceException e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }
            catch (Exception e)
            {
                OhError.WriteStackTrace(e, Console.Error);
            }

            //; OHyMoSJ の終了処理 ------------------------------------------------

            //if (useWinP)
            //{
            //    //; System.exit を呼ぶと、Swing 関係の環境も終了してくれます。
            //    Environment.Exit(0);
            //}
        }

        private static string NEWLINE
        {
            get { return Environment.NewLine; }
        }

        /// <summary> Shows the usage of the Main class of ohymosj.jar
        /// 
        /// </summary>
        private static void usage()
        {
            Console.Out.WriteLine("usage: ./OhymosNet [-a -e -p -c] \n" + "      or\n" + "       ./OhymosNet ScfFileName fileBasePart\n" + "sample:\n" + "   ./OhymosNet -e\n" + "   ./OhymosNet foo.scf arakura\n");
        }
    } // Main
}