//
//	C++/CLIɂSimpleDamf\NX̃\[Xt@CłB
//

#include "stdafx.h"
#include "SimpleDamC.h"

namespace OhymosNet
{
	
	using namespace System;
	using namespace System::IO;
	using namespace OhymosNet;


	// @begin {initialization} -----------------------------------------------
		
	void SimpleDamC::init(String^ aObjName, int aObjNum, String^ aDataStr, int aNrp)
	{

		_numberOfRp = aNrp;

		if (_numberOfRp < 1){
			_numberOfRp = 1;
		}

		_rP = gcnew array<OhReceiveDouble^>(_numberOfRp);
		_cRp = new double[_numberOfRp];

		for (int i = 0; i < _numberOfRp; i++)
		{
			_rP[i] = gcnew OhReceiveDouble();
			_rP[i]->init("rp", i);
		}

		OhElement::init(aObjName, aObjNum, 0, aDataStr);
	}


	void SimpleDamC::init(String^ aObjName, int aObjNum, OhStreamReader^ aSr)
	{
		
		int nrp;
		
		try
		{
			nrp = Int32::Parse(aSr->readWord());
		}
		catch (IOException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
		catch (FormatException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
		
		init(aObjName, aObjNum, nullptr, nrp);
	}

	// @end {initialization} -----------------------------------------------
		
	
	// @begin {constructors} -----------------------------------------------
		
	SimpleDamC::SimpleDamC()
	{
		
		_rP = nullptr;
		_sP0 = gcnew OhSendDouble("sp", 0);
		_sP1 = gcnew OhSendDouble("sp", 1);
		_numberOfRp = 0;
		_cRp = nullptr;
		
		_className = "SimpleDam";
	}
		
		
	SimpleDamC::SimpleDamC(String^ aObjName, int aObjNum, String^ aDataStr, int aNrp)
	{
		
		_rP = nullptr;
		_sP0 = gcnew OhSendDouble("sp", 0);
		_sP1 = gcnew OhSendDouble("sp", 1);
		_cRp = nullptr;
		
		_className = "SimpleDam";
		
		init(aObjName, aObjNum, aDataStr, aNrp);
	}
		
	// @end {constructors} -------------------------------------------------
	

	// @begin {methods} ----------------------------------------------------
	
	// 1. Methods about receive ports and send ports.
	
	void SimpleDamC::registerReceivePorts()
	{
		
		for (int i = 0; i < _numberOfRp; i++)
		{
			registerForCLI(_rP[i]);
		}
	}

	
	void SimpleDamC::registerSendPorts()
	{
		
		registerForCLI(_sP0); 
		registerForCLI(_sP1);
	}
	
	
	// 2. Parameter setting and initialization of states.

	void SimpleDamC::setParameter(OhStreamReader^ aSw)
	{
			
		double dt = 0.0;
	
		try
		{
			_qPass = Double::Parse(aSw->readWord());
			_ratio = Double::Parse(aSw->readWord());
			dt = Double::Parse(aSw->readWord());
			_dt = gcnew OhTime(dt);
		}
		catch (IOException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
		catch (FormatException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
		
		if (_qPass < 0.0)
		{
			throw gcnew OhError("_qPass should be non-negative but is " + _qPass);
		}
		
		if (_ratio < 0.0 || _ratio > 1.0)
		{
			throw gcnew OhError("_ratio should be in [0.0, 1.0] but it is " + _ratio);
		}
		
		if (_dt->toDouble() < OhTime::NEGLIGIBLY_SMALL_TIME)
		{
			throw gcnew OhError("_dt should be greater than or equal to " + OhTime::NEGLIGIBLY_SMALL_TIME + " but it is " + _dt);
		}
		
		if (_numberOfRp > 0)
		{
			for (int i = 0; i < _numberOfRp; i++)
			{
				try
				{
					_cRp[i] = Double::Parse(aSw->readWord());
				}
				catch (IOException^ e)
				{
					throw gcnew OhError(oPrintString("\t"));
				}
			}
		}
	}
	
	
	void SimpleDamC::saveParameter(StreamWriter^ aSw)
	{
		
		aSw->Write(_qPass + " " + _ratio + " " + _dt + NEWLINE);
		
		for (int i = 0; i < _numberOfRp; i++)
		{
			aSw->Write(_cRp[i] + " ");
		}
		aSw->Write(NEWLINE);
	}
	

	void SimpleDamC::setInitialState(OhStreamReader^ aSr)
	{
		
		int year = 0;
		long tsec = 0L;
		double frac = 0.0;
		
		try
		{
			year = Int32::Parse(aSr->readWord());
			tsec = Int64::Parse(aSr->readWord());
			frac = Double::Parse(aSr->readWord());
		}
		catch (IOException^ e)
		{
			throw gcnew OhError("Error in setInitialState()." + oPrintString("\t"));
		}
		catch (FormatException^ e)
		{
			throw gcnew OhError("Error in setInitialState().", oPrintString("\t"));
		}
       			
		if (year > 0)
		{
			_currentTime = gcnew OhTimePoint(year, tsec, frac);
		}
		
		try
		{
			_storageCrnt = Double::Parse(aSr->readWord());
		}
		catch (IOException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
		catch (FormatException^ e)
		{
			throw gcnew OhError(oPrintString("\t"));
		}
	}
	
	
	void SimpleDamC::saveTerminalState(StreamWriter^ aSw)
	{
		
		aSw->Write(_currentTime + " # _currentTime");
		aSw->Write(NEWLINE);
		aSw->Write(_storageCrnt + " # _storageCrnt (m^3)");
		aSw->Write(NEWLINE);
	}
	
	
	// 3. Calculation.
	
	bool SimpleDamC::initialOutput()
	{
		
		_qInCrnt = 0.0;
		
		if (_numberOfRp > 0)
		{
			int j = 0;
			
			for (j = 0; j < _numberOfRp; j++)
			{
				if (_rP[j]->canYouGetData(_currentTime) == false)
				{
					return false;
				}
				
				_qInCrnt += _cRp[j] * (_rP[j]->getData(_currentTime));
			}
		}
		
		if (_qInCrnt <= _qPass)
		{
			_qCrnt = _qInCrnt;
		}
		else
		{
			_qCrnt = _qPass + _ratio * (_qInCrnt - _qPass);
		}
		
		_sP0->CommentString = "OhTimePoint.Year OhTimePoint.Tsec(sec) " + "OhTimePoint.Frac(sec) discharge(m**3/sec)";
		_sP0->sendData(_currentTime, _qCrnt);
		
		_sP1->CommentString = "OhTimePoint.Year OhTimePoint.Tsec(sec) " + "OhTimePoint.Frac(sec) storage(m**3)";
		_sP1->sendData(_currentTime, _storageCrnt);
		
		return true;
	}
	
	
	OhTime^ SimpleDamC::calculateTimeStep()
	{
		return _dt;
	}
	

	bool SimpleDamC::canYouCalculate()
	{
		return canYouCalculate0();
	}
	

	bool SimpleDamC::work()
	{
		return work0();
	}
	

	bool SimpleDamC::calculate()
	{
	
		double dta;
		dta = _timeStep->toDouble();
		
		_qInNext = 0.0;
		
		OhTimePoint^ nextTime = OhTimePoint::add(_currentTime, _timeStep);
		
		if (_numberOfRp > 0)
		{
			int j = 0;
			for (j = 0; j < _numberOfRp; j++)
			{
				if (_rP[j]->canYouGetData(nextTime) == false)
				{
					return false;
				}
				_qInNext += _cRp[j] * (_rP[j]->getData(nextTime));
			}
		}
		
		if (_qInNext <= _qPass)
		{
			_qNext = _qInNext;
		}
		else
		{
			_qNext = _qPass + _ratio * (_qInNext - _qPass);
		}
		
		_storageNext = _storageCrnt + dta * (_qInCrnt + _qInNext) * 0.5 - dta * (_qCrnt + _qNext) * 0.5;
		
		_sP0->sendData(nextTime, _qNext);
		_sP1->sendData(nextTime, _storageNext);

		_storageCrnt = _storageNext;
		_qCrnt = _qNext;
		_qInCrnt = _qInNext;
		
		return true;
	}
	
	
	String^ SimpleDamC::basicInfo()
	{
		return "Simple dam model";
	}

}
